@@ -109,6 +109,7 @@ static void help(const char *prog)
 	printf("		- Use -f \"<format>\" so your shell doesn't break it up.\n");
 	printf("  -i <interval>	- Time between updates, in seconds\n");
 	printf("  -l <logfile>	- Log file name, or - for stdout\n");
+	printf("  -p <pidbase>  - Base name for PID file (defaults to \"upslog\")\n");
 	printf("  -s <ups>	- Monitor UPS <ups> - <upsname>@<host>[:<port>]\n");
 	printf("        	- Example: -s myups@server\n");
 	printf("  -u <user>	- Switch to <user> if started as root\n");
@@ -369,8 +370,10 @@ int main(int argc, char **argv)
 {
 	int	interval = 30, i;
 	char	*prog = NULL;
+	time_t	now, nextpoll = 0;
 	const	char	*user = NULL;
 	struct	passwd	*new_uid = NULL;
+	const   char    *pidfilebase = "upslog";
 
 	logformat = DEFAULT_LOGFORMAT;
 	user = RUN_AS_USER;
@@ -379,7 +382,7 @@ int main(int argc, char **argv)
 
 	prog = argv[0];
 
-	while ((i = getopt(argc, argv, "+hs:l:i:f:u:V")) != -1) {
+	 while ((i = getopt(argc, argv, "+hs:l:i:f:u:Vp:")) != -1) {
 		switch(i) {
 			case 'h':
 				help(prog);
@@ -407,6 +410,10 @@ int main(int argc, char **argv)
 
 			case 'V':
 				exit(EXIT_SUCCESS);
+
+			case 'p':
+				pidfilebase = optarg;
+				break;
 		}
 	}
 
@@ -476,13 +483,24 @@ int main(int argc, char **argv)
 
 	setup_signals();
 
-	writepid("upslog");
+	writepid(pidfilebase);
 
 	become_user(new_uid);
 
 	compile_format();
 
 	while (exit_flag == 0) {
+		time(&now);
+
+		if (nextpoll > now) {
+			/* there is still time left, so sleep it off */
+			sleep(difftime(nextpoll, now));
+			nextpoll += interval;
+		} else {
+			/* we spent more time in polling than the interval allows */
+			nextpoll = now + interval;
+		}
+
 		if (reopen_flag) {
 			upslogx(LOG_INFO, "Signal %d: reopening log file", 
 				reopen_flag);
@@ -501,8 +519,6 @@ int main(int argc, char **argv)
 		if (interval > 30) {
 			upscli_disconnect(&ups);
 		}
-
-		sleep(interval);
 	}
 
 	upslogx(LOG_INFO, "Signal %d: exiting", exit_flag);
