@@ -284,6 +284,24 @@ static void vupslog(int priority, const char *fmt, va_list va, int use_strerror)
 	if (use_strerror)
 		snprintfcat(buf, sizeof(buf), ": %s", strerror(errno));
 
+	if (nut_debug_level > 0) {
+		static struct timeval	start = { 0 };
+		struct timeval		now;
+	
+		gettimeofday(&now, NULL);
+	
+		if (start.tv_sec == 0) {
+			start = now;
+		}
+	
+		if (start.tv_usec > now.tv_usec) {
+			now.tv_usec += 1000000;
+			now.tv_sec -= 1;
+		}
+	
+		fprintf(stderr, "%4.0f.%06ld\t", difftime(now.tv_sec, start.tv_sec), (long)(now.tv_usec - start.tv_usec));
+	}
+
 	if (xbit_test(upslog_flags, UPSLOG_STDERR))
 		fprintf(stderr, "%s\n", buf);
 	if (xbit_test(upslog_flags, UPSLOG_SYSLOG))
