@@ -1,6 +1,8 @@
 /* state.c - Network UPS Tools common state management functions
 
-   Copyright (C) 2003  Russell Kroll <rkroll@exploits.org>
+   Copyright (C)
+	2003	Russell Kroll <rkroll@exploits.org>
+	2008	Arjen de Korte <adkorte-guest@alioth.debian.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,7 +35,7 @@ static void val_escape(struct st_tree_t *node)
 	char	etmp[ST_MAX_VALUE_LEN];
 
 	/* escape any tricky stuff like \ and " */
-	pconf_encode(node->val, etmp, sizeof(etmp));
+	pconf_encode(node->raw, etmp, sizeof(etmp));
 
 	/* if nothing was escaped, we don't need to do anything else */
 	if (!strcmp(node->raw, etmp)) {
@@ -41,14 +43,8 @@ static void val_escape(struct st_tree_t *node)
 		return;
 	}
 
-	/* first time: set a good starting place */
-	if (node->safesize == 0) {
-		node->safesize = strlen(etmp) + 1;
-		node->safe = xmalloc(node->safesize);
-	}
-
 	/* if the escaped value grew, deal with it */
-	if (strlen(etmp) > (node->safesize - 1)) {
+	if (node->safesize < (strlen(etmp) + 1)) {
 		node->safesize = strlen(etmp) + 1;
 		node->safe = xrealloc(node->safe, node->safesize);
 	}
@@ -59,8 +55,9 @@ static void val_escape(struct st_tree_t *node)
 
 static void st_tree_enum_free(struct enum_t *list)
 {
-	if (!list)
+	if (!list) {
 		return;
+	}
 
 	st_tree_enum_free(list->next);
 
@@ -87,137 +84,142 @@ static void st_tree_node_free(struct st_tree_t *node)
 /* add a subtree to another subtree */
 static void st_tree_node_add(struct st_tree_t **nptr, struct st_tree_t *sptr)
 {
-	struct	st_tree_t 	*node = *nptr;
-
-	if (!sptr)
-		return;
-
-	if (!node) {
-		*nptr = sptr;
+	if (!sptr) {
 		return;
 	}
 
-	if (strcmp(sptr->var, node->var) < 0)
-		st_tree_node_add(&node->left, sptr);
-	else
-		st_tree_node_add(&node->right, sptr);
+	while (*nptr) {
+
+		struct st_tree_t	*node = *nptr;
+
+		if (strcasecmp(node->var, sptr->var) > 0) {
+			nptr = &node->left;
+			continue;
+		}
+
+		if (strcasecmp(node->var, sptr->var) < 0) {
+			nptr = &node->right;
+			continue;
+		}
+
+		upsdebugx(1, "%s: duplicate value (shouldn't happen)", __func__);
+		return;
+	}
+
+	*nptr = sptr;
 }
 
 /* remove a variable from a tree */
 int state_delinfo(struct st_tree_t **nptr, const char *var)
 {
-	struct	st_tree_t	*node = *nptr;
+	while (*nptr) {
 
-	if (!node)
-		return 0;	/* variable not found! */
+		struct st_tree_t	*node = *nptr;
 
-	if (strcasecmp(var, node->var) < 0)
-		return state_delinfo(&node->left, var);
+		if (strcasecmp(node->var, var) > 0) {
+			nptr = &node->left;
+			continue;
+		}
 
-	if (strcasecmp(var, node->var) > 0)
-		return state_delinfo(&node->right, var);
+		if (strcasecmp(node->var, var) < 0) {
+			nptr = &node->right;
+			continue;
+		}
 
-	/* apparently, we've found it! */
+		/* whatever is on the left, hang it off current right */
+		st_tree_node_add(&node->right, node->left);
 
-	/* whatever is on the left, hang it off current right */
-	st_tree_node_add(&node->right, node->left);
+		/* now point the parent at the old right child */
+		*nptr = node->right;
 
-	/* now point the parent at the old right child */
-	*nptr = node->right;
+		st_tree_node_free(node);
 
-	st_tree_node_free(node);
+		return 1;
+	}
 
-	return 1;
+	return 0;	/* not found */
 }	
 
 /* interface */
 
 int state_setinfo(struct st_tree_t **nptr, const char *var, const char *val)
 {
-	struct	st_tree_t	*node = *nptr;
+	while (*nptr) {
 
-	if (!node) {
-		node = xmalloc(sizeof(struct st_tree_t));
+		struct st_tree_t	*node = *nptr;
 
-		node->var = xstrdup(var);
+		if (strcasecmp(node->var, var) > 0) {
+			nptr = &node->left;
+			continue;
+		}
 
-		node->rawsize = strlen(val) + 1;
-		node->raw = xmalloc(node->rawsize);
+		if (strcasecmp(node->var, var) < 0) {
+			nptr = &node->right;
+			continue;
+		}
+
+		/* updating an existing entry */
+		if (!strcasecmp(node->raw, val)) {
+			return 0;	/* no change */
+		}
+
+		/* changes should be ignored */
+		if (node->flags & ST_FLAG_IMMUTABLE) {
+			return 0;	/* no change */
+		}
+
+		/* expand the buffer if the value grows */
+		if (node->rawsize < (strlen(val) + 1)) {
+			node->rawsize = strlen(val) + 1;
+			node->raw = xrealloc(node->raw, node->rawsize);
+		}
+
+		/* store the literal value for later comparisons */
 		snprintf(node->raw, node->rawsize, "%s", val);
 
-		/* this is usually sufficient if nothing gets escaped */
-		node->val = node->raw;
-		node->safesize = 0;
-		node->safe = NULL;
-
-		/* but see if it needs escaping anyway */
 		val_escape(node);
 
-		/* these are updated by other functions */
-		node->flags = 0;
-		node->aux = 0;
-		node->enum_list = NULL;
-
-		node->left = NULL;
-		node->right = NULL;
-
-		/* now we're done with creating a new node, add it to the tree */
-		*nptr = node;
-
-		return 1;	/* added */
+		return 1;	/* changed */
 	}
 
-	if (strcasecmp(var, node->var) < 0)
-		return state_setinfo(&node->left, var, val);
+	*nptr = xcalloc(1, sizeof(**nptr));
 
-	if (strcasecmp(var, node->var) > 0)
-		return state_setinfo(&node->right, var, val);
+	(*nptr)->var = xstrdup(var);
+	(*nptr)->raw = xstrdup(val);
+	(*nptr)->rawsize = strlen(val) + 1;
 
-	/* var must equal node->var - updating an existing entry */
+	val_escape(*nptr);
 
-	if (!strcasecmp(node->raw, val))
-		return 0;		/* no change */
-
-	/* expand the buffer if the value grows */
-	if (strlen(val) > (node->rawsize - 1)) {
-		node->rawsize = strlen(val) + 1;
-		node->raw = xrealloc(node->raw, node->rawsize);
-		node->val = node->raw;
-	}
-
-	/* store the literal value for later comparisons */
-	snprintf(node->raw, node->rawsize, "%s", val);
-
-	val_escape(node);
-
-	return 1;	/* changed */
+	return 1;	/* added */
 }
 
-static void st_tree_enum_add(struct enum_t **list, const char *enc)
+static int st_tree_enum_add(struct enum_t **list, const char *enc)
 {
-	struct	enum_t	*item = *list;
+	struct enum_t	*item;
 
-	if (!item) {	/* end of list reached */
-		item = xmalloc(sizeof(struct enum_t));
-		item->val = xstrdup(enc);
-		item->next = NULL;
+	while (*list) {
 
-		/* now we're done creating it, add it to the list */
-		*list = item;
-
-		return;
+		if (strcmp((*list)->val, enc)) {
+			list = &(*list)->next;
+			continue;
+		}
+		
+		return 0;	/* duplicate */
 	}
 
-	/* don't add duplicates - silently ignore them */
-	if (!strcmp(item->val, enc))
-		return;
+	item = xcalloc(1, sizeof(*item));
+	item->val = xstrdup(enc);
+	item->next = *list;
 
-	st_tree_enum_add(&item->next, enc);
+	/* now we're done creating it, add it to the list */
+	*list = item;
+
+	return 1;	/* added */
 }
 
 int state_addenum(struct st_tree_t *root, const char *var, const char *val)
 {
-	struct	st_tree_t	*sttmp;
+	struct st_tree_t	*sttmp;
 	char	enc[ST_MAX_VALUE_LEN];
 
 	/* find the tree node for var */
@@ -232,14 +234,12 @@ int state_addenum(struct st_tree_t *root, const char *var, const char *val)
 	/* smooth over any oddities in the enum value */
 	pconf_encode(val, enc, sizeof(enc));
 
-	st_tree_enum_add(&sttmp->enum_list, enc);
-
-	return 1;
+	return st_tree_enum_add(&sttmp->enum_list, enc);
 }
 
 int state_setaux(struct st_tree_t *root, const char *var, const char *auxs)
 {
-	struct	st_tree_t	*sttmp;
+	struct st_tree_t	*sttmp;
 	int	aux;
 
 	/* find the tree node for var */
@@ -254,8 +254,9 @@ int state_setaux(struct st_tree_t *root, const char *var, const char *auxs)
 	aux = strtol(auxs, (char **) NULL, 10);
 
 	/* silently ignore matches */
-	if (sttmp->aux == aux)
+	if (sttmp->aux == aux) {
 		return 0;
+	}
 
 	sttmp->aux = aux;
 
@@ -264,61 +265,64 @@ int state_setaux(struct st_tree_t *root, const char *var, const char *auxs)
 
 const char *state_getinfo(struct st_tree_t *root, const char *var)
 {
-	struct	st_tree_t	*sttmp;
+	struct st_tree_t	*sttmp;
 
 	/* find the tree node for var */
 	sttmp = state_tree_find(root, var);
 
-	if (!sttmp)
+	if (!sttmp) {
 		return NULL;
+	}
 
 	return sttmp->val;
 }
 
 int state_getflags(struct st_tree_t *root, const char *var)
 {
-	struct	st_tree_t	*sttmp;
+	struct st_tree_t	*sttmp;
 
 	/* find the tree node for var */
 	sttmp = state_tree_find(root, var);
 
-	if (!sttmp)
+	if (!sttmp) {
 		return -1;
+	}
 
 	return sttmp->flags;
 }
 
 int state_getaux(struct st_tree_t *root, const char *var)
 {
-	struct	st_tree_t	*sttmp;
+	struct st_tree_t	*sttmp;
 
 	/* find the tree node for var */
 	sttmp = state_tree_find(root, var);
 
-	if (!sttmp)
+	if (!sttmp) {
 		return -1;
+	}
 
 	return sttmp->aux;
 }
 
 const struct enum_t *state_getenumlist(struct st_tree_t *root, const char *var)
 {
-	struct	st_tree_t	*sttmp;
+	struct st_tree_t	*sttmp;
 
 	/* find the tree node for var */
 	sttmp = state_tree_find(root, var);
 
-	if (!sttmp)
+	if (!sttmp) {
 		return NULL;
+	}
 
 	return sttmp->enum_list;
 }
 
-void state_setflags(struct st_tree_t *root, const char *var, int numflags,
-	char **flag)
+void state_setflags(struct st_tree_t *root, const char *var, int numflags, char **flag)
 {	
 	int	i;
-	struct	st_tree_t	*sttmp;
+	struct st_tree_t	*sttmp;
 
 	/* find the tree node for var */
 	sttmp = state_tree_find(root, var);
@@ -373,16 +377,16 @@ int state_addcmd(struct cmdlist_t **list, const char *cmd)
 	/* now we're done creating it, insert it in the list */
 	*list = item;
 
-	return 1;
+	return 1;	/* added */
 }
 
 void state_infofree(struct st_tree_t *node)
 {
-	if (!node)
+	if (!node) {
 		return;
+	}
 
 	state_infofree(node->left);
-
 	state_infofree(node->right);
 
 	st_tree_node_free(node);
@@ -390,8 +394,9 @@ void state_infofree(struct st_tree_t *node)
 
 void state_cmdfree(struct cmdlist_t *list)
 {
-	if (!list)
+	if (!list) {
 		return;
+	}
 
 	state_cmdfree(list->next);
 
@@ -425,53 +430,63 @@ int state_delcmd(struct cmdlist_t **list, const char *cmd)
 		return 1;	/* deleted */
 	}
 
-	return 0;
+	return 0;	/* not found */
 }
 
 static int st_tree_del_enum(struct enum_t **list, const char *val)
 {
-	struct	enum_t	*item = *list;
+	while (*list) {
 
-	if (!item)		/* not found */
-		return 0;
+		struct enum_t	*item = *list;
 
-	/* if this is not the right value, go on to the next */
-	if (strcmp(item->val, val))
-		return st_tree_del_enum(&item->next, val);
+		/* if this is not the right value, go on to the next */
+		if (strcasecmp(item->val, val)) {
+			list = &item->next;
+			continue;
+		}
 
-	/* we found it! */
+		/* we found it! */
+		*list = item->next;
 
-	*list = item->next;
+		free(item->val);
+		free(item);
 
-	free(item->val);
-	free(item);
+		return 1;	/* deleted */
+	}
 
-	return 1;	/* deleted */
+	return 0;	/* not found */
 }
 
 int state_delenum(struct st_tree_t *root, const char *var, const char *val)
 {
-	struct	st_tree_t	*sttmp;
+	struct st_tree_t	*sttmp;
 
 	/* find the tree node for var */
 	sttmp = state_tree_find(root, var);
 
-	if (!sttmp)
+	if (!sttmp) {
 		return 0;
+	}
 
 	return st_tree_del_enum(&sttmp->enum_list, val);
 }
 
 struct st_tree_t *state_tree_find(struct st_tree_t *node, const char *var)
 {
-	if (!node)
-		return NULL;
+	while (node) {
 
-	if (strcasecmp(var, node->var) < 0)
-		return state_tree_find(node->left, var);
+		if (strcasecmp(node->var, var) > 0) {
+			node = node->left;
+			continue;
+		}
 
-	if (strcasecmp(var, node->var) > 0)
-		return state_tree_find(node->right, var);
+		if (strcasecmp(node->var, var) < 0) {
+			node = node->right;
+			continue;
+		}
+
+		break;	/* found */
+	}
 
 	return node;
 }
