@@ -1,11 +1,13 @@
 /*  apc-hid.c - data to monitor APC USB/HID devices with NUT
  *
- *  Copyright (C)  
- *	2003 - 2008	Arnaud Quette <arnaud.quette@free.fr>
+ *  Copyright (C)
+ *	2003 - 2009	Arnaud Quette <arnaud.quette@free.fr>
  *	2005		John Stamp <kinsayder@hotmail.com>
- *  2005        Peter Selinger <selinger@users.sourceforge.net>
+ *	2005		Peter Selinger <selinger@users.sourceforge.net>
+ *	2009		Arjen de Korte <adkorte-guest@alioth.debian.org>
  *
  *  Sponsored by MGE UPS SYSTEMS <http://www.mgeups.com>
+ *   and Eaton <http://www.eaton.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,14 +26,12 @@
  *
  */
 
+#include "main.h"     /* for getval() */
 #include "usbhid-ups.h"
 #include "apc-hid.h"
-#include "extstate.h" /* for ST_FLAG_STRING */
-#include "main.h"     /* for getval() */
-#include "common.h"
 #include "usb-common.h"
 
-#define APC_HID_VERSION "APC HID 0.93"
+#define APC_HID_VERSION "APC HID 0.95"
 
 /* APC */
 #define APC_VENDORID 0x051d
@@ -40,7 +40,7 @@
 static usb_device_id_t apc_usb_device_table[] = {
 	/* various models */
 	{ USB_DEVICE(APC_VENDORID, 0x0002), NULL },
-	
+
 	/* Terminating entry */
 	{ -1, -1, NULL }
 };
@@ -78,8 +78,7 @@ info_lkp_t apc_date_conversion[] = {
 	{ 0, NULL, apc_date_conversion_fun }
 };
 
-/* This was determined empirically from observing a BackUPS LS 500.
- */
+/* This was determined empirically from observing a BackUPS LS 500 */
 static info_lkp_t apcstatusflag_info[] = {
 	{ 8, "!off", NULL },  /* Normal operation */
 	{ 16, "!off", NULL }, /* This occurs briefly during power-on, and corresponds to status 'DISCHRG'. */
@@ -87,6 +86,42 @@ static info_lkp_t apcstatusflag_info[] = {
 	{ 0, NULL, NULL }
 };
 
+/* Reason of the last battery transfer (from apcupsd) */
+static info_lkp_t apc_linefailcause_vrange_info[] = {
+	{ 1, "vrange", NULL },	/* Low line voltage */
+	{ 2, "vrange", NULL },	/* High line voltage */
+	{ 4, "vrange", NULL },	/* notch, spike, or blackout */
+	{ 8, "vrange", NULL },	/* Notch or blackout */
+	{ 9, "vrange", NULL },	/* Spike or blackout */
+	{ 0, "!vrange", NULL },		/* No transfers have ocurred */
+	{ 0, NULL, NULL }
+};
+
+static info_lkp_t apc_linefailcause_frange_info[] = {
+	{ 7, "frange", NULL },		/* Input frequency out of range */
+	{ 0, "!frange", NULL },		/* No transfers have ocurred */
+	{ 0, NULL, NULL }
+};
+
+#if 0
+/* these input.transfer.reason can't be mapped at the moment... */
+	{ 3, "ripple", NULL },		/* Ripple */
+	{ 5, "self test", NULL },	/* Self Test or Discharge Calibration commanded
+								 * Test usage, front button, or 2 week self test */
+	{ 6, "forced", NULL },		/* DelayBeforeShutdown or APCDelayBeforeShutdown */
+	{ 10, "forced", NULL },		/* Graceful shutdown by accessories */
+	{ 11, "self test", NULL },	/* Test usage invoked */
+	{ 12, "self test", NULL },	/* Front button initiated self test */
+	{ 13, "self test", NULL },	/* 2 week self test */
+	{ 0, NULL, NULL }
+#endif
+
+static info_lkp_t apc_sensitivity_info[] = {
+	{ 0, "low", NULL },
+	{ 1, "medium", NULL },
+	{ 2, "high", NULL },
+	{ 0, NULL, NULL }
+};
 
 /* --------------------------------------------------------------- */
 /*      Vendor-specific usage table */
@@ -101,16 +136,35 @@ static usage_lkp_t apc_usage_lkp[] = {
 	{ "APCBattReplaceDate",		0xff860016 },
 	{ "APCBattCapBeforeStartup",	0xff860019 }, /* FIXME: exploit */
 	{ "APC_UPS_FirmwareRevision",	0xff860042 },
+	{ "APCLineFailCause",	0xff860052 },
 	{ "APCStatusFlag",		0xff860060 },
+	{ "APCSensitivity",		0xff860061 },
+	/* usage seen in dumps but unknown:
+	 * - ff860027, ff860028
+	 * Path: UPS.ff860027, Type: Feature, ReportID: 0x3e, Offset: 0,
+	 * 	Size: 32, Value:0.000000
+	 */
 	{ "APCPanelTest",		0xff860072 }, /* FIXME: exploit */
 	{ "APCShutdownAfterDelay",	0xff860076 }, /* FIXME: exploit */
 	{ "APC_USB_FirmwareRevision",	0xff860079 }, /* FIXME: exploit */
 	{ "APCDelayBeforeReboot",	0xff86007c },
 	{ "APCDelayBeforeShutdown",	0xff86007d },
 	{ "APCDelayBeforeStartup",	0xff86007e }, /* FIXME: exploit */
+	/* usage seen in dumps but unknown:
+	 * - ff860080
+	 * Path: UPS.PresentStatus.ff860080, Type: Input, ReportID: 0x33,
+	 * 	Offset: 12, Size: 1, Value: 0.000000
+	 * - ff86001a
+	 * Path: UPS.Battery.ff86001a, Type: Input, ReportID: 0x1b,
+	 * 	Offset: 0, Size: 8, Value: 3.000000
+	 * - ff86001b
+	 * Path: UPS.Battery.ff86001b, Type: Input, ReportID: 0x1c,
+	 * 	Offset: 0, Size: 8, Value: 0.000000
+	 */
 
-	/* FIXME: what is BUP? To what vendor do these Usages belong?
-	 They seem to be here by mistake. -PS */
+	/* Note (Arnaud): BUP stands for BackUPS Pro
+	 * This is a HID uncompliant special (manufacturer) collection
+	 * FIXME: these need to be used... */
 	{ "BUPHibernate",		0x00850058 }, /* FIXME: exploit */
 	{ "BUPBattCapBeforeStartup",	0x00860012 }, /* FIXME: exploit */
 	{ "BUPDelayBeforeStartup",	0x00860076 }, /* FIXME: exploit */
@@ -120,36 +174,36 @@ static usage_lkp_t apc_usage_lkp[] = {
 };
 
 /*
- * USB USAGE NOTES for APC (from Russell Kroll in the old hidups
+ * USB USAGE NOTES for APC (from Russell Kroll in the old hidups)
  *
  * FIXME: read 0xff86.... instead of 0x(00)86....?
  *
- *  0x860013 == 44200155090 - capability again                   
- *           == locale 4, 4 choices, 2 bytes, 00, 15, 50, 90     
- *           == minimum charge to return online                  
+ *  0x860013 == 44200155090 - capability again
+ *           == locale 4, 4 choices, 2 bytes, 00, 15, 50, 90
+ *           == minimum charge to return online
  *
- *  0x860060 == "441HMLL" - looks like a 'capability' string     
- *           == locale 4, 4 choices, 1 byte each                 
- *           == line sensitivity (high, medium, low, low)        
- *  NOTE! the above does not seem to correspond to my info 
+ *  0x860060 == "441HMLL" - looks like a 'capability' string
+ *           == locale 4, 4 choices, 1 byte each
+ *           == line sensitivity (high, medium, low, low)
+ *  NOTE! the above does not seem to correspond to my info
  *
- *  0x860062 == D43133136127130                                  
- *           == locale D, 4 choices, 3 bytes, 133, 136, 127, 130 
- *           == high transfer voltage                            
+ *  0x860062 == D43133136127130
+ *           == locale D, 4 choices, 3 bytes, 133, 136, 127, 130
+ *           == high transfer voltage
  *
- *  0x860064 == D43103100097106                                  
- *           == locale D, 4 choices, 3 bytes, 103, 100, 097, 106 
- *           == low transfer voltage                             
+ *  0x860064 == D43103100097106
+ *           == locale D, 4 choices, 3 bytes, 103, 100, 097, 106
+ *           == low transfer voltage
  *
- *  0x860066 == 441HMLL (see 860060)                                   
+ *  0x860066 == 441HMLL (see 860060)
  *
- *  0x860074 == 4410TLN                                          
- *           == locale 4, 4 choices, 1 byte, 0, T, L, N          
- *           == alarm setting (5s, 30s, low battery, none)       
+ *  0x860074 == 4410TLN
+ *           == locale 4, 4 choices, 1 byte, 0, T, L, N
+ *           == alarm setting (5s, 30s, low battery, none)
  *
- *  0x860077 == 443060180300600                                  
- *           == locale 4, 4 choices, 3 bytes, 060,180,300,600    
- *           == wake-up delay (after power returns)              
+ *  0x860077 == 443060180300600
+ *           == locale 4, 4 choices, 3 bytes, 060,180,300,600
+ *           == wake-up delay (after power returns)
  */
 
 static usage_tables_t apc_utab[] = {
@@ -220,11 +274,17 @@ static hid_info_t apc_hid2nut[] = {
   { "BOOL", 0, 0, "UPS.PowerSummary.ShutdownImminent", NULL, NULL, 0, shutdownimm_info },
   { "BOOL", 0, 0, "UPS.PowerSummary.APCStatusFlag", NULL, NULL, HU_FLAG_QUICK_POLL, apcstatusflag_info }, /* APC Back-UPS LS 500 */
 
+  /* we map 2 times "input.transfer.reason" to be able to clear
+   * both vrange (voltage) and frange (frequency) */
+  { "BOOL", 0, 0, "UPS.Input.APCLineFailCause", NULL, NULL, 0, apc_linefailcause_vrange_info },
+  { "BOOL", 0, 0, "UPS.Input.APCLineFailCause", NULL, NULL, 0, apc_linefailcause_frange_info },
+
   /* Input page */
   { "input.voltage", 0, 0, "UPS.Input.Voltage", NULL, "%.1f", 0, NULL },
   { "input.voltage.nominal", 0, 0, "UPS.Input.ConfigVoltage", NULL, "%.0f", 0, NULL },
   { "input.transfer.low", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.Input.LowVoltageTransfer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
   { "input.transfer.high", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.Input.HighVoltageTransfer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
+  { "input.sensitivity", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.Input.APCSensitivity", NULL, "%s", HU_FLAG_SEMI_STATIC, apc_sensitivity_info },
 
   /* Output page */
   { "output.voltage", 0, 0, "UPS.Output.Voltage", NULL, "%.1f", 0, NULL },
@@ -243,8 +303,11 @@ static hid_info_t apc_hid2nut[] = {
    * ie: test.battery.start quick
    */
   { "test.battery.start.quick", 0, 0, "UPS.BatterySystem.Battery.Test", NULL, "1", HU_TYPE_CMD, NULL },
+  { "test.battery.start.quick", 0, 0, "UPS.Battery.Test", NULL, "1", HU_TYPE_CMD, NULL },	/* Back-UPS RS (experimental) */
   { "test.battery.start.deep", 0, 0, "UPS.BatterySystem.Battery.Test", NULL, "2", HU_TYPE_CMD, NULL },
+  { "test.battery.start.deep", 0, 0, "UPS.Battery.Test", NULL, "2", HU_TYPE_CMD, NULL },	/* Back-UPS RS (experimental) */
   { "test.battery.stop", 0, 0, "UPS.BatterySystem.Battery.Test", NULL, "3", HU_TYPE_CMD, NULL },
+  { "test.battery.stop", 0, 0, "UPS.Battery.Test", NULL, "3", HU_TYPE_CMD, NULL },	/* Back-UPS RS (experimental) */
   { "test.panel.start", 0, 0, "UPS.APCPanelTest", NULL, "1", HU_TYPE_CMD, NULL },
   { "test.panel.stop", 0, 0, "UPS.APCPanelTest", NULL, "0", HU_TYPE_CMD, NULL },
   { "test.panel.start", 0, 0, "UPS.PowerSummary.APCPanelTest", NULL, "1", HU_TYPE_CMD, NULL }, /* Back-UPS 500 */
