@@ -24,7 +24,7 @@
 #include "apcsmart.h"
 
 #define DRIVER_NAME	"APC Smart protocol driver"
-#define DRIVER_VERSION	"2.00"
+#define DRIVER_VERSION	"2.03"
 
 static upsdrv_info_t table_info = {
 	"APC command table",
@@ -106,13 +106,19 @@ static char *convert_data(struct apc_vartab_t *cmd_entry, char *upsval)
 			snprintf(tmp, sizeof(tmp), "%d", tval);
 			return tmp;
 
-		default:
-			upslogx(LOG_NOTICE, "Unable to handle conversion of %s",
-				cmd_entry->name);
-			return upsval;
+		case APC_F_REASON:
+			switch (upsval[0]) {
+				case 'R': return "unacceptable utility voltage rate of change";
+				case 'H': return "high utility voltage";
+				case 'L': return "low utility voltage";
+				case 'T': return "line voltage notch or spike";
+				case 'O': return "no transfers yet since turnon";
+				case 'S': return "simulated power failure or UPS test";
+				default: return upsval;
+			}
 	}
 
-	/* NOTREACHED */
+	upslogx(LOG_NOTICE, "Unable to handle conversion of %s", cmd_entry->name);
 	return upsval;
 }
 
@@ -557,19 +563,6 @@ static int firmware_table_lookup(void)
 		}
 
 		upsdebugx(2, "Firmware: [%s]", buf);
-
-		/* found one, force the model information */
-		if (!strcmp(buf, "6QD") || /* (APC600.) */
-				!strcmp(buf, "8QD") || /* (SmartUPS 1250, vintage 07/94.) */
-				!strcmp(buf, "8TI") || /* (SmartUPS 1250, vintage 08/24/95.) */
-				!strcmp(buf, "6TI") || /* (APC600.) */
-				!strcmp(buf, "6QI") || /* (APC600.) */
-				!strcmp(buf, "0ZI") || /* (APC Matrix 3000, vintage 10/99.) */
-				!strcmp(buf, "5UI")) { /* (APC Matrix 5000, vintage 10/93.) */
-			upsdebugx(1, "Found Smart-UPS");
-			dstate_setinfo("ups.model", "Smart-UPS");
-		}
-		else return 0;
 	}
 
 	/* this will be reworked if we get a lot of these things */
@@ -584,6 +577,12 @@ static int firmware_table_lookup(void)
 			upsdebugx(2, "Matched - cmdchars: %s",
 				compat_tab[i].cmdchars);
 
+			if (strspn(compat_tab[i].firmware, "05")) {
+				dstate_setinfo("ups.model", "Matrix-UPS");
+			} else {
+				dstate_setinfo("ups.model", "Smart-UPS");
+			}
+
 			/* matched - run the cmdchars from the table */
 			for (j = 0; j < strlen(compat_tab[i].cmdchars); j++)
 				protocol_verify(compat_tab[i].cmdchars[j]);
