@@ -23,7 +23,7 @@
 #include "serial.h"
 #include "timehead.h"
 
-#define APC_TABLE_VERSION	"version 2.0"
+#define APC_TABLE_VERSION	"version 2.1"
 
 /* Basic UPS reply line structure */
 #define ENDCHAR 10		/* APC ends responses with LF */
@@ -98,6 +98,7 @@
 #define APC_F_SECONDS	0x100000 /* Time in seconds */
 #define APC_F_MINUTES	0x110000 /* Time in minutes */
 #define APC_F_HOURS	0x120000 /* Time in hours */
+#define APC_F_REASON	0x130000 /* Reason of transfer */
 #define APC_F_LEAVE	0	/* Just pass this through */
 
 struct apc_vartab_t {
@@ -143,7 +144,7 @@ struct apc_vartab_t {
 	{ "input.transfer.high",
 				APC_F_VOLT,		'u' },
 	{ "input.transfer.reason", 
-				APC_POLL,		'G' },
+				APC_POLL|APC_F_REASON,	'G' },
 
 	{ "input.voltage.maximum",
 				APC_POLL|APC_F_VOLT,	'M' },
@@ -156,16 +157,16 @@ struct apc_vartab_t {
 				APC_F_VOLT,		'o' },
 
 	{ "ambient.humidity",  	APC_POLL|APC_F_PERCENT,	'h' },
-	{ "ambient.humidity.alarm.maximum", 
+	{ "ambient.humidity.high", 
 				APC_F_PERCENT,		'{' },
-	{ "ambient.humidity.alarm.minimum", 
+	{ "ambient.humidity.low", 
 				APC_F_PERCENT,		'}' },
 
 	{ "ambient.temperature", 	
 				APC_POLL|APC_F_CELSIUS, 't' },
-	{ "ambient.temperature.alarm.maximum",	
+	{ "ambient.temperature.high",	
 				APC_F_CELSIUS,		'[' },
-	{ "ambient.temperature.alarm.minimum", 	
+	{ "ambient.temperature.low", 	
 				APC_F_CELSIUS,		']' },
 
 	{ "battery.date",	APC_STRING,		'x' },
@@ -251,18 +252,34 @@ struct {
 	int	flags;
 } compat_tab[] =
 {
+	/* APC Matrix */
+	{ "0ZI",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz/<>", 0 },
+	{ "5UI",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz/<>", 0 },
+	{ "5ZM",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz/<>", 0 },
 	/* APC600 */
 	{ "6QD",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "6QI",    "79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "6TD",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
 	{ "6TI",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
-	{ "6QI",        "79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	/* SmartUPS 900 */
+	{ "7QD",    "79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "7QI",    "79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "7TD",    "79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "7TI",    "79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	/* SmartUPS 1250. */
+	{ "8QD",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "8QI",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "8TD",    "79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "8TI",    "79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
 	/* CS 350 */
 	{ "5.4.D",	"\1ABPQRSUYbdfgjmnx9",	0 },
 	/* Smart-UPS 600 */
-	{ "D6",         "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
-	{ "D5",         "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
-	{ "D4",         "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
-	/* SmartUPS 1250. */
-	{ "8QD",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "D9",     "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
+	{ "D8",     "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
+	{ "D7",     "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
+	{ "D6",     "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
+	{ "D5",     "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
+	{ "D4",     "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
 
 	{ NULL,		NULL,			0 },
 };
