@@ -10,6 +10,10 @@
 #define PW_MAX_TRY 3 /* How many times we try to send data. */
 
 #define PW_COMMAND_START_BYTE (unsigned char)0xAB
+#define PW_LAST_SEQ	(unsigned char)0x80	/* bit flag to indicate final sequence */
+#define PW_SEQ_MASK	(unsigned char)0x7F	/* bit mask to extract just the sequence # */
+
+#define PW_ANSWER_MAX_SIZE	256
 
 /* No	Autorisation required	*/ 
 #define PW_ID_BLOCK_REQ 	(unsigned char)0x31 /* Model name, ... length 1 */
@@ -20,7 +24,7 @@
 #define PW_BAT_TEST_REQ		(unsigned char)0x3B /* Charging, floating, ... length 1 */
 #define PW_LIMIT_BLOCK_REQ	(unsigned char)0x3C /* Configuration (Bypass thresholds,...).	length 1 */
 #define PW_TEST_RESULT_REQ	(unsigned char)0x3F /* ??. length 1 */
-#define PW_COMMAND_LIST_REQ	(unsigned char)0x40 /* ??. length 1 */
+#define PW_COMMAND_LIST_REQ	(unsigned char)0x40 /* Available commands. length 1 */
 #define PW_OUT_MON_BLOCK_REQ	(unsigned char)0x41 /* Outlet monitor request length 1 */
 #define PW_COM_CAP_REQ		(unsigned char)0x42 /* Request communication capabilities. length 2	*/
 #define PW_UPS_TOP_DATA_REQ	(unsigned char)0x43 /* Requsest ups topology data requset. length 1	*/
@@ -309,9 +313,21 @@
 #define BCMXCP_ALARM_CONFIG_ERROR			224
 #define BCMXCP_ALARM_REDUNDANCY_LOSS_DUE_TO_OVERLOAD	225
 #define BCMXCP_ALARM_ON_ALTERNATE_AC_SOURCE		226
+#define BCMXCP_ALARM_IN_HIGH_EFFICIENCY_MODE		227
+#define BCMXCP_ALARM_SYSTEM_NOTICE_ACTIVE		228
+#define BCMXCP_ALARM_SYSTEM_ALARM_ACTIVE		229
+#define BCMXCP_ALARM_ALTERNATE_POWER_SOURCE_NOT_AVAILABLE	230
+#define BCMXCP_ALARM_CURRENT_BALANCE_FAILURE		231
+#define BCMXCP_ALARM_CHECK_AIR_FILTER			232
+#define BCMXCP_ALARM_SUBSYSTEM_NOTICE_ACTIVE		233
+#define BCMXCP_ALARM_SUBSYSTEM_ALARM_ACTIVE		234
+#define BCMXCP_ALARM_CHARGER_ON_COMMAND			235
+#define BCMXCP_ALARM_CHARGER_OFF_COMMAND		236
+#define BCMXCP_ALARM_UPS_NORMAL				237
+#define BCMXCP_ALARM_EXTERNAL_COMMUNICATION_FAILURE	238
 
 #define BCMXCP_METER_MAP_MAX 91 /* Max no of entries in BCM/XCP meter map */
-#define	BCMXCP_ALARM_MAP_MAX 232 /* Max no of entries in BCM/XCP alarm map (adjusted upwards to nearest multi of 8 */
+#define	BCMXCP_ALARM_MAP_MAX 240 /* Max no of entries in BCM/XCP alarm map (adjusted upwards to nearest multi of 8 */
 
 typedef struct { /* Entry in BCM/XCP - UPS - NUT mapping table */
 	char *nut_entity;				/* The NUT variable name */
