@@ -1,6 +1,6 @@
 /*  belkin-hid.h - data to monitor Belkin UPS Systems USB/HID devices with NUT
  *
- *  Copyright (C)  
+ *  Copyright (C)
  *	2003 - 2008	Arnaud Quette <arnaud.quette@free.fr>
  *  2005        Peter Selinger <selinger@users.sourceforge.net>
  *
@@ -23,17 +23,18 @@
  *
  */
 
+#include "main.h"     /* for getval() */
 #include "usbhid-ups.h"
 #include "belkin-hid.h"
-#include "extstate.h" /* for ST_FLAG_STRING */
-#include "main.h"     /* for getval() */
-#include "common.h"
 #include "usb-common.h"
 
 #define BELKIN_HID_VERSION      "Belkin HID 0.12"
 
 /* Belkin */
-#define BELKIN_VENDORID 0x050d
+#define BELKIN_VENDORID	0x050d
+
+/* Liebert */
+#define LIEBERT_VENDORID	0x10af
 
 /* USB IDs device table */
 static usb_device_id_t belkin_usb_device_table[] = {
@@ -47,13 +48,18 @@ static usb_device_id_t belkin_usb_device_table[] = {
 	{ USB_DEVICE(BELKIN_VENDORID, 0x0912), NULL },
 	/* F6C550-AVR */
 	{ USB_DEVICE(BELKIN_VENDORID, 0x0551), NULL },
+	/* F6C1250-TW-RK */
+	{ USB_DEVICE(BELKIN_VENDORID, 0x0750), NULL },
 	/* F6C1500-TW-RK */
 	{ USB_DEVICE(BELKIN_VENDORID, 0x0751), NULL },
 	/* F6H375-USB */
 	{ USB_DEVICE(BELKIN_VENDORID, 0x0375), NULL },
 	/* F6C1100-UNV, F6C1200-UNV */
 	{ USB_DEVICE(BELKIN_VENDORID, 0x1100), NULL },
-	
+
+	/* Liebert PowerSure PSA UPS */
+	{ USB_DEVICE(LIEBERT_VENDORID, 0x0001), NULL },
+
 	/* Terminating entry */
 	{ -1, -1, NULL }
 };
@@ -67,7 +73,7 @@ static char *belkin_firmware_conversion_fun(double value)
 	static char buf[20];
 
 	snprintf(buf, sizeof(buf), "%ld", (long)value >> 4);
-	
+
 	return buf;
 }
 
@@ -85,7 +91,7 @@ static char *belkin_upstype_conversion_fun(double value)
 		return "line-interactive";
 	case 3:
 		return "simple online";
-	case 4: 
+	case 4:
 		return "simple offline";
 	case 5:
 		return "simple line-interactive";
@@ -131,7 +137,7 @@ static char *belkin_overload_conversion_fun(double value)
 		return "overload";
 	} else {
 		return "!overload";
-	}		
+	}
 }
 
 static info_lkp_t belkin_overload_conversion[] = {
@@ -183,7 +189,7 @@ static char *belkin_online_conversion_fun(double value)
 		return "!online";
 	} else {
 		return "online";
-	}		
+	}
 }
 
 static info_lkp_t belkin_online_conversion[] = {
@@ -222,7 +228,7 @@ static char *belkin_replacebatt_conversion_fun(double value)
 		return "replacebatt";
 	} else {
 		return "!replacebatt";
-	}		
+	}
 }
 
 static info_lkp_t belkin_replacebatt_conversion[] = {
@@ -259,7 +265,7 @@ static usage_lkp_t belkin_usage_lkp[] = {
 	{ "BELKINDelayBeforeShutdown",		0x00860057 }, /* R/W (seconds) */
 	{ "BELKINTest",				0x00860058 }, /* R/W: write: 0=no test, 1=quick test, 2=deep test, 3=abort test. Read: 0=no test, 1=passed, 2=warning, 3=error, 4=abort, 5=in progress */
 	{ "BELKINAudibleAlarmControl",		0x0086005a }, /* R/W: 1=disabled, 2=enabled, 3=muted */
-	
+
 	{ "BELKINDevice",			0x00860029 },
 	{ "BELKINVoltageSensitivity",		0x00860074 }, /* R/W: 0=normal, 1=reduced, 2=low */
 	{ "BELKINModelString",			0x00860075 },
@@ -372,7 +378,7 @@ static hid_info_t belkin_hid2nut[] = {
      principle. Hopefully other Belkin models will be better
      designed. Fixme: fill in the appropriate instant commands.  For a
      more detailed description of the problem and a possible (but not
-     yet implemented) workaround, see the belkinunv(8) man page. 
+     yet implemented) workaround, see the belkinunv(8) man page.
      -PS 2005/08/28 */
 
   /* end of structure. */
@@ -422,14 +428,17 @@ static char *belkin_format_serial(HIDDevice_t *hd) {
 
 /* this function allows the subdriver to "claim" a device: return 1 if
  * the device is supported by this subdriver, else 0. */
-static int belkin_claim(HIDDevice_t *hd) {
+static int belkin_claim(HIDDevice_t *hd)
+{
+	int status = is_usb_device_supported(belkin_usb_device_table, hd->VendorID, hd->ProductID);
 
-	int status = is_usb_device_supported(belkin_usb_device_table, hd->VendorID,
-								 hd->ProductID);
+	switch (status)
+	{
+	case POSSIBLY_SUPPORTED:
 
-	switch (status) {
-
-		case POSSIBLY_SUPPORTED:
+		switch (hd->VendorID)
+		{
+		case BELKIN_VENDORID:
 			/* reject any known non-UPS */
 			if (hd->ProductID == 0x0218)  /* F5U218-MOB 4-Port USB Hub */
 				return 0;
@@ -441,12 +450,22 @@ static int belkin_claim(HIDDevice_t *hd) {
 			possibly_supported("Belkin", hd);
 			return 0;
 
-		case SUPPORTED:
-			return 1;
-
-		case NOT_SUPPORTED:
-		default:
+		case LIEBERT_VENDORID:
+			/* by default, reject, unless the productid option is given */
+			if (getval("productid")) {
+				return 1;
+			}
+			possibly_supported("Liebert", hd);
 			return 0;
+		}
+		return 0;
+
+	case SUPPORTED:
+		return 1;
+
+	case NOT_SUPPORTED:
+	default:
+		return 0;
 	}
 }
 
