@@ -93,7 +93,18 @@
 
 #include "main.h"
 #include "serial.h"
-#include "belkinunv.h"
+
+#define DRIVER_NAME	"Belkin 'Universal UPS' driver"
+#define DRIVER_VERSION	"0.07"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Peter Selinger <selinger@users.sourceforge.net>",
+	DRV_STABLE,
+	{ NULL }
+};
 
 /* somewhat arbitrary buffer size - the longest actually occuring
    message is 18 bytes for the F6C800-UNV. But since message length is
@@ -246,7 +257,10 @@ static int belkin_nut_receive(unsigned char *buf, int bufsize) {
 		return -1;
 	}
 	r = ser_get_buf_len(upsfd, &buf[0], 1, 3, 0);
-	if (r==-1) {
+	if (r<0) {
+		upslog_with_errno(LOG_ERR, "Error reading from UPS");
+		return -1;
+	} else if (r==0) {
 		upslogx(LOG_ERR, "No response from UPS");
 		return -1;
 	} else if (buf[0]!=0x7e) {
@@ -854,8 +868,6 @@ void upsdrv_initinfo(void)
 	int val;
 	int i;
 
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
-
 	/* read hard-wired values */
 	val = belkin_nut_read_int(REG_VOLTRATING);
 	if (val!=-1) {
@@ -1295,12 +1307,6 @@ void upsdrv_makevartable(void)
 
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Belkin 'Universal UPS' driver %s (%s)\n\n", 
-	       DRV_VERSION, UPS_VERSION);
-}
-
 /* prep the serial port */
 void upsdrv_initups(void)
 {
