@@ -43,7 +43,21 @@
 
 #include "main.h"
 #include "serial.h"
-#include "bestfcom.h"
+
+#define DRIVER_NAME	"Best Ferrups/Fortress driver"
+#define DRIVER_VERSION	"0.12"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Andreas Wrede  <andreas@planix.com>\n" \
+	"John Stone  <johns@megapixel.com>\n" \
+	"Grant Taylor <gtaylor@picante.com>\n" \
+	"Russell Kroll <rkroll@exploits.org>",
+	DRV_EXPERIMENTAL,
+	{ NULL }
+};
 
 #define ENDCHAR			'\r'
 #define IGNCHARS		"\012"
@@ -193,7 +207,7 @@ static int execute(const char *cmd, char *result, int resultsize)
 	ser_get_line_alert(upsfd, buf, sizeof(buf), '\012', "",
 		POLL_ALERT, alert_handler, 0, 20000);
 
-	ser_send(upsfd, cmd);
+	ser_send(upsfd, "%s", cmd);
 
 	/* Give the UPS some time to chew on what we just sent */
 	usleep(50000);
@@ -248,7 +262,7 @@ void upsdrv_updateinfo(void)
 
 		char date[9], time[9], tmp[32];
 
-		upsdebugx(3, "f response: %d %s", strlen(fstring), fstring);
+		upsdebugx(3, "f response: %d %s", (int)strlen(fstring), fstring);
 
 		date[0]='\0';
 		time[0]='\0';
@@ -277,8 +291,8 @@ void upsdrv_updateinfo(void)
 		/* AC line frequency */
 		acfreq = ((double)bcd2i(&fstring[54], 4) / 100.0);
 
-		/* Runtime remaining */
-		btimeleft = bcd2i(&fstring[58], 4);
+		/* Runtime remaining (UPS reports minutes) */
+		btimeleft = bcd2i(&fstring[58], 4) * 60;
 
 		if (fc.model != FDxxxx) {
 			/* Iout.  int times 10 */
@@ -402,7 +416,7 @@ void upsdrv_updateinfo(void)
 
 	} else {
 
-		upsdebugx(1, "failed f response. strlen: %d", strlen(fstring));
+		upsdebugx(1, "failed f response. strlen: %d", (int)strlen(fstring));
 		dstate_datastale();
 
 	} /* if (execute("f\r", fstring, sizeof(fstring)) >= 80) */
@@ -445,14 +459,6 @@ void upsdrv_help(void)
 {
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Best Ferrups/Fortress %s (%s)\n",
-		DRV_VERSION, UPS_VERSION);
-
-	experimental_driver = 1;
-}
-
 static void sync_serial(void) {
 	char buffer[10];
 
@@ -658,7 +664,7 @@ void upsdrv_init_fc(const char *fcstring)
 {
 	char tmp[256];
 
-	upsdebugx(3, "fc response: %d %s", strlen(fcstring), fcstring);
+	upsdebugx(3, "fc response: %d %s", (int)strlen(fcstring), fcstring);
 
 	/* Obtain Model */
 	if (memcmp(fcstring, "$", 1)) {
@@ -739,7 +745,7 @@ void upsdrv_initups ()
 
 	inverter_status = 0;
 	fc.model = UNKNOWN;
-	if (execute("f\r", rstring, sizeof(rstring)) < 0 ) {
+	if (execute("f\r", rstring, sizeof(rstring)) < 1 ) {
 		fatalx(EXIT_FAILURE, "Failed format request in upsdrc_initups()");
 	}
 
