@@ -3,6 +3,7 @@
 
    Copyright (C) 2002  Russell Kroll <rkroll@exploits.org> (skeleton)
              (C) 2002  Holger Dietze <holger.dietze@advis.de>
+             (C) 2009  Stuart D. Gathman <stuart@bmsi.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,16 +17,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-*/
-
-/*
-	anything commented is optional
-	anything else is mandatory
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include "main.h"
+#include "serial.h"
 
+#define UPSDELAY 50000	/* 50 ms delay required for reliable operation */
+#define SER_WAIT_SEC	2	/* allow 2.0 sec for ser_get calls */
+#define SER_WAIT_USEC	0
 #define ENDCHAR		'\r'
 #define IGNCHARS	" \n"
 
@@ -33,8 +33,24 @@
 #define        inline  __inline
 #endif
 
-void instcmd (int auxcmd, int dlen, char *data);
-void upsdrv_setvar (int, int, char *);
+#define DRIVER_NAME             "Best Fortress UPS driver"
+#define DRIVER_VERSION  "0.02"
+
+/* driver description structure */
+upsdrv_info_t   upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Holger Dietze <holger.dietze@advis.de>\n"
+	"Stuart D. Gathman <stuart@bmsi.com>\n",
+	DRV_EXPERIMENTAL,
+	{ NULL }
+};
+
+static int instcmd (const char *cmdname, const char *extra);
+static int upsdrv_setvar (const char *varname, const char *val);
+
+/* rated VA load if known */
+static int maxload = 0;
 
 void upsdrv_initinfo(void)
 {
@@ -42,17 +58,19 @@ void upsdrv_initinfo(void)
 	dstate_setinfo("ups.model", "Fortress");
 	dstate_setinfo("battery.voltage.nominal", "24");
 
-	/*addinfo (INFO_ALRM_OVERLOAD, "", 0, 0);*/ /* Flag */
-	/*addinfo (INFO_ALRM_TEMP, "", 0, 0);*/ /* Flag */
+	/*dstate_setinfo ("alarm.overload", "0");*/ /* Flag */
+	/*dstate_setinfo ("alarm.temp", "0");*/ /* Flag */
+	if (maxload)
+		dstate_setinfo("ups.load", "0");
+	dstate_setinfo("output.voltamps", "0");
+	dstate_setinfo("ups.delay.shutdown", "10");	/* write only */
 
-	dstate_setinfo("ups.delay.shutdown", "10");	/* write only */	
-	
 	/* tunable via front panel: (european voltage level)
 	   parameter		factory default  range
 	   INFO_LOWXFER	196 V   p7=nnn   160-210
 	   INFO_HIGHXFER	254 V   p8=nnn   215-274
 	   INFO_LOBATTIME	2 min   p2=n     1-5
-	   
+
 	   comm mode    p6=0 dumb DONT USE (will lose access to parameter setting!)
 	   		p6=1 B1200
 			p6=2 B2400
@@ -63,19 +81,19 @@ void upsdrv_initinfo(void)
 	   echo off     e0
 	   echo on      e1
 	*/
-	dstate_setinfo("input.transfer.low", "");
+	dstate_setinfo("input.transfer.low", "%s", "");
 	dstate_setflags("input.transfer.low", ST_FLAG_STRING | ST_FLAG_RW);
 	dstate_setaux("input.transfer.low", 3);
 
-	dstate_setinfo("input.transfer.high", "");
+	dstate_setinfo("input.transfer.high", "%s", "");
 	dstate_setflags("input.transfer.high", ST_FLAG_STRING | ST_FLAG_RW);
 	dstate_setaux("input.transfer.high", 3);
 
-	dstate_setinfo("battery.runtime.low", "");
+	dstate_setinfo("battery.runtime.low", "%s", "");
 	dstate_setflags("battery.runtime.low", ST_FLAG_STRING | ST_FLAG_RW);
 	dstate_setaux("battery.runtime.low", 3);
 
-	upsh.instcmd = instcmd; 
+	upsh.instcmd = instcmd;
 	upsh.setvar = upsdrv_setvar;
 
 	dstate_addcmd("shutdown.return");
@@ -103,37 +121,78 @@ static int checksum (char * s)
 }
 
 /* set info to integer value */
-static inline void setinfo_int (int id, char * s, int len)
+static inline int setinfo_int (const char *key, const char * s, size_t len)
 {
 	char buf[10];
-	
+	int val;
+
 	if (len > sizeof(buf)) len = sizeof(buf)-1;
 	strncpy (buf, s, len);
 	buf[len] = 0;
-	setinfo (id, "%d", atoi (buf));
+	val = atoi(buf);
+	dstate_setinfo (key, "%d", val);
+	return val;
 }
 
 /* set info to integer value (for runtime remaining)
    value is expressed in minutes, but desired in seconds
  */
-static inline void setinfo_int_minutes (int id, char * s, int len)
+static inline void setinfo_int_minutes (const char *key, const char * s, size_t len)
 {
 	char buf[10];
-	
+
 	if (len > sizeof(buf)) len = sizeof(buf)-1;
 	strncpy (buf, s, len);
 	buf[len] = 0;
-	setinfo (id, "%d", 60*atoi (buf));
+	dstate_setinfo (key, "%d", 60*atoi (buf));
 }
 
 /* set info to float value */
-static inline void setinfo_float (int id, char * fmt, char * s, int len, double factor)
+static inline void setinfo_float (const char *key, char * fmt, const char * s, size_t len, double factor)
 {
 	char buf[10];
 	if (len > sizeof(buf)) len = sizeof(buf)-1;
 	strncpy (buf, s, len);
 	buf[len] = 0;
-	setinfo (id, fmt, factor * (double)atoi (buf));
+	dstate_setinfo (key, fmt, factor * (double)atoi (buf));
+}
+
+static int upssend(const char *fmt,...) {
+	int	ret;
+	char buf[1024], *p;
+	va_list	ap;
+	unsigned int	sent = 0;
+	int d_usec = UPSDELAY;
+
+	va_start(ap, fmt);
+	ret = vsnprintf(buf, sizeof(buf), fmt, ap);
+	va_end(ap);
+
+	if ((ret < 1) || (ret >= (int) sizeof(buf)))
+		upslogx(LOG_WARNING, "ser_send_pace: vsnprintf needed more "
+				"than %d bytes", (int)sizeof(buf));
+	for (p = buf; *p; p++) {
+		if (write(upsfd, p, 1) != 1)
+			return -1;
+
+		if (d_usec)
+			usleep(d_usec);
+
+		sent++;
+	}
+
+	return sent;
+}
+
+static int upsrecv(char *buf,size_t bufsize,char ec,const char *ic)
+{
+	return ser_get_line(upsfd, buf, bufsize - 1, ec, ic,
+			SER_WAIT_SEC, SER_WAIT_USEC);
+}
+
+static int upsflushin(int f,int verbose,const char *ignset)
+{
+	return ser_flush_in(upsfd, ignset, verbose);
 }
 
 /* read out UPS and store info */
@@ -141,47 +200,70 @@ void upsdrv_updateinfo(void)
 {
 	char temp[256];
 	char *p;
-	
-	int checksum_ok, is_online=1, is_off, low_batt, trimming, boosting;
-	
-	upssend ("f\r");
-	
-	do {
-		if (upsrecv (temp, sizeof(temp), ENDCHAR, IGNCHARS) <= 0) {
-			upsflushin (0, 0, "\r ");
-			upssend ("f\r");
-		}
-	} while (temp[0] == 0);
-	/* setinfo (INFO_, ""); */
-	p = temp;
-	
-	/*syslog (LOG_DAEMON | LOG_NOTICE,"ups: got '%s'\n", p);*/
-	/* status example:
-	   |Vi||Vo|    |Io||Psou|    |Vb||f| |tr||Ti|            CS
-	   000000000001000000000000023802370000000200004700000267500000990030000000000301BD
-	   1    1    2    2    3    3    4    4    5    5    6    6    7    7   78
-	   0    5    0    5    0    5    0    5    0    5    0    5    0    5    0    5   90
-	*/
+	int loadva;
+	int len;
+	int retry;
 
-	/* last bytes are a checksum:
-	   interpret response as hex string, sum of all bytes must be zero
-	*/
-	checksum_ok = (checksum (p) & 0xff) == 0;
+	int checksum_ok, is_online=1, is_off, low_batt, trimming, boosting;
+
+	for (retry = 0; retry < 5; ++retry) {
+		upsflushin (0, 0, "\r ");
+		upssend ("f\r");
+		do {
+			if (upsrecv (temp+2, sizeof temp - 2, ENDCHAR, IGNCHARS) <= 0) {
+				upsflushin (0, 0, "\r ");
+				upssend ("f\r");
+			}
+		} while (temp[2] == 0);
+
+		/*syslog (LOG_DAEMON | LOG_NOTICE,"ups: got '%s'\n", p);*/
+		/* status example:
+		   000000000001000000000000012201210000001200014500000280600000990025000000000301BE
+		   000000000001000000000000012401230000001200014800000280600000990025000000000301B7
+		   |Vi||Vo|    |Io||Psou|    |Vb||f| |tr||Ti|            CS
+		   000000000001000000000000023802370000000200004700000267500000990030000000000301BD
+		   1    1    2    2    3    3    4    4    5    5    6    6    7    7   78
+		   0    5    0    5    0    5    0    5    0    5    0    5    0    5    0    5   90
+		 */
+
+		/* last bytes are a checksum:
+		   interpret response as hex string, sum of all bytes must be zero
+		 */
+		checksum_ok = (checksum (temp+2) & 0xff) == 0;
+		/* setinfo (INFO_, ""); */
+
+		/* I can't figure out why this is missing the first two chars.
+		   But the first two chars are not used, so just set them to zero
+		   when missing. */
+		len = strlen(temp+2);
+		temp[0] = '0';
+		temp[1] = '0';
+		p = temp+2;
+		if (len == 78)
+			p = temp;
+		else if (len != 80)
+			checksum_ok = 0;
+		if (checksum_ok) break;
+		sleep(SER_WAIT_SEC);
+	}
 
 	if (!checksum_ok) {
 		dstate_datastale();
 		return;
 	}
-	
-	setinfo_int (INFO_UTILITY, p+24,4);
-	setinfo_int (INFO_OUTVOLT, p+28,4);
-	setinfo_float (INFO_BATTVOLT, "%.1f", p+50,4, 0.1);
-	setinfo_float (INFO_CURRENT, "%.1f", p+36,4, 0.1);
-	setinfo_int (INFO_OUT_VA, p+40,6);
-	setinfo_float (INFO_ACFREQ, "%.1f", p+54,3, 0.1);
-	setinfo_int_minutes (INFO_BATT_RUNTIME, p+58,4);
-	setinfo_int (INFO_UPSTEMP, p+62,4);
-	
+	/* upslogx(LOG_INFO, "updateinfo: %s", p); */
+
+	setinfo_int ("input.voltage", p+24,4);
+	setinfo_int ("output.voltage", p+28,4);
+	setinfo_float ("battery.voltage", "%.1f", p+50,4, 0.1);
+	setinfo_float ("output.current", "%.1f", p+36,4, 0.1);
+	loadva = setinfo_int ("output.voltamps", p+40,6);
+	if (maxload)
+		dstate_setinfo ("ups.load", "%d", loadva * 100 / maxload);
+	setinfo_float ("input.frequency", "%.1f", p+54,3, 0.1);
+	setinfo_int_minutes ("battery.runtime", p+58,4);
+	setinfo_int ("ups.temperature", p+62,4);
+
 	is_online = p[17] == '0';
 	low_batt = fromhex(p[21]) & 8 || fromhex(p[20]) & 1;
 	is_off = p[11] == '0';
@@ -190,10 +272,15 @@ void upsdrv_updateinfo(void)
 			 (brownouts are very rare here and I can't
 			 simulate one) */
 
-	setinfo (INFO_STATUS, "%s%s%s",
-		 is_online ? (is_off ? "OFF " : "OL ") : "OB ",
-		 low_batt ? "LB " : "",
-		 trimming ? "TRIM" : boosting ? "BOOST" : "");
+	status_init();
+	if (low_batt)
+		status_set("LB ");
+	else if (trimming)
+		status_set("TRIM");
+	else if (boosting)
+		status_set("BOOST");
+	else
+		status_set(is_online ? (is_off ? "OFF " : "OL ") : "OB ");
 
 	/* setinfo(INFO_STATUS, "%s%s",
 	 *	(util < lownorm) ? "BOOST ", "",
@@ -202,6 +289,7 @@ void upsdrv_updateinfo(void)
 	 *	((flags & TIOCM_CTS) == TIOCM_CTS) ? "OB" : "OL");
 	 */
 
+	status_commit();
 	dstate_dataok();
 }
 
@@ -211,7 +299,7 @@ void upsdrv_updateinfo(void)
 /* all UPS tunable parameters are set with command
    'p%d=%s'
 */
-int setparam (int parameter, int dlen, char * data)
+int setparam (int parameter, int dlen, const char * data)
 {
 	char reply[80];
 	upssend ("p%d=%*s\r", parameter, dlen, data);
@@ -238,34 +326,36 @@ static void autorestart (int restart)
 }
 
 /* set UPS parameters */
-void upsdrv_setvar (int info, int len, char * data)
-{
+static int upsdrv_setvar (const char *var, const char * data) {
 	int parameter;
-	switch (info) {
-	case INFO_LOWXFER:
+	int len = strlen(data);
+	upsdebugx(1, "Setvar: %s %s", var, data);
+	if (strcmp("input.transfer.low", var) == 0) {
 		parameter = 7;
-		break;
-	case INFO_HIGHXFER:
+	}
+	else if (strcmp("input.transfer.high", var) == 0) {
 		parameter = 8;
-		break;
-	case INFO_LOBATTIME:
+	}
+	else if (strcmp("battery.runtime.low", var) == 0) {
 		parameter = 2;
-		break;
-	default:
-		return;
+	}
+	else {
+		upslogx(LOG_INFO, "Setvar: unsettable variable %s", var);
+		return STAT_SET_UNKNOWN;
 	}
 	ups_setsuper (1);
 	if (setparam (parameter, len, data)) {
-		setinfo (info, "%*s", len, data);
+		dstate_setinfo (var, "%*s", len, data);
 	}
 	ups_setsuper (0);
+	return STAT_SET_HANDLED;
 }
 
 void upsdrv_shutdown(void)
 {
 	const	char	*grace;
 
-	grace = getdata(INFO_PDNGRACE);
+	grace = dstate_getinfo("ups.delay.shutdown");
 
 	if (!grace)
 		grace = "1"; /* apparently, OFF0 does not work */
@@ -278,26 +368,26 @@ void upsdrv_shutdown(void)
 	/* OFF will powercycle when line power is available again */
 }
 
-void instcmd (int auxcmd, int dlen, char *data)
+static int instcmd (const char *cmdname, const char *extra)
 {
 	const char *p;
-	
-	switch (auxcmd) {
-	case CMD_OFF:
+
+	if (!strcasecmp(cmdname, "load.off")) {
 		printf ("powering off\n");
 		autorestart (0);
 		upssend ("OFF1\r");
-		break;
-	case CMD_SHUTDOWN:
-		p = getdata (INFO_PDNGRACE);
+		return STAT_INSTCMD_HANDLED;
+	}
+	else if (!strcasecmp(cmdname, "shutdown.return")) {
+		p = dstate_getinfo ("ups.delay.shutdown");
 		if (!p) p = "1";
 		printf ("shutdown in %s seconds\n", p);
 		autorestart (1);
 		upssend ("OFF%s\r", p);
-		break;
-	default:
-		upslogx(LOG_INFO, "instcmd: unknown type 0x%04x", auxcmd);
+		return STAT_INSTCMD_HANDLED;
 	}
+	upslogx(LOG_INFO, "instcmd: unknown command %s", cmdname);
+	return STAT_INSTCMD_UNKNOWN;
 }
 
 void upsdrv_help(void)
@@ -307,17 +397,8 @@ void upsdrv_help(void)
 /* list flags and values that you want to receive via -x */
 void upsdrv_makevartable(void)
 {
-	/* allow '-x xyzzy' */
-	/* addvar(VAR_FLAG, "xyzzy", "Enable xyzzy mode"); */
-	
-	/* allow '-x foo=<some value>' */
-	/* addvar(VAR_VALUE, "foo", "Override foo setting"); */
-	addvar (VAR_VALUE, "speed", "serial line speed");
-}
-
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Best Fortress UPS driver 0.01 (%s)\n\n", UPS_VERSION);
+	addvar (VAR_VALUE, "baudrate", "serial line speed");
+	addvar (VAR_VALUE, "max_load", "rated VA load VA");
 }
 
 struct {
@@ -335,7 +416,10 @@ void upsdrv_initups(void)
 {
 	speed_t speed = B1200;
 
-	char * speed_val = getval ("speed");
+	char * speed_val = getval("baudrate");
+	char * max_load = getval("max_load");
+
+	if (max_load) maxload = atoi(max_load);
 
 	if (speed_val) {
 		int i;
@@ -345,10 +429,11 @@ void upsdrv_initups(void)
 		}
 		speed = speed_table[i].speed;
 	}
-	
-	open_serial(device_path, speed);
+
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, speed);
 	/* TODO: probe ups type */
-	
+
 	/* the upsh handlers can't be done here, as they get initialized
 	 * shortly after upsdrv_initups returns to main.
 	 */
