@@ -2,7 +2,7 @@
  *
  *  Copyright (C)
  *  2003 - 2008 Arnaud Quette <arnaud.quette@free.fr>
- *  2005 - 2006 Peter Selinger <selinger@users.sourceforge.net>         
+ *  2005 - 2006 Peter Selinger <selinger@users.sourceforge.net>
  *
  *  Note: this subdriver was initially generated as a "stub" by the
  *  path-to-subdriver script. It must be customized.
@@ -23,14 +23,12 @@
  *
  */
 
+#include "main.h"     /* for getval() */
 #include "usbhid-ups.h"
 #include "cps-hid.h"
-#include "extstate.h" /* for ST_FLAG_STRING */
-#include "main.h"     /* for getval() */
-#include "common.h"
 #include "usb-common.h"
 
-#define CPS_HID_VERSION      "CyberPower HID 0.2"
+#define CPS_HID_VERSION      "CyberPower HID 0.3"
 
 /* Cyber Power Systems */
 #define CPS_VENDORID 0x0764
@@ -54,6 +52,8 @@ static usb_device_id_t cps_usb_device_table[] = {
 	{ USB_DEVICE(CPS_VENDORID, 0x0005), NULL },
 	/* Dynex DX-800U? */
 	{ USB_DEVICE(CPS_VENDORID, 0x0501), &cps_battery_scale },
+	/* OR2200LCDRM2U */
+	{ USB_DEVICE(CPS_VENDORID, 0x0601), NULL },
 
 	/* Terminating entry */
 	{ -1, -1, NULL }
