@@ -120,6 +120,7 @@ typedef struct {
 					char *value);*/	/* NUT function */
 } method_lkp_t;
 
+#if 0
 /* Data to lookup between NUT commands and HAL/DBus methods
  * for dstate_addcmd() */
 static method_lkp_t nut2hal_cmd[] =
@@ -137,6 +138,7 @@ static method_lkp_t nut2hal_cmd[] =
 	/* Terminating element */
 	{ NULL, NULL, NULL }
 };
+#endif
 
 /* Data to lookup between NUT and HAL for dstate_setinfo() */
 static info_lkp_t nut2hal_info[] =
@@ -303,8 +305,6 @@ void dstate_init(const char *prog, const char *port)
 
 const char *dstate_getinfo(const char *var)
 {
-	char *value = NULL;
-	int ivalue;
 	info_lkp_t *nut2hal_info = find_nut_info(var, NULL);
 
 /* FIXME: use the data exposed by NUT on the DBus when available */
@@ -314,20 +314,18 @@ const char *dstate_getinfo(const char *var)
 		switch (nut2hal_info->hal_type)
 		{
 /*			case HAL_TYPE_INT:
-				ivalue = hal_get_int(nut2hal_info->hal_name);
-				value = xstrdup("12345");
-				snprintf(value, 5, "%i", ivalue);
-				break;
+				static char	value[8];
+				snprintf(value, sizeof(value), "%i", hal_get_int(nut2hal_info->hal_name));
+				return value;
 			case HAL_TYPE_BOOL:
 				hal_set_bool(cs, nut2hal_info->hal_name, TRUE);
 				break;
 */
 			case HAL_TYPE_STRING:
-				value = hal_get_string(nut2hal_info->hal_name);
-				break;
+				return hal_get_string(nut2hal_info->hal_name);
 		}
 	}	
-	return value;
+	return NULL;
 }
 
 int dstate_setinfo(const char *var, const char *fmt, ...)
@@ -384,6 +382,12 @@ int dstate_setinfo(const char *var, const char *fmt, ...)
 	return ret;
 }
 
+
+int dstate_addenum(const char *var, const char *fmt, ...)
+{
+	return 0;
+}
+	
 int dstate_delinfo(const char *var)
 {
 	return 0;
@@ -420,10 +424,14 @@ void dstate_free(void)
 }
 
 /* extrafd: provided for waking up based on the driver's UPS fd */
-int dstate_poll_fds(int interval, int extrafd)
+int dstate_poll_fds(struct timeval timeout, int extrafd)
 {
-	return 0;
-}	
+	/* drivers expect us to limit the polling rate here */
+	sleep(timeout.tv_sec);
+
+	/* the timeout expired */
+	return 1;
+}
 
 /* clean out the temp space for a new pass */
 void status_init(void)
