@@ -22,11 +22,24 @@
 
 #include "main.h"
 #include "serial.h"
-#include "isbmex.h"
 
 #include <math.h>		/* for sqrt */
 #include <string.h>
 
+#define DRIVER_NAME	"ISBMEX UPS driver"
+#define DRIVER_VERSION	"0.06"
+
+/* driver description structure */
+upsdrv_info_t	upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Ricardo Martinezgarza <ricardo@nexxis.com.mx>\n" \
+	"Edscott Wilson Garcia <edscott@imp.mx>\n" \
+	"Russell Kroll <rkroll@exploits.org>",
+	DRV_STABLE,
+	{ NULL }
+};
+
 #define xDEBUG
 
 #ifdef DEBUG
@@ -131,8 +144,6 @@ void upsdrv_initinfo(void)
 	 
  	 /* addinfo(INFO_, "", 0, 0); */
 	 /*printf("Using %s %s on %s\n", getdata(INFO_MFR), getdata(INFO_MODEL), device_path);*/
-
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 }
 
 static const char *getpacket(int *we_know){
@@ -330,12 +341,6 @@ void upsdrv_makevartable(void)
 {
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - ISBMEX UPS driver %s (%s)\n\n",
-		DRV_VERSION, UPS_VERSION);
-}
-
 void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path);
