@@ -53,8 +53,8 @@ static const char *hid_lookup_path(const HIDNode_t usage, usage_tables_t *utab);
 static long hid_lookup_usage(const char *name, usage_tables_t *utab);
 static int string_to_path(const char *string, HIDPath_t *path, usage_tables_t *utab);
 static int path_to_string(char *string, size_t size, const HIDPath_t *path, usage_tables_t *utab);
-static long get_unit_expo(const long UnitType);
-static double exponent(double a, int b);
+static int8_t get_unit_expo(const HIDData_t *hiddata);
+static double exponent(double a, int8_t b);
 
 /* ---------------------------------------------------------------------- */
 /* report buffering system */
@@ -85,17 +85,17 @@ void free_report_buffer(reportbuf_t *rbuf)
    with free_report_buffer(). */
 reportbuf_t *new_report_buffer(HIDDesc_t *pDesc)
 {
-	HIDData_t *pData;
-	reportbuf_t *rbuf;
-	int    i, id;
- 
+	HIDData_t	*pData;
+	reportbuf_t	*rbuf;
+	int		i, id;
+
 	if (!pDesc)
 		return NULL;
- 
+
 	rbuf = calloc(1, sizeof(*rbuf));
-        if (!rbuf) {
-               return NULL;
- 	}
+	if (!rbuf) {
+		return NULL;
+	}
 
 	/* now go through all items that are part of this report */
 	for (i=0; i<pDesc->nitems; i++) {
@@ -107,7 +107,7 @@ reportbuf_t *new_report_buffer(HIDDesc_t *pDesc)
 		/* skip reports that already have been allocated */
 		if (rbuf->data[id])
 			continue;
- 
+
 		/* first byte holds id */
 		rbuf->len[id] = pDesc->replen[id] + 1;
 
@@ -119,7 +119,7 @@ reportbuf_t *new_report_buffer(HIDDesc_t *pDesc)
 		rbuf->data[id] = calloc(rbuf->len[id], sizeof(*(rbuf->data[id])));
 		if (rbuf->data[id])
 			continue;
- 
+
 		/* on failure, give up what we got so far */
 		free_report_buffer(rbuf);
 		return NULL;
@@ -141,6 +141,7 @@ static int refresh_report_buffer(reportbuf_t *rbuf, hid_dev_handle_t udev, HIDDa
 {
 	int	id = pData->ReportID;
 	int	r;
+	unsigned char	buf[SMALLBUF];
 
 	if (rbuf->ts[id] + age > time(NULL)) {
 		/* buffered report is still good; nothing to do */
@@ -148,20 +149,24 @@ static int refresh_report_buffer(reportbuf_t *rbuf, hid_dev_handle_t udev, HIDDa
 		return 0;
 	}
 
-	r = comm_driver->get_report(udev, id, rbuf->data[id], rbuf->len[id]);
+	r = comm_driver->get_report(udev, id, buf, sizeof(buf));
 	if (r <= 0) {
 		return -1;
 	}
 
-	if (rbuf->len[id] > r) {
-		upsdebugx(4, "refresh_report_buffer: expected %d bytes, but got only %d", rbuf->len[id], r);
+	/* broken report descriptors are common, so store whatever we can */
+	memcpy(rbuf->data[id], buf, rbuf->len[id]);
+
+	if (rbuf->len[id] != r) {
+		upsdebugx(2, "%s: expected %d bytes, but got %d instead", __func__, rbuf->len[id], r);
+		upsdebug_hex(3, "Report[err]", buf, r);
+	} else {
+		upsdebug_hex(3, "Report[get]", rbuf->data[id], rbuf->len[id]);
 	}
 
-	/* have valid report */
+	/* have (valid) report */
 	time(&rbuf->ts[id]);
 
-	upsdebug_hex(3, "Report[get]", rbuf->data[id], rbuf->len[id]);
-
 	return 0;
 }
 
@@ -199,11 +204,11 @@ static int set_item_buffered(reportbuf_t *rbuf, hid_dev_handle_t udev, HIDData_t
 		return -1;
 	}
 
+	upsdebug_hex(3, "Report[set]", rbuf->data[id], rbuf->len[id]);
+
 	/* expire report */
 	rbuf->ts[id] = 0;
 
-	upsdebug_hex(3, "Report[set]", rbuf->data[id], rbuf->len[id]);
-
 	return 0;
 }
 
@@ -214,24 +219,19 @@ static int set_item_buffered(reportbuf_t *rbuf, hid_dev_handle_t udev, HIDData_t
 static int file_report_buffer(reportbuf_t *rbuf, unsigned char *buf, int buflen)
 {
 	int id = buf[0];
-	
-	if (rbuf->len[id] < buflen) {
-		upsdebugx(1, "file_report_buffer: expected %d bytes, but got %d!", rbuf->len[id], buflen);
-		upsdebug_hex(3, "Report[err]", buf, buflen);
-		errno = ERANGE;
-		return -1;
-	}
-
-	if (buflen < rbuf->len[id]) {
-		upsdebugx(4, "file_report_buffer: expected %d bytes, but got only %d", rbuf->len[id], buflen);
-	}
 
+	/* broken report descriptors are common, so store whatever we can */
 	memcpy(rbuf->data[id], buf, rbuf->len[id]);
 
-	/* have valid (?) report */
-	time(&(rbuf->ts[id]));
+	if (rbuf->len[id] != buflen) {
+		upsdebugx(2, "%s: expected %d bytes, but got %d instead", __func__, rbuf->len[id], buflen);
+		upsdebug_hex(3, "Report[err]", buf, buflen);
+	} else {
+		upsdebug_hex(3, "Report[int]", rbuf->data[id], rbuf->len[id]);
+	}
 
-	upsdebug_hex(3, "Report[int]", rbuf->data[id], rbuf->len[id]);
+	/* have (valid) report */
+	time(&rbuf->ts[id]);
 
 	return 0;
 }
@@ -240,8 +240,10 @@ static int file_report_buffer(reportbuf_t *rbuf, unsigned char *buf, int buflen)
 
 /* Units and exponents table (HID PDC, 3.2.3) */
 #define NB_HID_UNITS 10
-static const long HIDUnits[NB_HID_UNITS][2]=
-{
+static struct {
+	const long	Type;
+	const int8_t	Expo;
+} HIDUnits[NB_HID_UNITS] = {
 	{ 0x00000000, 0 },	/* None */
 	{ 0x00F0D121, 7 },	/* Voltage */
 	{ 0x00100001, 0 },	/* Ampere */
@@ -263,6 +265,10 @@ static const long HIDUnits[NB_HID_UNITS][2]=
 void HIDDumpTree(hid_dev_handle_t udev, usage_tables_t *utab)
 {
 	int	i;
+#ifndef SHUT_MODE
+	/* extract the VendorId for further testing */
+	int vendorID = usb_device((struct usb_dev_handle *)udev)->descriptor.idVendor;
+#endif
 
 	/* Do not go further if we already know nothing will be displayed.
 	 * Some reports take a while before they timeout, so if these are
@@ -278,6 +284,19 @@ void HIDDumpTree(hid_dev_handle_t udev, usage_tables_t *utab)
 		double		value;
 		HIDData_t	*pData = &pDesc->item[i];
 
+		/* skip reports 254/255 for Eaton / MGE / Dell due to special handling needs */
+#ifdef SHUT_MODE
+		if ((pData->ReportID == 254) || (pData->ReportID == 255)) {
+			continue;
+		}
+#else
+		if ((vendorID == 0x0463) || (vendorID == 0x047c)) {
+			if ((pData->ReportID == 254) || (pData->ReportID == 255)) {
+				continue;
+			}
+		}
+#endif
+
 		/* Get data value */
 		if (HIDGetDataValue(udev, pData, &value, MAX_TS) == 1) {
 			upsdebugx(1, "Path: %s, Type: %s, ReportID: 0x%02x, Offset: %i, Size: %i, Value: %f",
@@ -350,7 +369,7 @@ int HIDGetDataValue(hid_dev_handle_t udev, HIDData_t *hiddata, double *Value, in
 
 	r = get_item_buffered(reportbuf, udev, hiddata, &hValue, age);
 	if (r<0) {
-		upsdebug_with_errno(1, "Can't retrieve Report %i", hiddata->ReportID);
+		upsdebug_with_errno(1, "Can't retrieve Report %02x", hiddata->ReportID);
 		return -errno;
 	}
 
@@ -360,7 +379,7 @@ int HIDGetDataValue(hid_dev_handle_t udev, HIDData_t *hiddata, double *Value, in
 	*Value = logical_to_physical(hiddata, hValue);
 	
 	/* Process exponents and units */
-	*Value *= (double)exponent(10, (int)hiddata->UnitExp - get_unit_expo(hiddata->Unit));
+	*Value *= exponent(10, get_unit_expo(hiddata));
 
 	return 1;
 }
@@ -410,21 +429,21 @@ int HIDSetDataValue(hid_dev_handle_t udev, HIDData_t *hiddata, double Value)
 	}
 
 	/* Test if Item is settable */
-        /* FIXME: not constant == volatile, but
-        * it doesn't imply that it's RW! */
+	/* FIXME: not constant == volatile, but
+	* it doesn't imply that it's RW! */
 	if (hiddata->Attribute == ATTR_DATA_CST) {
 		return 0;
 	}
 
 	/* Process exponents and units */
-	Value /= exponent(10, (int)hiddata->UnitExp - get_unit_expo(hiddata->Unit));
+	Value /= exponent(10, get_unit_expo(hiddata));
 	
 	/* Convert Physical Min, Max and Value into Logical */
 	hValue = physical_to_logical(hiddata, Value);
 
 	r = set_item_buffered(reportbuf, udev, hiddata, hValue);
 	if (r<0) {
-		upsdebug_with_errno(1, "Can't set Report %i", hiddata->ReportID);
+		upsdebug_with_errno(1, "Can't set Report %02x", hiddata->ReportID);
 		return -errno;
 	}
 
@@ -450,20 +469,20 @@ bool_t HIDSetItemValue(hid_dev_handle_t udev, const char *hidpath, double Value,
  */
 int HIDGetEvents(hid_dev_handle_t udev, HIDData_t **event, int eventsize)
 {
-	unsigned char	buf[128];
+	unsigned char	buf[SMALLBUF];
 	int		itemCount = 0;
 	int		buflen, r, i;
 	HIDData_t	*pData;
 
 	/* needs libusb-0.1.8 to work => use ifdef and autoconf */
-	if ((buflen = comm_driver->get_interrupt(udev, buf, sizeof(buf), 250)) <= 0) {
-
+	buflen = comm_driver->get_interrupt(udev, buf, sizeof(buf), 250);
+	if (buflen <= 0) {
 		return buflen;	/* propagate "error" or "no event" code */
 	}
 
 	r = file_report_buffer(reportbuf, buf, buflen);
 	if (r < 0) {
-		upsdebug_with_errno(1, "HIDGetEvents: failed to buffer report");
+		upsdebug_with_errno(1, "%s: failed to buffer report", __func__);
 		return -errno;
 	}
 
@@ -482,13 +501,17 @@ int HIDGetEvents(hid_dev_handle_t udev, HIDData_t **event, int eventsize)
 
 		/* maximum number of events reached? */
 		if (itemCount >= eventsize) {
-			upsdebugx(1, "HIDGetEvents: too many events (truncated)");
+			upsdebugx(1, "%s: too many events (truncated)", __func__);
 			break;
 		}
 
 		event[itemCount++] = pData;
 	}
 
+	if (itemCount == 0) {
+		upsdebugx(1, "%s: unexpected input report (ignored)", __func__);
+	}
+
 	return itemCount;
 }
 
@@ -571,25 +594,27 @@ static long physical_to_logical(HIDData_t *Data, double physical)
 	return logical;
 }
 
-static long get_unit_expo(const long UnitType)
+static int8_t get_unit_expo(const HIDData_t *hiddata)
 {
-	int i;
-	
-	for (i=0; i < NB_HID_UNITS; i++)
-	{
-		if (HIDUnits[i][0] == UnitType)
-		{
-			upsdebugx(5, "get_unit_expo: %08x found %ld", (unsigned int)UnitType, HIDUnits[i][1]);
-			return HIDUnits[i][1];
+	int	i;
+	int8_t	unit_expo = hiddata->UnitExp;
+
+	upsdebugx(5, "Unit = %08x, UnitExp = %d", (uint32_t)(hiddata->Unit), hiddata->UnitExp);
+
+	for (i = 0; i < NB_HID_UNITS; i++) {
+
+		if (HIDUnits[i].Type == hiddata->Unit) {
+			unit_expo -= HIDUnits[i].Expo;
+			break;
 		}
 	}
 
-	upsdebugx(5, "get_unit_expo: %08x not found!", (unsigned int)UnitType);
-	return -1;
+	upsdebugx(5, "Exponent = %d", unit_expo);
+	return unit_expo;
 }
 
 /* exponent function: return a^b */
-static double exponent(double a, int b)
+static double exponent(double a, int8_t b)
 {
 	if (b>0)
 		return (a * exponent(a, --b));		/* a * a ... */
@@ -608,7 +633,7 @@ static int string_to_path(const char *string, HIDPath_t *path, usage_tables_t *u
 	char	buf[SMALLBUF];
 	char	*token, *last; 
 	
-	snprintf(buf, sizeof(buf), string);
+	snprintf(buf, sizeof(buf), "%s", string);
 
 	for (token = strtok_r(buf, ".", &last); token != NULL; token = strtok_r(NULL, ".", &last))
 	{
@@ -619,13 +644,6 @@ static int string_to_path(const char *string, HIDPath_t *path, usage_tables_t *u
 			continue;
 		}
 
-		/* indexed collection */
-		if (strlen(token) == strspn(token, "[1234567890]"))
-		{
-			path->Node[i++] = 0x00ff0000 + atoi(token+1);
-			continue;
-		}
-
 		/* translate unnamed path components such as "ff860024" */
 		if (strlen(token) == strspn(token, "1234567890abcdefABCDEF"))
 		{
@@ -633,6 +651,13 @@ static int string_to_path(const char *string, HIDPath_t *path, usage_tables_t *u
 			continue;
 		}
 
+		/* indexed collection */
+		if (strlen(token) == strspn(token, "[1234567890]"))
+		{
+			path->Node[i++] = 0x00ff0000 + atoi(token+1);
+			continue;
+		}
+
 		/* Uh oh, typo in usage table? */
 		upsdebugx(1, "string_to_path: couldn't parse %s from %s", token, string);
 	}
@@ -794,7 +819,7 @@ usage_lkp_t hid_usage_lkp[] = {
 	{  "VoltageOutOfRange",			0x00840063 },
 	{  "FrequencyOutOfRange",		0x00840064 },
 	{  "Overload",				0x00840065 }, 
-        /* Note: the correct spelling is "Overload", not "OverLoad",
+	/* Note: the correct spelling is "Overload", not "OverLoad",
 	 * according to the official specification, "Universal Serial
 	 * Bus Usage Tables for HID Power Devices", Release 1.0,
 	 * November 1, 1997 */
