@@ -25,11 +25,12 @@
 #ifndef MEGATEC_H
 #define MEGATEC_H
 
-
-#define DRV_VERSION "1.5.14"
+/* FIXME (AQ): non compliant version (should be X.YZ)
+ * USB and serial should also be versioned */
 
 
 #ifdef MEGATEC_SUBDRV
+	extern upsdrv_info_t megatec_subdrv_info;
 	void megatec_subdrv_makevartable(void);
 	void megatec_subdrv_banner(void);
 #else
