@@ -23,12 +23,27 @@
 #include "megatec.h"
 #include "libusb.h"
 #include "serial.h"
+#include "usb-common.h"
 
 #include <stdio.h>
 #include <limits.h>
 #include <string.h>
 #include <stdlib.h>
 
+#define SUB_DRIVER_VERSION	"0.10"
+#define SUB_DRIVER_NAME	"Serial-over-USB transport layer"
+
+/* driver description structure */
+upsdrv_info_t megatec_subdrv_info = {
+	SUB_DRIVER_NAME,
+	SUB_DRIVER_VERSION,
+	"Andrey Lelikov <nut-driver@lelik.org>\n" \
+	"Alexander Gordeev <lasaine@lvk.cs.msu.su>\n" \
+	"Jon Gough <jon.gough@eclipsesystems.com.au>",
+	DRV_STABLE,
+	{ NULL }
+};
+
 /*
     This is a communication driver for "USB HID" UPS-es which use proprietary
 usb-to-serial converter and speak megatec protocol. Usually these are cheap
@@ -49,89 +64,103 @@ static USBDevice_t usbdevice;
 static USBDeviceMatcher_t *reopen_matcher = NULL;
 static USBDeviceMatcher_t *regex_matcher = NULL;
 
-typedef struct {
-	char	*name;
-	int	(*get_data) (char *buffer, int buffer_size);
-	int	(*set_data) (const char *str);
-} subdriver_t;
+static int (*get_data)(char *buffer, int buffer_size) = NULL;
+static int (*set_data)(const char *str) = NULL;
 
 /* agiler-old subdriver definition */
 static int get_data_agiler_old(char *buffer, int buffer_size);
 static int set_data_agiler_old(const char *str);
 
-static subdriver_t agiler_old_subdriver = {
-	"agiler-old",
-	get_data_agiler_old,
-	set_data_agiler_old
-};
+static void *agiler_old_subdriver(void)
+{
+	get_data = &get_data_agiler_old;
+	set_data = &set_data_agiler_old;
+	return NULL;
+}
 
 /* agiler subdriver definition */
 static int get_data_agiler(char *buffer, int buffer_size);
 static int set_data_agiler(const char *str);
 
-static subdriver_t agiler_subdriver = {
-	"agiler",
-	get_data_agiler,
-	set_data_agiler
-};
+static void *agiler_subdriver(void)
+{
+	get_data = &get_data_agiler;
+	set_data = &set_data_agiler;
+	return NULL;
+}
+
+/* Phoenixtec Power Co subdriver definition */
+static int get_data_phoenix(char *buffer, int buffer_size);
+static int set_data_phoenix(const char *str);
+
+static void *phoenix_subdriver(void)
+{
+	get_data = &get_data_phoenix;
+	set_data = &set_data_phoenix;
+	return NULL;
+}
 
 /* krauler (ablerex) subdriver definition */
 static int get_data_krauler(char *buffer, int buffer_size);
 static int set_data_krauler(const char *str);
 
-static subdriver_t krauler_subdriver = {
-	"krauler",
-	get_data_krauler,
-	set_data_krauler
+static void *krauler_subdriver(void)
+{
+	get_data = &get_data_krauler;
+	set_data = &set_data_krauler;
+	return NULL;
+}
+
+/* list of subdrivers for manual overrides */
+static const struct {
+	const char	*name;
+	void		*(*handler)(void);
+} subdriver[] = {
+	{ "agiler-old",	&agiler_old_subdriver },
+	{ "agiler",	&agiler_subdriver },
+	{ "phoenix",	&phoenix_subdriver },
+	{ "krauler",	&krauler_subdriver },
+	/* end of list */
+	{ NULL }
 };
 
-/* list of subdrivers */
-static subdriver_t *subdriver_list[] = {
-	&agiler_old_subdriver,
-	&agiler_subdriver,
-	&krauler_subdriver,
-	NULL	/* end of list */
-};
-
-/* selected subdriver */
-subdriver_t *subdriver = NULL;
-
-typedef struct {
-	int vid;
-	int pid;
-	subdriver_t *subdriver;
-} usb_ups_t;
-
 /* list of all known devices */
-static usb_ups_t KnownDevices[] = {
-	{0x05b8, 0x0000, &agiler_old_subdriver},	/* Agiler UPS */
-	{0x0001, 0x0000, &krauler_subdriver},		/* Krauler UP-M500VA */
-	{0xffff, 0x0000, &krauler_subdriver},		/* Ablerex 625L USB */
-	{0x0665, 0x5161, &agiler_old_subdriver},	/* Belkin F6C1200-UNV */
-	{0x06da, 0x0003, &agiler_old_subdriver},	/* Mustek Powermust */
-	{0x0f03, 0x0001, &agiler_old_subdriver},	/* Unitek Alpha 1200Sx */
-	{-1, -1, NULL}		/* end of list */
+static usb_device_id_t megatec_usb_id[] = {
+	/* Agiler UPS */
+	{ USB_DEVICE(0x05b8, 0x0000), &agiler_subdriver},
+	/* Krauler UP-M500VA */
+	{ USB_DEVICE(0x0001, 0x0000), &krauler_subdriver},
+	/* Ablerex 625L USB */
+	{ USB_DEVICE(0xffff, 0x0000), &krauler_subdriver},
+	/* Belkin F6C1200-UNV */
+	{ USB_DEVICE(0x0665, 0x5161), &phoenix_subdriver},
+	/* Mustek Powermust */
+	{ USB_DEVICE(0x06da, 0x0003), &phoenix_subdriver},
+	/* Unitek Alpha 1200Sx */
+	{ USB_DEVICE(0x0f03, 0x0001), &phoenix_subdriver},
+	/* end of list */
+	{-1, -1, NULL}
 };
 
 static int subdriver_match_func(USBDevice_t *d, void *privdata)
 {
-	usb_ups_t *p;
-
 	if(getval("subdriver"))
 		return 1;
 
-	for (p = KnownDevices; p->vid != -1; p++) {
-		if ((p->vid == d->VendorID) && (p->pid == d->ProductID)) {
-			subdriver = p->subdriver;
-			return 1;
-		}
-	}
+	switch (is_usb_device_supported(megatec_usb_id, d->VendorID, d->ProductID))
+	{
+	case SUPPORTED:
+		return 1;
 
-	return 0;
+	case POSSIBLY_SUPPORTED:
+	case NOT_SUPPORTED:
+	default:
+		return 0;
+	}
 }
 
 static USBDeviceMatcher_t subdriver_matcher = {
-	subdriver_match_func,
+	&subdriver_match_func,
 	NULL,
 	NULL
 };
@@ -149,12 +178,6 @@ static void usb_open_error(const char *port)
 		);
 }
 
-void megatec_subdrv_banner()
-{
-	printf("Serial-over-USB transport layer for Megatec protocol driver [%s]\n", progname);
-	printf("Andrey Lelikov (c) 2006, Alexander Gordeev (c) 2006-2007, Jon Gough (c) 2007\n\n");
-}
-
 /* FIXME: Fix "serial" variable (which conflicts with "serial" variable in megatec.c) */
 void megatec_subdrv_makevartable()
 {
@@ -181,7 +204,7 @@ int ser_open(const char *port)
 	/* pick up the subdriver name if set explicitly */
 	if(subdrv)
 	{
-		subdriver_t **p;
+		int	i;
 
 		if(!vid && !pid && !vend && !prod)
 		{
@@ -190,16 +213,16 @@ int ser_open(const char *port)
 				" variables.\n");
 		}
 
-		for (p = subdriver_list; *p; p++)
+		for (i = 0; subdriver[i].name; i++)
 		{
-			if (!strcasecmp(subdrv, (*p)->name))
+			if (!strcasecmp(subdrv, subdriver[i].name))
 			{
-				subdriver = *p;
+				(*subdriver[i].handler)();
 				break;
 			}
 		}
 
-		if(!subdriver)
+		if(!subdriver[i].name)
 			fatalx(EXIT_FAILURE, "No subdrivers named \"%s\" found!", subdrv);
 	}
 
@@ -261,7 +284,7 @@ int ser_flush_io(int fd)
 
 	/* flush input buffers */
 	for (i = 0; i < 10; i++) {
-		if (subdriver->get_data(flush_buf, sizeof(flush_buf)) < 1)
+		if ((*get_data)(flush_buf, sizeof(flush_buf)) < 1)
 			break;
 	}
 
@@ -319,20 +342,18 @@ void usb_comm_fail(int res, const char *msg)
 		case -EBUSY:
 			upslogx(LOG_WARNING, "%s: Device claimed by another process", msg);
 			fatalx(EXIT_FAILURE, "Terminating: EBUSY");
-			//upsdrv_cleanup();
-			break;
 
 		default:
 			upslogx(LOG_WARNING, "%s: Device detached? (error %d: %s)", msg, res, usb_strerror());
 
 			if(reconnect_ups()) {
-				//upsdrv_initinfo();
+				/* upsdrv_initinfo(); */
 			}
 			break;
 	}
 }
 
-unsigned int ser_send_pace(int fd, unsigned long d_usec, const char *fmt, ...)
+int ser_send_pace(int fd, unsigned long d_usec, const char *fmt, ...)
 {
 	char buf[128];
 	size_t len;
@@ -353,7 +374,7 @@ unsigned int ser_send_pace(int fd, unsigned long d_usec, const char *fmt, ...)
 		buf[sizeof(buf) - 1] = 0;
 	}
 
-	ret = subdriver->set_data(buf);
+	ret = (*set_data)(buf);
 	if(ret < 0) {
 		usb_comm_fail(ret, "ser_send_pace");
 	}
@@ -361,7 +382,7 @@ unsigned int ser_send_pace(int fd, unsigned long d_usec, const char *fmt, ...)
 	return ret;
 }
 
-int ser_get_line(int fd, char *buf, size_t buflen, char endchar, const char *ignset, long d_sec, long d_usec)
+int ser_get_line(int fd, void *buf, size_t buflen, char endchar, const char *ignset, long d_sec, long d_usec)
 {
 	int len;
 	char *src, *dst, c;
@@ -369,7 +390,7 @@ int ser_get_line(int fd, char *buf, size_t buflen, char endchar, const char *ign
 	if ((udev == NULL) && (! reconnect_ups()))
 		return -1;
 
-	len = subdriver->get_data((char *)buf, buflen);
+	len = (*get_data)((char *)buf, buflen);
 	if (len < 0) {
 		usb_comm_fail(len, "ser_get_line");
 		return len;
@@ -472,6 +493,62 @@ static int get_data_agiler(char *buffer, int buffer_size)
 }
 
 
+/*
+    Phoenixtec Power Co serial-to-usb device.
+*/
+static char	phoenix_buffer[32];
+
+static int set_data_phoenix(const char *str)
+{
+	unsigned int	count;
+
+	memset(phoenix_buffer, '\0', sizeof(phoenix_buffer));
+
+	snprintf(phoenix_buffer, sizeof(phoenix_buffer), "%s", str);
+
+	if (!strcmp(phoenix_buffer, "I\r") || !strcmp(phoenix_buffer, "C\r")) {
+		/* Ignore these, since they seem to lock up the connection */
+		return strlen(phoenix_buffer);
+	}
+
+	for (count = 0; count < strlen(phoenix_buffer); count += 8) {
+
+		if (usb->set_report(udev, 0, (unsigned char *)(phoenix_buffer + count), 8) < 1) {
+			return -1;
+		}
+	}
+
+	return strlen(phoenix_buffer);
+}
+
+static int get_data_phoenix(char *buffer, int buffer_size)
+{
+	int	count;
+
+	memset(buffer, '\0', buffer_size);
+
+	if (!strcmp(phoenix_buffer, "I\r") || !strcmp(phoenix_buffer, "C\r")) {
+		/* Echo back unsupported commands */
+		snprintf(buffer, buffer_size, "%s", phoenix_buffer);
+		return strlen(buffer);
+	}
+
+	for (count = 8; count <= buffer_size; count += 8) {
+
+		/* Read data in 8-byte chunks, break on a timeout */
+		if (usb->get_interrupt(udev, (unsigned char *)&buffer[count-8], 8, 1000) < 0) {
+			return count-8;
+		}
+
+		upsdebugx(3, "get_data_phoenix: got so far [%s]", buffer);
+		upsdebug_hex(4, "get_data_phoenix", (unsigned char *)buffer, count);
+	}
+
+	upsdebugx(3, "get_data_phoenix: buffer too small");
+	return -1;
+}
+
+
 /*
     Krauler serial-over-usb device.
 
