@@ -1,7 +1,7 @@
 /*  mge-hid.c - data to monitor MGE UPS SYSTEMS HID (USB and serial) devices
  *
- *  Copyright (C) 2003 - 2008
- *  			Arnaud Quette <arnaud.quette@mgeups.fr>
+ *  Copyright (C) 2003 - 2009
+ *  			Arnaud Quette <arnaud.quette@free.fr>
  *
  *  Sponsored by MGE UPS SYSTEMS <http://www.mgeups.com>
  *
@@ -22,26 +22,31 @@
  *
  */
 
+#include "main.h"		/* for getval() */
 #include "usbhid-ups.h"
 #include "mge-hid.h"
-#include "extstate.h"	/* for ST_FLAG_STRING */
-#include "main.h"		/* for getval() */
-#include "common.h"
 
-#define MGE_HID_VERSION		"MGE HID 1.12"
+#define MGE_HID_VERSION		"MGE HID 1.18"
+
+/* (prev. MGE Office Protection Systems, prev. MGE UPS SYSTEMS) */
+/* Eaton */
+#define MGE_VENDORID		0x0463
+
+/* Dell */
+#define DELL_VENDORID		0x047c
 
 #ifndef SHUT_MODE
 #include "usb-common.h"
 
-/* MGE Office Protection Systems, prev. MGE UPS Systems */
-#define MGE_VENDORID		0x0463
-
 /* USB IDs device table */
 static usb_device_id_t mge_usb_device_table[] = {
 	/* various models */
 	{ USB_DEVICE(MGE_VENDORID, 0x0001), NULL },
 	{ USB_DEVICE(MGE_VENDORID, 0xffff), NULL },
-	
+
+	/* various models */
+	{ USB_DEVICE(DELL_VENDORID, 0xffff), NULL },
+
 	/* Terminating entry */
 	{ -1, -1, NULL }
 };
@@ -62,11 +67,93 @@ typedef enum {
 	MGE_PULSAR_M = 0x200,		/* MGE Pulsar M series */
 		MGE_PULSAR_M_2200,
 		MGE_PULSAR_M_3000,
-		MGE_PULSAR_M_3000_XL
+		MGE_PULSAR_M_3000_XL,
+	MGE_PEGASUS = 0x400
 } models_type_t;
 
 static models_type_t	mge_type = MGE_DEFAULT;
-static char		mge_scratch_buf[16];
+static char		mge_scratch_buf[20];
+
+/* The HID path 'UPS.PowerSummary.Time' reports Unix time (ie the number of
+ * seconds since 1970-01-01 00:00:00. This has to be split between ups.date and
+ * ups.time */
+static char *mge_date_conversion_fun(double value)
+{
+	time_t	sec = value;
+
+	if (strftime(mge_scratch_buf, sizeof(mge_scratch_buf), "%Y/%m/%d", localtime(&sec)) == 10) {
+		return mge_scratch_buf;
+	}
+
+	upsdebugx(3, "%s: can't compute date %g", __func__, value);
+	return NULL;
+}
+
+static char *mge_time_conversion_fun(double value)
+{
+	time_t sec = value;
+
+	if (strftime(mge_scratch_buf, sizeof(mge_scratch_buf), "%H:%M:%S", localtime(&sec)) == 8) {
+		return mge_scratch_buf;
+	}
+
+	upsdebugx(3, "%s: can't compute time %g", __func__, value);
+	return NULL;
+}
+
+#ifdef HAVE_STRPTIME
+/* Conversion back retrieve ups.time to build the full unix time */
+static double mge_date_conversion_nuf(const char *value)
+{
+	struct tm	mge_tm;
+
+	/* build a full value (date) + time string */
+	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%s %s", value, dstate_getinfo("ups.time"));
+
+	if (strptime(mge_scratch_buf, "%Y/%m/%d %H:%M:%S", &mge_tm) != NULL) {
+		/* Ignore DST offset */
+		mge_tm.tm_isdst = 0;
+		return mktime(&mge_tm);
+	}
+
+	upsdebugx(3, "%s: can't compute date %s", __func__, value);
+	return 0;
+}
+
+/* Conversion back retrieve ups.date to build the full unix time */
+static double mge_time_conversion_nuf(const char *value)
+{
+	struct tm	mge_tm;
+
+	/* build a full date + value (time) string */
+	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%s %s", dstate_getinfo("ups.date"), value);
+
+	if (strptime(mge_scratch_buf, "%Y/%m/%d %H:%M:%S", &mge_tm) != NULL) {
+		/* Ignore DST offset */
+		mge_tm.tm_isdst = 0;
+		return mktime(&mge_tm);
+	}
+
+	upsdebugx(3, "%s: can't compute time %s", __func__, value);
+	return 0;
+}
+
+static info_lkp_t mge_date_conversion[] = {
+	{ 0, NULL, mge_date_conversion_fun, mge_date_conversion_nuf }
+};
+
+static info_lkp_t mge_time_conversion[] = {
+	{ 0, NULL, mge_time_conversion_fun, mge_time_conversion_nuf }
+};
+#else
+static info_lkp_t mge_date_conversion[] = {
+	{ 0, NULL, mge_date_conversion_fun, NULL }
+};
+
+static info_lkp_t mge_time_conversion[] = {
+	{ 0, NULL, mge_time_conversion_fun, NULL }
+};
+#endif /* HAVE_STRPTIME */
 
 /* The HID path 'UPS.PowerSummary.ConfigVoltage' only reports
    'battery.voltage.nominal' for specific UPS series. Ignore
@@ -160,11 +247,140 @@ static info_lkp_t mge_emergency_stop[] = {
 	{ 0, NULL, NULL }
 };
 
-/* FIXME: limit to ups.model ~= Protection Station... */
+static info_lkp_t mge_wiring_fault[] = {
+	{ 1, "Wiring fault!", NULL },
+	{ 0, NULL, NULL }
+};
+
+static info_lkp_t mge_config_failure[] = {
+	{ 1, "Fatal EEPROM fault!", NULL },
+	{ 0, NULL, NULL }
+};
+
+static info_lkp_t mge_inverter_volthi[] = {
+	{ 1, "Inverter AC voltage too high!", NULL },
+	{ 0, NULL, NULL }
+};
+
+static info_lkp_t mge_inverter_voltlo[] = {
+	{ 1, "Inverter AC voltage too low!", NULL },
+	{ 0, NULL, NULL }
+};
+
+static info_lkp_t mge_short_circuit[] = {
+	{ 1, "Output short circuit!", NULL },
+	{ 0, NULL, NULL }
+};
+
+info_lkp_t mge_onbatt_info[] = {
+	{ 1, "!online", NULL },
+	{ 0, "online", NULL },
+	{ 0, NULL, NULL }
+};
+/* allow limiting to ups.model ~= Protection Station */
+static char *eaton_check_pegasus_fun(double value)
+{
+	switch (mge_type & 0xFF00)	/* Ignore model byte */
+	{
+	case MGE_PEGASUS:
+		break;
+
+	default:
+		return NULL;
+	}
+
+	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.0f", value);
+	return mge_scratch_buf;
+}
+
 static info_lkp_t pegasus_threshold_info[] = {
-	{ 10, "10", NULL },
-	{ 25, "25", NULL },
-	{ 60, "60", NULL },
+	{ 10, "10", eaton_check_pegasus_fun },
+	{ 25, "25", eaton_check_pegasus_fun },
+	{ 60, "60", eaton_check_pegasus_fun },
+	{ 0, NULL, NULL }
+};
+
+/* Limit nominal output voltage according to HV or LV models */
+static char *nominal_output_voltage_fun(double value)
+{
+	static long	nominal = -1;
+
+	if (nominal < 0) {
+		nominal = value;
+	}
+
+	switch ((long)nominal)
+	{
+	/* LV models */
+	case 100:
+	case 110:
+	case 120:
+	case 127:
+		switch ((long)value)
+		{
+		case 100:
+		case 110:
+		case 120:
+		case 127:
+			break;
+		default:
+			return NULL;
+		}
+		break;
+
+	/* HV models */
+	/* 208V */
+	case 200:
+	case 208:
+		switch ((long)value)
+		{
+		case 200:
+		case 208:
+			break;
+		default:
+			return NULL;
+		}
+		break;
+
+	/* HV models */
+	/* 230V */
+	case 220:
+	case 230:
+	case 240:
+		switch ((long)value)
+		{
+		case 220:
+		case 230:
+		case 240:
+			break;
+		default:
+			return NULL;
+		}
+		break;
+
+	default:
+		upsdebugx(3, "%s: can't autodetect settable voltages from %g", __func__, value);
+	}
+
+	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.0f", value);
+	return mge_scratch_buf;
+}
+
+static info_lkp_t nominal_output_voltage_info[] = {
+	/* HV models */
+	/* 208V */
+	{ 200, "200", nominal_output_voltage_fun },
+	{ 208, "208", nominal_output_voltage_fun },
+	/* HV models */
+	/* 230V */
+	{ 220, "220", nominal_output_voltage_fun },
+	{ 230, "230", nominal_output_voltage_fun },
+	{ 240, "240", nominal_output_voltage_fun },
+	/* LV models */
+	{ 100, "100", nominal_output_voltage_fun },
+	{ 110, "110", nominal_output_voltage_fun },
+	{ 120, "120", nominal_output_voltage_fun },
+	{ 127, "127", nominal_output_voltage_fun },
 	{ 0, NULL, NULL }
 };
 
@@ -177,7 +393,9 @@ static usage_lkp_t mge_usage_lkp[] = {
 	{ "Undefined",				0xffff0000 },
 	{ "STS",				0xffff0001 },
 	{ "Environment",			0xffff0002 },
-	/* 0xffff0003-0xffff000f	=>	Reserved */
+	{ "Statistic",			0xffff0003 },
+	{ "StatisticSystem",		0xffff0004 },
+	/* 0xffff0005-0xffff000f	=>	Reserved */
 	{ "Phase",				0xffff0010 },
 	{ "PhaseID",				0xffff0011 },
 	{ "Chopper",				0xffff0012 },
@@ -206,7 +424,9 @@ static usage_lkp_t mge_usage_lkp[] = {
 	{ "RedundancyLevel",			0xffff0029 },
 	{ "RedundancyLost",			0xffff002a },
 	{ "NotificationStatus",			0xffff002b },
-	/* 0xffff002c-0xffff003f	=>	Reserved */
+	{ "ProtectionLost",			0xffff002c },
+	{ "ConfigurationFailure",			0xffff002d },
+	/* 0xffff002e-0xffff003f	=>	Reserved */
 	{ "SwitchType",				0xffff0040 },
 	{ "ConverterType",			0xffff0041 },
 	{ "FrequencyConverterMode",		0xffff0042 },
@@ -264,7 +484,13 @@ static usage_lkp_t mge_usage_lkp[] = {
 	{ "DCBusUnbalanced", 			0xffff0076 },
 	{ "FanFailure", 			0xffff0077 },
 	{ "WiringFault", 			0xffff0078 },
-	/* 0xffff0079-0xffff007f	=>	Reserved */
+	{ "Floating", 			0xffff0079 },
+	{ "OverCurrent", 			0xffff007a },
+	{ "RemainingActivePower", 			0xffff007b },
+	{ "Energy", 			0xffff007c },
+	{ "Threshold", 			0xffff007d },
+	{ "OverThreshold", 			0xffff007e },
+	/* 0xffff007f	=>	Reserved */
 	{ "Sensor",				0xffff0080 },
 	{ "LowHumidity",			0xffff0081 },
 	{ "HighHumidity",			0xffff0082 },
@@ -281,12 +507,25 @@ static usage_lkp_t mge_usage_lkp[] = {
 	{ "Time",				0xffff0097 },
 	{ "Code",				0xffff0098 },
 	{ "DataValid",				0xffff0099 },
-	/* 0xffff009a-0xffff00df	=>	Reserved */
+	{ "ToggleTimer",				0xffff009a },
+	/* 0xffff009b-0xffff009f	=>	Reserved */
+	{ "PDU",				0xffff00a0 },
+	{ "Breaker",				0xffff00a1 },
+	{ "BreakerID",				0xffff00a2 },
+	{ "OverVoltage",				0xffff00a3 },
+	{ "Tripped",				0xffff00a4 },
+	{ "OverEnergy",				0xffff00a5 },
+	{ "OverHumidity",				0xffff00a6 },
+	{ "LCDControl",				0xffff00a6 },
+	/* 0xffff00a8-0xffff00df	=>	Reserved */
 	{ "COPIBridge",				0xffff00e0 },
 	/* 0xffff00e1-0xffff00ef	=>	Reserved */
 	{ "iModel",				0xffff00f0 },
 	{ "iVersion",				0xffff00f1 },
-	/* 0xffff00f2-0xffff00ff	=>	Reserved */
+	{ "iTechnicalLevel",		0xffff00f2 },
+	{ "iPartNumber",			0xffff00f3 },
+	{ "iReferenceNumber",		0xffff00f4 },
+	/* 0xffff00f5-0xffff00ff	=>	Reserved */
 
 	/* end of table */
 	{ NULL, 0 }
@@ -347,10 +586,10 @@ static models_name_t mge_model_names [] =
 	{ "PROTECTIONCENTER", "500", MGE_DEFAULT, "Protection Center 500" },
 	{ "PROTECTIONCENTER", "675", MGE_DEFAULT, "Protection Center 675" },
 
-	/* Protection Station (TBR) */
-/*	{ "Protection Station", "500", MGE_DEFAULT, NULL }, */
-/*	{ "Protection Station", "650", MGE_DEFAULT, NULL }, */
-/*	{ "Protection Station", "800", MGE_DEFAULT, NULL }, */
+	/* Protection Station */
+	{ "Protection Station", "500", MGE_PEGASUS, NULL },
+	{ "Protection Station", "650", MGE_PEGASUS, NULL },
+	{ "Protection Station", "800", MGE_PEGASUS, NULL },
 
 	/* Evolution models */
 	{ "Evolution", "500", MGE_DEFAULT, "Pulsar Evolution 500" },
@@ -454,6 +693,7 @@ static hid_info_t mge_hid2nut[] =
 	{ "battery.charge.restart", ST_FLAG_RW | ST_FLAG_STRING, 3, "UPS.PowerSummary.RestartLevel", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "battery.capacity", 0, 0, "UPS.BatterySystem.Battery.DesignCapacity", NULL, "%s", HU_FLAG_STATIC, mge_battery_capacity },	/* conversion needed from As to Ah */
 	{ "battery.runtime", 0, 0, "UPS.PowerSummary.RunTimeToEmpty", NULL, "%.0f", 0, NULL },
+	{ "battery.runtime.elapsed", 0, 0, "UPS.StatisticSystem.Input.[1].Statistic.[1].Time", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL },
 	{ "battery.temperature", 0, 0, "UPS.BatterySystem.Battery.Temperature", NULL, "%.1f", 0, NULL },
 	{ "battery.type", 0, 0, "UPS.PowerSummary.iDeviceChemistry", NULL, "%s", HU_FLAG_STATIC, stringid_conversion },
 	{ "battery.voltage", 0, 0, "UPS.BatterySystem.Voltage", NULL, "%.1f", 0, NULL },
@@ -489,35 +729,60 @@ static hid_info_t mge_hid2nut[] =
 	{ "ups.start.auto", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Input.[1].AutomaticRestart", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
 	{ "ups.start.battery", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Input.[3].StartOnBattery", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
 	{ "ups.start.reboot", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.ForcedReboot", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
+#ifdef HAVE_STRPTIME
+	{ "ups.date", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.PowerSummary.Time", NULL, "%s", 0, mge_date_conversion },
+	{ "ups.time", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.PowerSummary.Time", NULL, "%s", 0, mge_time_conversion },
+#else
+	{ "ups.date", 0, 0, "UPS.PowerSummary.Time", NULL, "%s", 0, mge_date_conversion },
+	{ "ups.time", 0, 0, "UPS.PowerSummary.Time", NULL, "%s", 0, mge_time_conversion },
+#endif /* HAVE_STRPTIME */
 	{ "ups.type", 0, 0, "UPS.PowerConverter.ConverterType", NULL, "%s", HU_FLAG_STATIC, mge_upstype_conversion },
 
 	/* Special case: boolean values that are mapped to ups.status and ups.alarm */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ACPresent", NULL, NULL, HU_FLAG_QUICK_POLL, online_info },
+	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[3].PresentStatus.Used", NULL, NULL, 0, mge_onbatt_info },
+	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.Used", NULL, NULL, 0, online_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Discharging", NULL, NULL, HU_FLAG_QUICK_POLL, discharging_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Charging", NULL, NULL, HU_FLAG_QUICK_POLL, charging_info },
+	/* FIXME: on Dell, the above requires an "AND" with "UPS.BatterySystem.Charger.Mode = 1" */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.BelowRemainingCapacityLimit", NULL, NULL, HU_FLAG_QUICK_POLL, lowbatt_info },
+	/* Output overload, Level 1 (FIXME: add the level?) */
+	{ "BOOL", 0, 0, "UPS.PowerConverter.Output.Overload.[1].PresentStatus.OverThreshold", NULL, NULL, 0, overload_info },
+	/* Output overload, Level 2 (FIXME: add the level?) */
+	{ "BOOL", 0, 0, "UPS.PowerConverter.Output.Overload.[2].PresentStatus.OverThreshold", NULL, NULL, 0, overload_info },
+	/* Output overload, Level 3 (FIXME: add the level?) */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Overload", NULL, NULL, 0, overload_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.NeedReplacement", NULL, NULL, 0, replacebatt_info },
+	/* FIXME: on Dell, the above requires an "AND" with "UPS.BatterySystem.Battery.Test = 3 " */
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.Buck", NULL, NULL, 0, trim_info },
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.Boost", NULL, NULL, 0, boost_info },
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.VoltageOutOfRange", NULL, NULL, 0, vrange_info },
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.FrequencyOutOfRange", NULL, NULL, 0, frange_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Good", NULL, NULL, 0, off_info },
-	/* { "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.Used", NULL, NULL, 0, online_info }, */
+	{ "BOOL", 0, 0, "UPS.BatterySystem.Charger.PresentStatus.Used", NULL, NULL, 0, off_info },
+	/* FIXME: on Dell, the above requires an "AND" with "UPS.BatterySystem.Charger.Mode = 4 (ABM Resting)" */
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[2].PresentStatus.Used", NULL, NULL, 0, bypass_auto_info }, /* Automatic bypass */
-	/* { "BOOL", 0, 0, "UPS.PowerConverter.Input.[3].PresentStatus.Used", NULL, NULL, 0, onbatt_info }, */
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[4].PresentStatus.Used", NULL, NULL, 0, bypass_manual_info }, /* Manual bypass */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.FanFailure", NULL, NULL, 0, fanfail_info },
 	{ "BOOL", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.Present", NULL, NULL, 0, nobattery_info },
 	{ "BOOL", 0, 0, "UPS.BatterySystem.Charger.PresentStatus.InternalFailure", NULL, NULL, 0, chargerfail_info },
 	{ "BOOL", 0, 0, "UPS.BatterySystem.Charger.PresentStatus.VoltageTooHigh", NULL, NULL, 0, battvolthi_info },
+	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.VoltageTooHigh", NULL, NULL, 0, battvolthi_info },
+	/* Battery DC voltage too high! */
+	{ "BOOL", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.VoltageTooHigh", NULL, NULL, 0, battvolthi_info },
 	{ "BOOL", 0, 0, "UPS.BatterySystem.Charger.PresentStatus.VoltageTooLow", NULL, NULL, 0, battvoltlo_info },
+	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.VoltageTooLow", NULL, NULL, 0, battvoltlo_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.InternalFailure", NULL, NULL, 0, commfault_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.OverTemperature", NULL, NULL, 0, overheat_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ShutdownImminent", NULL, NULL, 0, shutdownimm_info },
 
 	/* Vendor specific ups.alarm */
 	{ "ups.alarm", 0, 0, "UPS.PowerSummary.PresentStatus.EmergencyStop", NULL, NULL, 0, mge_emergency_stop },
+	{ "ups.alarm", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.WiringFault", NULL, NULL, 0, mge_wiring_fault },
+	{ "ups.alarm", 0, 0, "UPS.PowerSummary.PresentStatus.ConfigurationFailure", NULL, NULL, 0, mge_config_failure },
+	{ "ups.alarm", 0, 0, "UPS.PowerConverter.Inverter.PresentStatus.VoltageTooHigh", NULL, NULL, 0, mge_inverter_volthi },
+	{ "ups.alarm", 0, 0, "UPS.PowerConverter.Inverter.PresentStatus.VoltageTooLow", NULL, NULL, 0, mge_inverter_voltlo },
+	{ "ups.alarm", 0, 0, "UPS.PowerConverter.Output.PresentStatus.ShortCircuit", NULL, NULL, 0, mge_short_circuit },
 
 	/* Input page */
 	{ "input.voltage", 0, 0, "UPS.PowerConverter.Input.[1].Voltage", NULL, "%.1f", 0, NULL },
@@ -572,13 +837,14 @@ static hid_info_t mge_hid2nut[] =
 	{ "output.L1-L2.voltage", 0, 0, "UPS.PowerConverter.Output.Phase.[12].Voltage", NULL, "%.1f", 0, NULL },
 	{ "output.L2-L3.voltage", 0, 0, "UPS.PowerConverter.Output.Phase.[23].Voltage", NULL, "%.1f", 0, NULL },
 	{ "output.L3-L1.voltage", 0, 0, "UPS.PowerConverter.Output.Phase.[31].Voltage", NULL, "%.1f", 0, NULL },
-	{ "output.voltage.nominal", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.Flow.[4].ConfigVoltage", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
+	{ "output.voltage.nominal", ST_FLAG_RW | ST_FLAG_STRING, 3, "UPS.Flow.[4].ConfigVoltage", NULL, "%.0f", HU_FLAG_SEMI_STATIC | HU_FLAG_ENUM, nominal_output_voltage_info },
 	{ "output.current", 0, 0, "UPS.PowerConverter.Output.Current", NULL, "%.2f", 0, NULL },
 	{ "output.L1.current", 0, 0, "UPS.PowerConverter.Output.Phase.[1].Current", NULL, "%.1f", 0, NULL },
 	{ "output.L2.current", 0, 0, "UPS.PowerConverter.Output.Phase.[2].Current", NULL, "%.1f", 0, NULL },
 	{ "output.L3.current", 0, 0, "UPS.PowerConverter.Output.Phase.[3].Current", NULL, "%.1f", 0, NULL },
 	{ "output.current.nominal", 0, 0, "UPS.Flow.[4].ConfigCurrent", NULL, "%.2f", 0, NULL },
 	{ "output.frequency", 0, 0, "UPS.PowerConverter.Output.Frequency", NULL, "%.1f", 0, NULL },
+	/* FIXME: can be RW (50/60) (or on .nominal)? */
 	{ "output.frequency.nominal", 0, 0, "UPS.Flow.[4].ConfigFrequency", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "output.powerfactor", 0, 0, "UPS.PowerConverter.Output.PowerFactor", NULL, "%s", 0, mge_powerfactor_conversion },
 
@@ -586,10 +852,10 @@ static hid_info_t mge_hid2nut[] =
 	{ "outlet.id", 0, 0, "UPS.OutletSystem.Outlet.[1].OutletID", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "outlet.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[1].OutletID", NULL, "Main Outlet", HU_FLAG_ABSENT, NULL },
 	{ "outlet.switchable", 0, 0, "UPS.OutletSystem.Outlet.[1].PresentStatus.Switchable", NULL, "%s", HU_FLAG_STATIC, yes_no_info },
-	/* The line below is the power consumption threshold on the master outlet
-	 * used as criteria to manage the slave outlets. Values: 10, 25 or 60 VA.
-	 * The default value is 25. */
-	{ "outlet.power", ST_FLAG_RW | ST_FLAG_STRING, 3, "UPS.OutletSystem.Outlet.[1].ConfigApparentPower", NULL, "%s", HU_FLAG_SEMI_STATIC, pegasus_threshold_info },
+	/* On Protection Station, the line below is the power consumption threshold
+	 * on the master outlet used to automatically power off the slave outlets.
+	 * Values: 10, 25 (default) or 60 VA. */
+	{ "outlet.power", ST_FLAG_RW | ST_FLAG_STRING, 6, "UPS.OutletSystem.Outlet.[1].ConfigApparentPower", NULL, "%s", HU_FLAG_SEMI_STATIC | HU_FLAG_ENUM, pegasus_threshold_info },
 	{ "outlet.1.id", 0, 0, "UPS.OutletSystem.Outlet.[2].OutletID", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "outlet.1.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[2].OutletID", NULL, "PowerShare Outlet 1", HU_FLAG_ABSENT, NULL },
 	{ "outlet.1.switchable", 0, 0, "UPS.OutletSystem.Outlet.[2].PresentStatus.Switchable", NULL, "%s", HU_FLAG_STATIC, yes_no_info },
@@ -680,6 +946,11 @@ static char *mge_format_model(HIDDevice_t *hd) {
 	char	model[SMALLBUF];
 	double	value;
 
+	/* Dell has already a fully formatted name in iProduct */
+	if (hd->VendorID == DELL_VENDORID) {
+		return hd->Product;
+	}
+
 	/* Get iProduct and iModel strings */
 	snprintf(product, sizeof(product), "%s", hd->Product ? hd->Product : "unknown");
 
@@ -699,7 +970,7 @@ static char *mge_format_model(HIDDevice_t *hd) {
 }
 
 static char *mge_format_mfr(HIDDevice_t *hd) {
-	return hd->Vendor ? hd->Vendor : "MGE UPS SYSTEMS";
+	return hd->Vendor ? hd->Vendor : "Eaton";
 }
 
 static char *mge_format_serial(HIDDevice_t *hd) {
@@ -721,7 +992,7 @@ static int mge_claim(HIDDevice_t *hd) {
 			if (getval("productid")) {
 				return 1;
 			}
-			possibly_supported("MGE", hd);
+			possibly_supported("Eaton / MGE", hd);
 			return 0;
 
 		case SUPPORTED:
