@@ -1,5 +1,5 @@
 /*  mge-shut.c - monitor MGE UPS for NUT with SHUT protocol
- * 
+ *
  *  Copyright (C) 2002 - 2008
  *     Arnaud Quette <arnaud.quette@gmail.com>
  *
@@ -36,8 +36,8 @@
 /*                  Define "technical" constants                   */
 /* --------------------------------------------------------------- */
 
-#define DRIVER_NAME	"MGE UPS SYSTEMS/SHUT driver"
-#define DRIVER_VERSION	"0.68"
+#define DRIVER_NAME	"Eaton / SHUT driver"
+#define DRIVER_VERSION	"0.69"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -52,7 +52,7 @@ upsdrv_info_t upsdrv_info = {
 
 /* global variables */
 int commstatus = 0;
-int lowbatt = DEFAULT_LOWBATT;
+int lowbatt = -1;
 int ondelay = DEFAULT_ONDELAY;
 int offdelay = DEFAULT_OFFDELAY;
 int notification = DEFAULT_NOTIFICATION;
@@ -100,9 +100,9 @@ static char *hu_find_infoval(info_lkp_t *hid2info, long value);
 void upsdrv_initinfo (void)
 {
 	mge_info_item_t *item;
-	
+
 	upsdebugx(2, "entering initinfo()\n");
-	
+
 	/* Get complete Model information */
 	shut_identify_ups ();
 
@@ -111,7 +111,7 @@ void upsdrv_initinfo (void)
 
 	/* Device capabilities enumeration ----------------------------- */
 	for ( item = mge_info ; item->type != NULL ; item++ ) {
-		
+
 		/* Check if we are asked to stop (reactivity++) */
 		if (exit_flag != 0)
 		  return;
@@ -119,7 +119,7 @@ void upsdrv_initinfo (void)
 		/* avoid redundancy when multiple defines (RO/RW) */
 		if (dstate_getinfo(item->type) != NULL)
 			continue;
-		
+
 		/* Special case for handling server side variables */
 		if (item->shut_flags & SHUT_FLAG_ABSENT) {
 			/* Check if exists (if necessary) before creation */
@@ -135,15 +135,15 @@ void upsdrv_initinfo (void)
 			  dstate_setflags(item->type, item->flags);
 			  continue;
 			}
-			
+
 			dstate_setinfo(item->type, "%s", item->dfl);
 			dstate_setflags(item->type, item->flags);
 
 			/* Set max length for strings, if needed */
 			if (item->flags & ST_FLAG_STRING)
 				dstate_setaux(item->type, item->length);
-			
-			/* disable reading now 
+
+			/* disable reading now
 			item->shut_flags &= ~SHUT_FLAG_OK;*/
 		} else {
 			if (hid_get_value(item->item_path) != 0 ) {
@@ -172,7 +172,11 @@ void upsdrv_initinfo (void)
 	/* install handlers */
 	upsh.setvar = hid_set_value; /* setvar; */
 	upsh.instcmd = instcmd;
-	
+
+	/* check if low battery level has been given to set it */
+	if (lowbatt != -1) {
+		hid_set_value("battery.charge.low", getval ("lowbatt"));
+	}
 }
 
 /* --------------------------------------------------------------- */
@@ -181,9 +185,9 @@ void upsdrv_updateinfo (void)
 {
 	mge_info_item_t *item;
 	char *nutvalue;
-	
+
 	upsdebugx(2, "entering upsdrv_updateinfo()");
-	
+
 	if (commstatus == 0) {
 		if (shut_ups_start () != 0) {
 			upsdebugx(2, "No communication with UPS, retrying");
@@ -208,11 +212,11 @@ void upsdrv_updateinfo (void)
 
     if (item->shut_flags & SHUT_FLAG_OK) {
 
-			if(hid_get_value(item->item_path) != 0 ) {				
+			if(hid_get_value(item->item_path) != 0 ) {
 				upsdebugx(3, "%s: hValue = %ld",	item->item_path, hValue);
 				/* upsdebugx(3, "%s: hValue = %ld (%ld)",
 					item->item_path, hValue, hData.LogMax); */
-				
+
 				/* need lookup'ed translation */
 				if (item->hid2info != NULL)
 				  {
@@ -243,12 +247,12 @@ void upsdrv_shutdown (void)
 	if (sdtype == SD_RETURN) {
 		/* set DelayBeforeStartup */
 		snprintf(val, sizeof(val), "%d", ondelay);
-		hid_set_value("ups.delay.start", val);
+		hid_set_value("ups.timer.start", val);
 	}
 
 	/* set DelayBeforeShutdown */
 	snprintf(val, sizeof(val), "%d", offdelay);
-	hid_set_value("ups.delay.shutdown", val);
+	hid_set_value("ups.timer.shutdown", val);
 }
 
 /* --------------------------------------------------------------- */
@@ -264,13 +268,13 @@ void upsdrv_help (void)
 void upsdrv_makevartable (void)
 {
 	char msg[MAX_STRING];
-	
+
 	upsdebugx (2, "entering upsdrv_makevartable()");
-  
+
 	snprintf(msg, sizeof(msg), "Set low battery level, in %% (default=%d).",
 		DEFAULT_LOWBATT);
 	addvar (VAR_VALUE, "lowbatt", msg);
-	
+
 	snprintf(msg, sizeof(msg), "Set shutdown delay, in seconds (default=%d).",
 		DEFAULT_OFFDELAY);
 	addvar (VAR_VALUE, "offdelay", msg);
@@ -278,7 +282,7 @@ void upsdrv_makevartable (void)
 	snprintf(msg, sizeof(msg), "Set startup delay, in ten seconds units (default=%d).",
 		DEFAULT_ONDELAY);
 	addvar (VAR_VALUE, "ondelay", msg);
-	
+
 	snprintf(msg, sizeof(msg), "Set notification type, 1 = no, 2 = light, 3 = yes (default=%d).",
 		DEFAULT_NOTIFICATION);
 	addvar (VAR_VALUE, "notification", msg);
@@ -294,15 +298,15 @@ void upsdrv_initups (void)
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B2400);
 	setline (1);
-  
+
 	/* get battery lowlevel */
 	if (getval ("lowbatt"))
 		lowbatt = atoi (getval ("lowbatt"));
-  
+
 	/* on delay */
 	if (getval ("ondelay"))
 		ondelay = atoi (getval ("ondelay"));
-	
+
 	/* shutdown delay */
 	if (getval ("offdelay"))
 		offdelay = atoi (getval ("offdelay"));
@@ -339,7 +343,7 @@ int instcmd(const char *cmdname, const char *extra)
 		upsdrv_shutdown();
 		return STAT_INSTCMD_HANDLED;
 	}
-	
+
 	/* Shutdown UPS and stay off when power is restored */
 	if (!strcasecmp(cmdname, "shutdown.stayoff")) {
 		sdtype = SD_STAYOFF;
@@ -350,14 +354,14 @@ int instcmd(const char *cmdname, const char *extra)
 	/* Power off the load immediatly */
 	if (!strcasecmp(cmdname, "load.off")) {
 		/* set DelayBeforeShutdown to 0 */
-		hid_set_value("ups.delay.shutdown", "0");
+		hid_set_value("ups.timer.shutdown", "0");
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	/* Power on the load immediatly */
 	if (!strcasecmp(cmdname, "load.on")) {
 		/* set DelayBeforeStartup to 0 */
-		hid_set_value("ups.delay.start", "0");
+		hid_set_value("ups.timer.start", "0");
 		return STAT_INSTCMD_HANDLED;
 	}
 
@@ -367,7 +371,7 @@ int instcmd(const char *cmdname, const char *extra)
 		hid_set_value("ups.test.result", "1");
 		return STAT_INSTCMD_HANDLED;
 	}
-	
+
 	/* Stop battery test */
 	if (!strcasecmp(cmdname, "test.battery.stop")) {
 		/* set Test to 3 (Abort test) */
@@ -382,7 +386,7 @@ int instcmd(const char *cmdname, const char *extra)
 
 /*****************************************************************************
  * shut_ups_start ()
- * 
+ *
  * initiate communication with the UPS
  *
  * return 0 on success, -1 on failure
@@ -392,7 +396,7 @@ int shut_ups_start ()
 {
 	u_char c = SHUT_SYNC, r[1];
 	int try;
-	
+
 	upsdebugx (2, "entering shut_ups_start()\n");
 	r[0] = '\0';
 
@@ -428,7 +432,7 @@ int shut_ups_start ()
 
 /**********************************************************************
  * shut_identify_ups ()
- * 
+ *
  * Get SHUT device complete name
  *
  * return 0 on success, -1 on failure
@@ -440,11 +444,11 @@ int shut_identify_ups ()
 	char model[MAX_STRING];
 	char *finalname = NULL;
 	int retcode, tries=MAX_TRY;
-	
+
 	if (commstatus == 0)
 		return -1;
-  
-	upsdebugx (2, "entering shut_identify_ups(0x%04x, 0x%04x)\n", 
+
+	upsdebugx (2, "entering shut_identify_ups(0x%04x, 0x%04x)\n",
 				device_descriptor.dev_desc.iManufacturer,
 				device_descriptor.dev_desc.iProduct);
 
@@ -454,7 +458,7 @@ int shut_identify_ups ()
 		if (shut_get_string(device_descriptor.dev_desc.iProduct, string, 0x25) > 0)
 		{
 			strcpy(model, string);
-			
+
 			if(hid_get_value("UPS.PowerSummary.iModel") != 0 )
 			{
 				if((shut_get_string(hValue, string, 0x25)) > 0)
@@ -477,13 +481,13 @@ int shut_identify_ups ()
 
 				tries = 0;
 			}
-	
+
 			dstate_setinfo("ups.model", "%s", finalname);
 		}
 		else
 			tries--;
 	}
-		
+
 	/* Get strings iSerialNumber */
 	if (((retcode = shut_get_string(device_descriptor.dev_desc.iSerialNumber, string, 0x25)) > 0)
 		&& strcmp(string, "") && string[0] != '\t') {
@@ -503,14 +507,14 @@ int shut_identify_ups ()
  * wait for an ACK packet
  *
  * returns 0 on success, -1 on error, -2 on NACK, -3 on NOTIFICATION
- * 
+ *
  *********************************************************************/
 int shut_wait_ack (void)
 {
 	u_char c[1];
 
 	c[0] = '\0';
-	
+
 	serial_read (DEFAULT_TIMEOUT, &c[0]);
 	if (c[0] == SHUT_OK) {
 		upsdebugx (2, "shut_wait_ack(): ACK received");
@@ -532,13 +536,13 @@ int shut_wait_ack (void)
  * char_read (char *bytes, int size, int read_timeout)
  *
  * reads size bytes from the serial port
- * 
+ *
  * bytes     - buffer to store the data
  * size      - size of the data to get
  * read_timeout - serial timeout (in milliseconds)
- * 
+ *
  * return -1 on error, -2 on timeout, nb_bytes_readen on success
- * 
+ *
  *********************************************************************/
 static int char_read (char *bytes, int size, int read_timeout)
 {
@@ -546,13 +550,13 @@ static int char_read (char *bytes, int size, int read_timeout)
 	fd_set readfs;
 	int readen = 0;
 	int rc = 0;
-	
+
 	FD_ZERO (&readfs);
 	FD_SET (upsfd, &readfs);
-	
+
 	serial_timeout.tv_usec = (read_timeout % 1000) * 1000;
 	serial_timeout.tv_sec = (read_timeout / 1000);
-	
+
 	rc = select (upsfd + 1, &readfs, NULL, NULL, &serial_timeout);
 	if (0 == rc)
 		return -2;			/* timeout */
@@ -576,13 +580,13 @@ static int char_read (char *bytes, int size, int read_timeout)
 
 /**********************************************************************
  * serial_read (int read_timeout)
- *  
+ *
  * return data one byte at a time
  *
  * read_timeout - serial timeout (in milliseconds)
  *
  * returns 0 on success, -1 on error, -2 on timeout
- * 
+ *
  **********************************************************************/
 int serial_read (int read_timeout, u_char *readbuf)
 {
@@ -591,9 +595,9 @@ int serial_read (int read_timeout, u_char *readbuf)
 	static u_char *cachee = cache;
 	int recv;
 	*readbuf = '\0';
-	
+
 	/* if still data in cache, get it */
-	if (cachep < cachee) { 
+	if (cachep < cachee) {
 		*readbuf = *cachep++;
 		return 0;
 		/* return (int) *cachep++; */
@@ -607,9 +611,9 @@ int serial_read (int read_timeout, u_char *readbuf)
 	cachee = cache + recv;
 	cachep = cache;
 	cachee = cache + recv;
-	
+
 	if (recv) {
-		upsdebugx(5,"received: %02x", *cachep); 
+		upsdebugx(5,"received: %02x", *cachep);
 		*readbuf = *cachep++;
 		return 0;
 	}
@@ -623,9 +627,9 @@ int serial_read (int read_timeout, u_char *readbuf)
  *
  * buf       - data to send
  * len       - lenght of data to send
- * 
+ *
  * returns number of bytes written on success, -1 on error
- * 
+ *
  **********************************************************************/
 int serial_send (u_char *buf, int len)
 {
@@ -634,7 +638,7 @@ int serial_send (u_char *buf, int len)
 	return write (upsfd, buf, len);
 }
 
-/* 
+/*
  * Serial HID UPS Transfer (SHUT) functions
  *********************************************************************/
 /* Get and parse UPS status */
@@ -644,7 +648,7 @@ void  shut_ups_status(void)
 
 	/* clear status buffer before begining */
 	status_init();
-	
+
 	/* Ensure to have at least basic status */
         while (try < MAX_TRY) {
 	  if((retcode = hid_get_value("UPS.PowerSummary.PresentStatus.ACPresent")) != 0 ) {
@@ -673,7 +677,7 @@ void  shut_ups_status(void)
 		if(hValue == 1)
 			status_set("LB");
 	}
-	
+
 	if(hid_get_value("UPS.PowerSummary.PresentStatus.BelowRemainingCapacityLimit") != 0 ) {
 		if(hValue == 1)
 			status_set("LB");
@@ -688,7 +692,7 @@ void  shut_ups_status(void)
 		if(hValue == 1)
 			status_set("RB");
 	}
-  
+
 	if(hid_get_value("UPS.PowerSummary.PresentStatus.Good") != 0 ) {
 		if(hValue == 0)
 			status_set("OFF");
@@ -714,7 +718,7 @@ u_char shut_checksum(const u_char *buf, int bufsize)
 {
 	int i;
 	u_char chk=0;
-	
+
 	for(i=0; i<bufsize; i++)
 		chk^=buf[i];
 	return chk;
@@ -735,9 +739,9 @@ int shut_packet_send (hid_data_t *hdata, int datalen, u_char token)
 	short Retry=1;
 	short Size;
 	int i;
-	
+
 	upsdebugx (3, "entering shut_packet_send (%i)", datalen);
-	
+
 	while(datalen>0 && Retry>0)
 	{
 		Size=(datalen>=8) ? 8 : datalen;
@@ -745,29 +749,29 @@ int shut_packet_send (hid_data_t *hdata, int datalen, u_char token)
 		/* Packets need only to be sent once
 		 * NACK handling should take care of the rest */
 		if (Retry == 1) {
-			
+
 			/* Forge SHUT Frame */
 			SHUTRequest.bType = SHUT_TYPE_REQUEST + token;
 			SHUTRequest.bLength = (Size<<4) + Size;
 			SHUTRequest.data = *hdata;
 			/* memcpy(&SHUTRequest.data.raw_pkt, hdata->raw_pkt, Size); */
-			
+
 			sdata.shut_pkt = SHUTRequest;
 			sdata.raw_pkt[(Size+3) - 1] = shut_checksum(sdata.shut_pkt.data.raw_pkt, Size);
-	
+
 			upsdebugx (4, "shut_checksum = %2x", sdata.raw_pkt[(Size+3)-1]);
-	
+
 			serial_send (sdata.raw_pkt, Size+3);
 		}
 		i = shut_wait_ack ();
 		if (i == 0) {
 			datalen-=Size;
 			Retry=5;
-			
+
 			upsdebugx (4, "received ACK");
 			break;
 		} else if ((i == -1) || (i == -3)) {
-			/* retry a finite number of times if something wrong happened while 
+			/* retry a finite number of times if something wrong happened while
 			 * sending like a notification or a NACK */
 			if (Retry >= MAX_TRY) {
 				upsdebugx(2, "Max tries reached while waiting for ACK, still getting errors");
@@ -795,11 +799,11 @@ int shut_packet_recv (u_char *Buf, int datalen)
 	shut_data_t   sdata;
 
 	upsdebugx (4, "entering shut_packet_recv (%i)", datalen);
-	
+
 	while(datalen>0 && Retry<3)
 	{
 		if(serial_read (DEFAULT_TIMEOUT, &Start[0]) >= 0)
-		{	  
+		{
 			sdata.shut_pkt.bType = Start[0];
 			if(Start[0]==SHUT_SYNC)
 			{
@@ -807,21 +811,21 @@ int shut_packet_recv (u_char *Buf, int datalen)
 				memcpy(Buf, Start, 1);
 				return 1;
 			}
-			else 
+			else
 			{
 				/* if(((Start[1] = serial_read (DEFAULT_TIMEOUT)) >= 0) && */
 				if((serial_read (DEFAULT_TIMEOUT, &Start[1]) >= 0) &&
 					((Start[1]>>4)==(Start[1]&0x0F)))
 				{
-					upsdebug_hex(3, "Receive", Start, 2); 
+					upsdebug_hex(3, "Receive", Start, 2);
 					Size=Start[1]&0x0F;
 					sdata.shut_pkt.bLength = Size;
 					for(recv=0;recv<Size;recv++)
 						if(serial_read (DEFAULT_TIMEOUT, &Frame[recv]) < 0)
 							break;
-						
-					upsdebug_hex(3, "Receive", Frame, Size); 
-					
+
+					upsdebug_hex(3, "Receive", Frame, Size);
+
 					serial_read (DEFAULT_TIMEOUT, &Chk[0]);
 					if(Chk[0]==shut_checksum(Frame, Size))
 					{
@@ -831,9 +835,9 @@ int shut_packet_recv (u_char *Buf, int datalen)
 						Buf+=Size;
 						Pos+=Size;
 						Retry=0;
-						
+
 						shut_token_send(SHUT_OK);
-						
+
 						if(Start[0]&SHUT_PKT_LAST) {
 							if ((Start[0]&SHUT_PKT_LAST) == SHUT_TYPE_NOTIFY) {
 							/* TODO: process notification (dropped for now) */
@@ -864,13 +868,13 @@ int shut_packet_recv (u_char *Buf, int datalen)
 	return 0;
 }
 
-/* 
+/*
  * Human Interface Device (HID) functions
  *********************************************************************/
 
 /**********************************************************************
  * shut_get_descriptor(int desctype, u_char *pkt)
- * 
+ *
  * get descriptor specified by DescType and return it in Buf
  *
  * desctype  - from shutdataType
@@ -884,20 +888,20 @@ int shut_get_descriptor(int desctype, u_char *pkt, int reportlen)
 	hid_packet_t HIDRequest;
 	hid_data_t   data;
 	int retcode;
-	
+
 	upsdebugx (2, "entering shut_get_descriptor(n %02x, %i)",
 				desctype, reportlen);
-	
+
 	HIDRequest.bmRequestType = REQUEST_TYPE_USB+(desctype>=HID_DESCRIPTOR?1:0);
 	HIDRequest.bRequest = 0x06;
 	HIDRequest.wValue = (desctype<<8);
 	HIDRequest.wIndex = 0x0000;
 	HIDRequest.wLength = reportlen;
-	
+
 	align_request(&HIDRequest);
 
 	data.hid_pkt = HIDRequest;
-	
+
 /*	if((retcode = shut_packet_send (&data, sizeof(data), SHUT_PKT_LAST)) > 0) */
 	if((retcode = shut_packet_send (&data, 8, SHUT_PKT_LAST)) > 0)
 	{
@@ -914,7 +918,7 @@ int shut_get_descriptor(int desctype, u_char *pkt, int reportlen)
 
 /**********************************************************************
  * shut_get_string(int index, u_char *pkt, int reportlen)
- * 
+ *
  * get descriptor specified by DescType and return it in Buf
  *
  * index     - from shutdataType
@@ -930,19 +934,19 @@ int shut_get_string(int strindex, char *string, int stringlen)
 	hid_data_t   data;
 	int retcode;
 	u_char buf[MAX_STRING];
-	
+
 	upsdebugx (2, "entering shut_get_string(%02x)", strindex);
-	
+
 	HIDRequest.bmRequestType = REQUEST_TYPE_USB;
 	HIDRequest.bRequest = 0x06;
 	HIDRequest.wValue = strindex+(STRING_DESCRIPTOR<<8);
 	HIDRequest.wIndex = 0x0000;
 	HIDRequest.wLength = (stringlen<<8); /* (reportlen&0xFF)&(reportlen>>8); */
-	
+
 	align_request(&HIDRequest);
-	
+
 	data.hid_pkt = HIDRequest;
-	
+
 	if((retcode = shut_packet_send (&data, 8, SHUT_PKT_LAST)) >0)
 	{
 		upsdebug_hex(3, "shut_get_string", data.raw_pkt, 8);
@@ -961,7 +965,7 @@ int shut_get_string(int strindex, char *string, int stringlen)
 
 /**********************************************************************
  * shut_get_report(int id, u_char *pkt, int reportlen)
- * 
+ *
  * get report specified by id and return it in pkt
  *
  * id        - from shutdataType
@@ -976,7 +980,7 @@ int shut_get_report(int id, u_char *pkt, int reportlen)
 	hid_packet_t HIDRequest;
 	hid_data_t   data;
 	int retcode;
-	
+
 	upsdebugx (2, "entering shut_get_report(id: %02x, len: %02x)", id, reportlen);
 
 	HIDRequest.bmRequestType = REQUEST_TYPE_GET_REPORT;
@@ -984,11 +988,11 @@ int shut_get_report(int id, u_char *pkt, int reportlen)
 	HIDRequest.wValue = id+(HID_REPORT_TYPE_FEATURE<<8);
 	HIDRequest.wIndex = 0x0000;
 	HIDRequest.wLength = reportlen;
-	
+
 	align_request(&HIDRequest);
-	
+
 	data.hid_pkt = HIDRequest;
-	
+
 /*	if((retcode = shut_packet_send (&data, sizeof(data), SHUT_PKT_LAST)) > 0) */
 	if((retcode = shut_packet_send (&data, 8, SHUT_PKT_LAST)) > 0)
 	{
@@ -1005,7 +1009,7 @@ int shut_get_report(int id, u_char *pkt, int reportlen)
 
 /**********************************************************************
  * shut_set_report(int id, u_char *pkt, int reportlen)
- * 
+ *
  * set report specified by id using pkt as value
  *
  * id        - from shutdataType
@@ -1020,15 +1024,15 @@ int shut_set_report(int id, u_char *pkt, int reportlen)
 	hid_packet_t HIDRequest;
 	hid_data_t   data;
 	int retcode;
-	
+
 	upsdebugx (2, "entering shut_set_report(id: %02x, len: %02x)", id, reportlen);
-	
+
 	HIDRequest.bmRequestType = REQUEST_TYPE_SET_REPORT;
 	HIDRequest.bRequest = 0x09;
 	HIDRequest.wValue = id+(HID_REPORT_TYPE_FEATURE<<8);
 	HIDRequest.wIndex = 0x0000;
 	HIDRequest.wLength = reportlen;
-	
+
 	align_request(&HIDRequest);
 
 	data.hid_pkt = HIDRequest;
@@ -1048,7 +1052,7 @@ int shut_set_report(int id, u_char *pkt, int reportlen)
 
 /**********************************************************************
  * hid_init_device()
- * 
+ *
  * Get Device/HID/Report descriptors from device and initialise
  * HID Parser for further actions
  *
@@ -1058,34 +1062,34 @@ int shut_set_report(int id, u_char *pkt, int reportlen)
 int hid_init_device()
 {
 	int retcode;
-	
+
 	/* Get HID descriptor */
 	if((retcode = shut_get_descriptor(HID_DESCRIPTOR, hid_descriptor.raw_desc, 0x09)) > 0)
 	{
 		upsdebug_hex(3, "shut_get_descriptor(hid)", hid_descriptor.raw_desc, retcode);
-		
+
 		/* WORKAROUND: need to be fixed */
 		hid_descriptor.hid_desc.wDescriptorLength = hid_descriptor.raw_desc[7] +
 			(hid_descriptor.raw_desc[8]<<8);
-		
+
 		upsdebugx(3, "HID Descriptor: \nbLength: \t\t0x%02x\nbDescriptorType: \t0x%02x\n",
 				hid_descriptor.hid_desc.bLength,
 				hid_descriptor.hid_desc.bDescriptorType);
-		
+
 		upsdebugx(3, "bcdHID: \t\t0x%04x\nbCountryCode: \t\t0x%02x\nbNumDescriptors: \t0x%02x\n",
 				hid_descriptor.hid_desc.bcdHID,
 				hid_descriptor.hid_desc.bCountryCode,
 				hid_descriptor.hid_desc.bNumDescriptors);
-		
+
 		upsdebugx(3, "bReportDescriptorType: \t0x%02x\nwDescriptorLength: \t0x%04x",
 				hid_descriptor.hid_desc.bReportDescriptorType,
 				hid_descriptor.hid_desc.wDescriptorLength);
-		
+
 		/* Get Device descriptor */
 		if((retcode = shut_get_descriptor(DEVICE_DESCRIPTOR, device_descriptor.raw_desc, 0x12)) > 0)
-		{	
+		{
 			upsdebug_hex(3, "shut_get_descriptor(device)", device_descriptor.raw_desc, retcode);
-			
+
 			upsdebugx(2, "Device Descriptor: \nbLength: \t\t0x%02x\nbDescriptorType:\
 				\t0x%02x\nbcdUSB: \t\t0x%04x\nbDeviceClass: \t\t0x%02x\nbDeviceSubClass:\
 				\t0x%02x\nbDeviceProtocol: \t0x%02x\nbMaxPacketSize0:\
@@ -1095,24 +1099,24 @@ int hid_init_device()
 				device_descriptor.dev_desc.bLength,
 				device_descriptor.dev_desc.bDescriptorType,
 				device_descriptor.dev_desc.bcdUSB,
-				device_descriptor.dev_desc.bDeviceClass, 
+				device_descriptor.dev_desc.bDeviceClass,
 				device_descriptor.dev_desc.bDeviceSubClass,
 				device_descriptor.dev_desc.bDeviceProtocol,
 				device_descriptor.dev_desc.bMaxPacketSize0,
-				device_descriptor.dev_desc.idVendor, 
+				device_descriptor.dev_desc.idVendor,
 				device_descriptor.dev_desc.idProduct,
 				device_descriptor.dev_desc.bcdDevice,
 				device_descriptor.dev_desc.iManufacturer,
 				device_descriptor.dev_desc.iProduct,
 				device_descriptor.dev_desc.iSerialNumber,
-				device_descriptor.dev_desc.bNumConfigurations);	
+				device_descriptor.dev_desc.bNumConfigurations);
 
 			/* Get Report descriptor */
 			if((retcode = shut_get_descriptor(REPORT_DESCRIPTOR, raw_buf,
 				hid_descriptor.hid_desc.wDescriptorLength)) > 0) {
 
 				upsdebug_hex(3, "shut_get_descriptor(report)", raw_buf, retcode);
-				
+
 				/* Parse Report Descriptor */
 				Free_ReportDesc(pDesc);
 				pDesc = Parse_ReportDesc(raw_buf, retcode);
@@ -1138,24 +1142,24 @@ ushort lookup_path(const char *HIDpath, HIDData_t *data)
 	ushort i = 0, cond = 1;
 	int cur_usage;
 	char buf[MAX_STRING];
-	char *start, *end; 
-	
+	char *start, *end;
+
 	strncpy(buf, HIDpath, strlen(HIDpath));
 	buf[strlen(HIDpath)] = '\0';
 	start = end = buf;
-	
+
 	upsdebugx(3, "entering lookup_path(%s)", buf);
 
 	while (cond) {
-	
+
 		if ((end = strchr(start, '.')) == NULL) {
-			cond = 0;			
+			cond = 0;
 		}
 		else
 			*end = '\0';
-	
+
 		upsdebugx(4, "parsing %s", start);
-	
+
 		/* lookup code */
 		if ((cur_usage = hid_lookup_usage(start)) == -1) {
 			upsdebugx(4, "%s wasn't found", start);
@@ -1163,9 +1167,9 @@ ushort lookup_path(const char *HIDpath, HIDData_t *data)
 		}
 		else {
 			data->Path.Node[i] = cur_usage;
-			i++; 
+			i++;
 		}
-	
+
 		if(cond)
 			start = end +1 ;
 	}
@@ -1175,11 +1179,11 @@ ushort lookup_path(const char *HIDpath, HIDData_t *data)
 
 /* Lookup this usage name to find its code (page + index) */
 int hid_lookup_usage(char *name)
-{	
+{
 	int i;
-	
+
 	upsdebugx(4, "Looking up %s", name);
-	
+
 	if (name[0] == '[') /* manage indexed collection */
 		return (0x00FF0000 + atoi(&name[1]));
 	else {
@@ -1201,21 +1205,21 @@ int hid_get_value(const char *item_path)
 {
 	int i, retcode;
    HIDData_t hData;
-	
+
 	upsdebugx(3, "entering hid_get_value(%s)", item_path);
-	
+
 	/* Prepare path of HID object */
 	hData.Type = ITEM_FEATURE;
 	hData.ReportID = 0;
 
 	if((retcode = lookup_path(item_path, &hData)) > 0) {
 		upsdebugx(3, "Path depth = %i\n", retcode);
-		
+
 		for (i = 0; i<retcode; i++)
 			upsdebugx(4, "%i: Usage(%08x)\n", i, hData.Path.Node[i]);
-			
+
 		hData.Path.Size = retcode;
-    
+
 		/* Get info on object (reportID, offset and size) */
 		if (FindObject(pDesc,&hData) == 1) {
 			if (shut_get_report(hData.ReportID, raw_buf, MAX_REPORT_SIZE) > 0) {
@@ -1240,7 +1244,7 @@ int hid_get_value(const char *item_path)
 	return 0;
 }
 
-  /* 
+  /*
    * Internal functions
  ****************************************************************************/
 
@@ -1254,24 +1258,24 @@ int hid_get_value(const char *item_path)
  */
 void make_string(u_char *buf, int datalen, char *string)
 {
-	int i,		/* Skip size and type */ 
+	int i,		/* Skip size and type */
 		j=0;
-	
+
 	upsdebugx(4, "String descriptor: size = 0x%02x, type = 0x%02x", buf[0], buf[1]);
-	
+
 	/* TODO: add clean support for UNICODE */
 	for(i=2;i<datalen;i++) {
-		if(buf[i]!=0x00) {      
+		if(buf[i]!=0x00) {
 			string[j]=buf[i];
 			j++;
 		}
-	} 
+	}
 	string[j++]='\0';
 }
 
 /*
  * set RTS to on and DTR to off
- * 
+ *
  * set : 1 to set comm
  * set : 0 to stop commupsh.
  */
@@ -1298,7 +1302,7 @@ float expo(int a, int b)
 		return (float) a * expo(a,b-1);
 	if (b<0)
 		return (float)((float)(1/(float)a) * (float) expo(a,b+1));
-	
+
 	/* not reached */
 	return -1;
 }
@@ -1333,10 +1337,10 @@ int hid_set_value(const char *varname, const char *val)
 	int retcode, i, replen;
 	mge_info_item_t *shut_info_p;
    HIDData_t hData;
-		
+
 	upsdebugx(2, "============== entering hid_set_value(%s, %s) ==============", varname, val);
-	
-	/* 1) retrieve and check netvar & item_path */	
+
+	/* 1) retrieve and check netvar & item_path */
 	shut_info_p = shut_find_info(varname);
 
 	if (shut_info_p == NULL || shut_info_p->type == NULL ||
@@ -1371,37 +1375,37 @@ int hid_set_value(const char *varname, const char *val)
 
 	if((retcode = lookup_path(shut_info_p->item_path, &hData)) > 0) {
 		upsdebugx(3, "Path depth = %i\n", retcode);
-		
+
 		for (i = 0; i<retcode; i++)
 			upsdebugx(4, "%i: Usage(%08x)\n", i, hData.Path.Node[i]);
-			
+
 		hData.Path.Size = retcode;
-    
+
 		/* Get info on object (reportID, offset and size) */
 		if (FindObject(pDesc,&hData) == 1) {
 			replen = shut_get_report(hData.ReportID, raw_buf, MAX_REPORT_SIZE);
-			
+
 			GetValue((const u_char *) raw_buf, &hData, &hValue);
-			
+
 			/* Test if Item is settable */
 			if (hData.Attribute != ATTR_DATA_CST) {
 				/* Set new value for this item */
 				hValue = atol(val);
 				SetValue(&hData, raw_buf, hValue);
 				shut_set_report(hData.ReportID, raw_buf, replen);
-				
+
 				/* check if set succeed ! => disabled for now
 				if (shut_get_report(hData.ReportID, raw_buf, MAX_REPORT_SIZE) > 0) {
 					GetValue((const u_char *) raw_buf, &hData, &hValue);
 					upsdebugx(3, "Value = %d", hValue);
-				
+
 					if (hValue != atol(val))
 						upsdebugx(3, "FAILED");
 					else
 						upsdebugx(3, "SUCCEED");
 				} else
 					upsdebugx(3, "FAILED");
-				*/				
+				*/
 				return STAT_SET_HANDLED;
 			}
 			else
@@ -1412,7 +1416,7 @@ int hid_set_value(const char *varname, const char *val)
 	}
 	else
 		upsdebugx(3, "Can't lookup object's path");
-	
+
 	return STAT_SET_UNKNOWN;
 }
 
@@ -1424,7 +1428,7 @@ mge_info_item_t *shut_find_info(const char *varname)
 	for (shut_info_p = &mge_info[0]; shut_info_p->type != NULL; shut_info_p++)
 		if (!strcasecmp(shut_info_p->type, varname))
 			return shut_info_p;
-		
+
 	fatalx(EXIT_FAILURE, "shut_find_info: unknown info type: %s", varname);
 	return NULL;
 }
@@ -1433,16 +1437,16 @@ mge_info_item_t *shut_find_info(const char *varname)
 static char *hu_find_infoval(info_lkp_t *hid2info, long value)
 {
   info_lkp_t *info_lkp;
-  
+
   upsdebugx(3, "hu_find_infoval: searching for value = %ld\n", value);
-  
+
   for (info_lkp = hid2info; (info_lkp != NULL) &&
 	 (strcmp(info_lkp->nut_value, "NULL")); info_lkp++) {
-    
+
     if (info_lkp->hid_value == value) {
       upsdebugx(3, "hu_find_infoval: found %s (value: %ld)\n",
 		info_lkp->nut_value, value);
-      
+
       return info_lkp->nut_value;
     }
   }
