@@ -1,7 +1,7 @@
 /* mge-utalk.c - monitor MGE UPS for NUT with UTalk protocol
  *
  *  Copyright (C) 2002 - 2005
- *     Arnaud Quette <arnaud.quette@free.fr>  & <arnaud.quette@mgeups.com>
+ *     Arnaud Quette <arnaud.quette@gmail.com>
  *     Hans Ekkehard Plesser <hans.plesser@itf.nlh.no>
  *     Martin Loyer <martin@ouifi.net>
  *     Patrick Agrain <patrick.agrain@alcatel.fr>
@@ -10,7 +10,7 @@
  *     Marek Kralewski <marek@mercy49.de>
  *
  *  This driver is a collaborative effort by the above people,
- *  Sponsored by MGE UPS SYSTEMS <http://opensource.mgeups.com/>
+ *  Sponsored by MGE UPS SYSTEMS
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,6 +28,30 @@
  *
  */
 
+/*
+ * IMPLEMENTATION DETAILS
+ * 
+ * Not all UTalk models provide all possible information, settings and commands.
+ * mge-utalk checks on startup which variables and commands are available from
+ * the UPS, and re-reads these regularly. Thus, startup is a bit slow, but this
+ * should not matter much.
+ * 
+ * mge-utalk.h defines a struct array that tells the driver how to read
+ * variables from the UPS and publish them as NUT data.
+ * 
+ * "ups.status" variable is not included in this array, since it contains
+ * information that requires several calls to the UPS and more advanced analysis
+ * of the reponses. The function get_ups_status does this job.
+ * 
+ * Note that MGE enumerates the status "bits" from right to left,
+ * i.e., if buf[] contains the reponse to command "Ss" (read system status),
+ * then buf[0] contains "bit" Ss.1.7 (General alarm), while buf[7] contains
+ * "bit" Ss.1.0 (Load unprotected). 
+ * 
+ * enable_ups_comm() is called before each attempt to read/write data
+ * from/to the UPS to re synchronise the communication.
+ */
+
 #include <ctype.h>
 #include <sys/ioctl.h>
 #include "timehead.h"
@@ -39,8 +63,24 @@
 /*                  Define "technical" constants                   */
 /* --------------------------------------------------------------- */
 
-#define DRIVER_NAME    "MGE UPS SYSTEMS/U-Talk driver"
-#define DRIVER_VERSION "0.88"
+#define DRIVER_NAME	"MGE UPS SYSTEMS/U-Talk driver"
+#define DRIVER_VERSION	"0.89"
+
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Arnaud Quette <ArnaudQuette@gmail.com>\n" \
+	"Hans Ekkehard Plesser <hans.plesser@itf.nlh.no>\n" \
+	"Martin Loyer <martin@ouifi.net>\n" \
+	"Patrick Agrain <patrick.agrain@alcatel.fr>\n" \
+	"Nicholas Reilly <nreilly@magma.ca>\n" \
+	"Dave Abbott <d.abbott@dcs.shef.ac.uk>\n" \
+	"Marek Kralewski <marek@mercy49.de>",
+	DRV_STABLE,
+	{ NULL }
+};
 
 /* delay after sending each char to UPS (in MICROSECONDS) */
 #define MGE_CHAR_DELAY 0
@@ -62,7 +102,6 @@
 #define SD_STAYOFF	1
 
 int sdtype = SD_RETURN;
-static unsigned int pollinterval;
 static time_t lastpoll; /* Timestamp the last polling */
 
 /* --------------------------------------------------------------- */
@@ -97,14 +136,6 @@ static void format_model_name(char *model);
 /*                    UPS Driver Functions                         */
 /* --------------------------------------------------------------- */
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - %s %s (%s)\n", 
-		DRIVER_NAME, DRIVER_VERSION, UPS_VERSION);
-}
-
-/* --------------------------------------------------------------- */
-
 void upsdrv_makevartable(void)
 {
 	char temp[BUFFLEN];
@@ -141,11 +172,6 @@ void upsdrv_initups(void)
 	if (testvar ("oldmac"))
 		RTS = ~TIOCM_RTS;
 	
-	if (dstate_getinfo ("driver.parameter.pollinterval") != NULL)
-		pollinterval = atoi (dstate_getinfo ("driver.parameter.pollinterval"));
-	else
-		pollinterval = 2;
-
 	/* Init serial line */
 	ioctl(upsfd, TIOCMBIC, &RTS);
 	enable_ups_comm();
@@ -383,7 +409,7 @@ void upsdrv_updateinfo(void)
 	}
 
 	/* Don't overload old units (at startup) */
-	if ( (unsigned int)time(NULL) <= (unsigned int)(lastpoll + pollinterval) )
+	if ( (unsigned int)time(NULL) <= (unsigned int)(lastpoll + poll_interval) )
 		return;
 
 	/* update all other ok variables */
@@ -398,7 +424,7 @@ void upsdrv_updateinfo(void)
 			
 			if ( bytes_rcvd > 0 && buf[0] != '?' )  {
 				extract_info(buf, item, infostr, sizeof(infostr));
-				dstate_setinfo(item->type, infostr);
+				dstate_setinfo(item->type, "%s", infostr);
 				upsdebugx(2, "updateinfo: %s == >%s<", item->type, infostr);
 				dstate_dataok();
 			} else {
@@ -621,7 +647,7 @@ static void enable_ups_comm(void)
 	char buf[8];
 	
 	/* only enable communication if needed! */
-	if ( mge_command(buf, 8, "Si") == -1)
+	if ( mge_command(buf, 8, "Si") <= 0)
 	{
 		mge_command(NULL, 0, "Z");   /* send Z twice --- speeds up re-connect */
 		mge_command(NULL, 0, "Z");
