@@ -1,7 +1,8 @@
-/* mge-xml.c		Model specific routines for MGE XML protocol UPSes 
+/* mge-xml.c	Model specific routines for Eaton / MGE XML protocol UPSes
 
    Copyright (C)
-	2008		Arjen de Korte <adkorte-guest@alioth.debian.org>
+	2008-2009	Arjen de Korte <adkorte-guest@alioth.debian.org>
+	2009		Arnaud Quette <ArnaudQuette@Eaton.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,9 +31,14 @@
 #include "netxml-ups.h"
 #include "mge-xml.h"
 
-#define MGE_XML_VERSION		"MGEXML/0.12"
+#define MGE_XML_VERSION		"MGEXML/0.21"
 #define MGE_XML_INITUPS		"/"
-#define MGE_XML_INITINFO	"/mgeups/product.xml"
+#define MGE_XML_INITINFO	"/mgeups/product.xml /product.xml /ws/product.xml"
+
+#define ST_FLAG_RW		0x0001
+#define ST_FLAG_STATIC		0x0002
+
+static int	mge_ambient_value = 0;
 
 static char	mge_scratch_buf[256];
 
@@ -80,23 +86,31 @@ typedef enum {
 	/* /GET_OBJECT */
 
 	SET_OBJECT = 400,	/* "/setvalue.cgi" */
-		SO_OBJECT
+		SO_OBJECT,
 	/* /SET_OBJECT */
 
+	ALARM = 500,
+
+	XML_CLIENT = 600,
+		XC_GENERAL = 610,
+			XC_STARTUP,
+			XC_SHUTDOWN,
+			XC_BROADCAST
+
 } mge_xml_state_t;
 
 typedef struct {
-	char	*nutname;	/* NUT variable name */
+	const char	*nutname;	/* NUT variable name */
 	uint32_t nutflags;	/* NUT flags (to set in addinfo) */
 	size_t	nutlen;		/* length of the NUT string */
-	char	*xmlname;	/* XML varianle name */
+	const char	*xmlname;	/* XML variable name */
 	uint32_t xmlflags;	/* XML flags (to be used to determine what kind of variable this is */
 	size_t	xmllen;		/* length of the XML string */
-	char	*(*convert)(const char *value);	/* conversion function from XML<->NUT value (returns
+	const char	*(*convert)(const char *value);	/* conversion function from XML<->NUT value (returns
 						   NULL if no further processing is required) */
 } xml_info_t;
 
-static char *online_info(const char *val)
+static const char *online_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(ONLINE);
@@ -107,7 +121,7 @@ static char *online_info(const char *val)
 	return NULL;
 }
 
-static char *discharging_info(const char *val)
+static const char *discharging_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(DISCHRG);
@@ -118,7 +132,7 @@ static char *discharging_info(const char *val)
 	return NULL;
 }
 
-static char *charging_info(const char *val)
+static const char *charging_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(CHRG);
@@ -129,7 +143,7 @@ static char *charging_info(const char *val)
 	return NULL;
 }
 
-static char *lowbatt_info(const char *val)
+static const char *lowbatt_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(LOWBATT);
@@ -140,7 +154,7 @@ static char *lowbatt_info(const char *val)
 	return NULL;
 }
 
-static char *overload_info(const char *val)
+static const char *overload_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(OVERLOAD);
@@ -151,7 +165,7 @@ static char *overload_info(const char *val)
 	return NULL;
 }
 
-static char *replacebatt_info(const char *val)
+static const char *replacebatt_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(REPLACEBATT);
@@ -162,7 +176,7 @@ static char *replacebatt_info(const char *val)
 	return NULL;
 }
 
-static char *trim_info(const char *val)
+static const char *trim_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(TRIM);
@@ -173,7 +187,7 @@ static char *trim_info(const char *val)
 	return NULL;
 }
 
-static char *boost_info(const char *val)
+static const char *boost_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(BOOST);
@@ -184,7 +198,7 @@ static char *boost_info(const char *val)
 	return NULL;
 }
 
-static char *bypass_aut_info(const char *val)
+static const char *bypass_aut_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(BYPASSAUTO);
@@ -195,7 +209,7 @@ static char *bypass_aut_info(const char *val)
 	return NULL;
 }
 
-static char *bypass_man_info(const char *val)
+static const char *bypass_man_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(BYPASSMAN);
@@ -206,7 +220,7 @@ static char *bypass_man_info(const char *val)
 	return NULL;
 }
 
-static char *off_info(const char *val)
+static const char *off_info(const char *val)
 {
 	if (val[0] == '0') {
 		STATUS_SET(OFF);
@@ -220,7 +234,7 @@ static char *off_info(const char *val)
 /* note: this value is reverted (0=set, 1=not set). We report "battery
    not installed" rather than "battery installed", so that devices
    that don't implement this variable have a battery by default */
-static char *nobattery_info(const char *val)
+static const char *nobattery_info(const char *val)
 {
 	if (val[0] == '0') {
 		STATUS_SET(NOBATTERY);
@@ -231,7 +245,7 @@ static char *nobattery_info(const char *val)
 	return NULL;
 }
 
-static char *fanfail_info(const char *val)
+static const char *fanfail_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(FANFAIL);
@@ -241,8 +255,8 @@ static char *fanfail_info(const char *val)
 
 	return NULL;
 }
-
-static char *shutdownimm_info(const char *val)
+/*
+static const char *shutdownimm_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(SHUTDOWNIMM);
@@ -252,8 +266,8 @@ static char *shutdownimm_info(const char *val)
 
 	return NULL;
 }
-
-static char *overheat_info(const char *val)
+ */
+static const char *overheat_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(OVERHEAT);
@@ -264,7 +278,7 @@ static char *overheat_info(const char *val)
 	return NULL;
 }
 
-static char *commfault_info(const char *val)
+static const char *commfault_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(COMMFAULT);
@@ -275,7 +289,7 @@ static char *commfault_info(const char *val)
 	return NULL;
 }
 
-static char *internalfailure_info(const char *val)
+static const char *internalfailure_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(INTERNALFAULT);
@@ -286,7 +300,7 @@ static char *internalfailure_info(const char *val)
 	return NULL;
 }
 
-static char *battvoltlo_info(const char *val)
+static const char *battvoltlo_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(BATTVOLTLO);
@@ -297,7 +311,7 @@ static char *battvoltlo_info(const char *val)
 	return NULL;
 }
 
-static char *battvolthi_info(const char *val)
+static const char *battvolthi_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(BATTVOLTHI);
@@ -308,7 +322,7 @@ static char *battvolthi_info(const char *val)
 	return NULL;
 }
 
-static char *chargerfail_info(const char *val)
+static const char *chargerfail_info(const char *val)
 {
 	if ((val[0] == '1') || !strncasecmp(val, "Yes", 3)) {
 		STATUS_SET(CHARGERFAIL);
@@ -319,7 +333,7 @@ static char *chargerfail_info(const char *val)
 	return NULL;
 }
 
-static char *vrange_info(const char *val)
+static const char *vrange_info(const char *val)
 {
 	if ((val[0] == '1') || !strncasecmp(val, "Yes", 3)) {
 		STATUS_SET(VRANGE);
@@ -330,7 +344,7 @@ static char *vrange_info(const char *val)
 	return NULL;
 }
 
-static char *frange_info(const char *val)
+static const char *frange_info(const char *val)
 {
 	if ((val[0] == '1') || !strncasecmp(val, "Yes", 3)) {
 		STATUS_SET(FRANGE);
@@ -341,7 +355,7 @@ static char *frange_info(const char *val)
 	return NULL;
 }
 
-static char *fuse_fault_info(const char *val)
+static const char *fuse_fault_info(const char *val)
 {
 	if (val[0] == '1') {
 		STATUS_SET(FUSEFAULT);
@@ -352,7 +366,7 @@ static char *fuse_fault_info(const char *val)
 	return NULL;
 }
 
-static char *yes_no_info(const char *val)
+static const char *yes_no_info(const char *val)
 {
 	switch(val[0])
 	{
@@ -366,7 +380,7 @@ static char *yes_no_info(const char *val)
 	}
 }
 
-static char *on_off_info(const char *val)
+static const char *on_off_info(const char *val)
 {
 	switch(val[0])
 	{
@@ -380,7 +394,7 @@ static char *on_off_info(const char *val)
 	}
 }
 
-static char *convert_deci(const char *val)
+static const char *convert_deci(const char *val)
 {
 	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.1f", 0.1 * (float)atoi(val));
 
@@ -388,7 +402,7 @@ static char *convert_deci(const char *val)
 }
 
 /* Ignore a zero value if the UPS is not switched off */
-static char *ignore_if_zero(const char *val)
+static const char *ignore_if_zero(const char *val)
 {
 	if (atoi(val) == 0) {
 		return NULL;
@@ -399,17 +413,17 @@ static char *ignore_if_zero(const char *val)
 
 /* Set the 'ups.date' from the combined value
  * (ex. 2008/03/01 15:23:26) and return the time */
-static char *split_date_time(const char *val)
+static const char *split_date_time(const char *val)
 {
 	char	*last = NULL;
 
 	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%s", val);
-	dstate_setinfo("ups.date", "%s", strtok_r(mge_scratch_buf, " ", &last));
+	dstate_setinfo("ups.date", "%s", strtok_r(mge_scratch_buf, " -", &last));
 
 	return strtok_r(NULL, " ", &last);
 }
 
-static char *url_convert(const char *val)
+static const char *url_convert(const char *val)
 {
 	char	buf[256], *last = NULL;
 
@@ -419,19 +433,19 @@ static char *url_convert(const char *val)
 	return mge_scratch_buf;
 }
 
-static char *mge_battery_capacity(const char *val)
+static const char *mge_battery_capacity(const char *val)
 {
 	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.2f", (float)atoi(val) / 3600);
 	return mge_scratch_buf;
 }
 
-static char *mge_powerfactor_conversion(const char *val)
+static const char *mge_powerfactor_conversion(const char *val)
 {
 	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.2f", (float)atoi(val) / 100);
 	return mge_scratch_buf;
 }
 
-static char *mge_beeper_info(const char *val)
+static const char *mge_beeper_info(const char *val)
 {
 	switch (atoi(val))
 	{
@@ -445,7 +459,7 @@ static char *mge_beeper_info(const char *val)
 	return NULL;
 }
 
-static char *mge_upstype_conversion(const char *val)
+static const char *mge_upstype_conversion(const char *val)
 {
 	switch (atoi(val))
 	{
@@ -463,7 +477,7 @@ static char *mge_upstype_conversion(const char *val)
 	return NULL;
 }
 
-static char *mge_sensitivity_info(const char *val)
+static const char *mge_sensitivity_info(const char *val)
 {
 	switch (atoi(val))
 	{
@@ -477,7 +491,7 @@ static char *mge_sensitivity_info(const char *val)
 	return NULL;
 }
 
-static char *mge_test_result_info(const char *val)
+static const char *mge_test_result_info(const char *val)
 {
 	switch (atoi(val))
 	{
@@ -497,7 +511,29 @@ static char *mge_test_result_info(const char *val)
 	return NULL;
 }
 
-#define ST_FLAG_RW	0
+static const char *mge_ambient_info(const char *val)
+{
+	switch (mge_ambient_value)
+	{
+	case 1:
+		return (char *)val;
+	default:
+		return NULL;
+	}
+}
+
+static const char *mge_timer_shutdown(const char *val)
+{
+	const char	*delay = dstate_getinfo("ups.delay.shutdown");
+
+	if ((delay) && (atoi(val) > -1) && (atoi(val) < atoi(delay))) {
+		STATUS_SET(SHUTDOWNIMM);
+	} else {
+		STATUS_CLR(SHUTDOWNIMM);
+	}
+
+	return val;
+}
 
 static xml_info_t mge_xml2nut[] = {
 	/* Special case: boolean values that are mapped to ups.status and ups.alarm */
@@ -526,44 +562,46 @@ static xml_info_t mge_xml2nut[] = {
 	{ NULL, 0, 0, "UPS.PowerSummary.PresentStatus.InternalFailure", 0, 0, internalfailure_info },
 	{ NULL, 0, 0, "UPS.PowerSummary.PresentStatus.CommunicationLost", 0, 0, commfault_info },
 	{ NULL, 0, 0, "UPS.PowerSummary.PresentStatus.OverTemperature", 0, 0, overheat_info },
-	{ NULL, 0, 0, "UPS.PowerSummary.PresentStatus.ShutdownImminent", 0, 0, shutdownimm_info },
+	/* { NULL, 0, 0, "UPS.PowerSummary.PresentStatus.ShutdownImminent", 0, 0, shutdownimm_info }, */
 
 	/* Battery page */
 	{ "battery.charge", 0, 0, "UPS.PowerSummary.RemainingCapacity", 0, 0, NULL },
-	{ "battery.charge.low", ST_FLAG_RW, 5, "UPS.PowerSummary.RemainingCapacityLimitSetting", 0, 0, NULL },
+	{ "battery.charge.low", 0, 0, "UPS.PowerSummary.RemainingCapacityLimitSetting", 0, 0, NULL },
 	{ "battery.charge.low", 0, 0, "UPS.PowerSummary.RemainingCapacityLimit", 0, 0, NULL }, /* Read only */
-	{ "battery.charge.restart", ST_FLAG_RW, 3, "UPS.PowerSummary.RestartLevel", 0, 0, NULL },
+	{ "battery.charge.restart", 0, 0, "UPS.PowerSummary.RestartLevel", 0, 0, NULL },
 	{ "battery.capacity", 0, 0, "UPS.BatterySystem.Battery.DesignCapacity", 0, 0, mge_battery_capacity }, /* conversion needed from As to Ah */
 	{ "battery.runtime", 0, 0, "UPS.PowerSummary.RunTimeToEmpty", 0, 0, NULL },
+	{ "battery.runtime.low", 0, 0, "System.RunTimeToEmptyLimit", 0, 0, NULL },
 	{ "battery.temperature", 0, 0, "UPS.BatterySystem.Battery.Temperature", 0, 0, NULL },
-	{ "battery.type", 0, 0, "UPS.PowerSummary.iDeviceChemistry", 0, 0, NULL },
-	{ "battery.type", 0, 0, "UPS.PowerSummary.iDeviceChemistery", 0, 0, NULL }, /* [sic] */
+	{ "battery.type", ST_FLAG_STATIC, 0, "UPS.PowerSummary.iDeviceChemistry", 0, 0, NULL },
+	{ "battery.type", ST_FLAG_STATIC, 0, "UPS.PowerSummary.iDeviceChemistery", 0, 0, NULL }, /* [sic] */
 	{ "battery.voltage", 0, 0, "UPS.PowerSummary.Voltage", 0, 0, NULL },
-	{ "battery.voltage.nominal", 0, 0, "UPS.BatterySystem.ConfigVoltage", 0, 0, NULL },
-	{ "battery.voltage.nominal", 0, 0, "UPS.PowerSummary.ConfigVoltage", 0, 0, NULL }, /* mge_battery_voltage_nominal */
+	{ "battery.voltage.nominal", ST_FLAG_STATIC, 0, "UPS.BatterySystem.ConfigVoltage", 0, 0, NULL },
+	{ "battery.voltage.nominal", ST_FLAG_STATIC, 0, "UPS.PowerSummary.ConfigVoltage", 0, 0, NULL }, /* mge_battery_voltage_nominal */
 	{ "battery.current", 0, 0, "UPS.PowerSummary.Current", 0, 0, NULL },
-	{ "battery.protection", ST_FLAG_RW, 5, "UPS.BatterySystem.Battery.DeepDischargeProtection", 0, 0, yes_no_info },
-	{ "battery.energysave", ST_FLAG_RW, 5, "UPS.PowerConverter.Input[3].EnergySaving", 0, 0, yes_no_info },
+	{ "battery.protection", 0, 0, "UPS.BatterySystem.Battery.DeepDischargeProtection", 0, 0, yes_no_info },
+	{ "battery.energysave", 0, 0, "UPS.PowerConverter.Input[3].EnergySaving", 0, 0, yes_no_info },
 
 	/* UPS page */
-	{ "ups.mfr", 0, 0, "UPS.PowerSummary.iManufacturer", 0, 0, NULL },
-	{ "ups.model", 0, 0, "UPS.PowerSummary.iProduct", 0, 0, NULL },
-	{ "ups.model.aux", 0, 0, "UPS.PowerSummary.iModel", 0, 0, NULL },
-	{ "ups.model", 0, 0, "System.Description", 0, 0, NULL },
+	{ "ups.mfr", ST_FLAG_STATIC, 0, "UPS.PowerSummary.iManufacturer", 0, 0, NULL },
+	{ "ups.model", ST_FLAG_STATIC, 0, "System.Description", 0, 0, NULL },
+	{ "ups.model", ST_FLAG_STATIC, 0, "UPS.PowerSummary.iProduct", 0, 0, NULL },
+	{ "ups.model.aux", ST_FLAG_STATIC, 0, "UPS.PowerSummary.iModel", 0, 0, NULL },
 	{ "ups.time", 0, 0, "System.LastAcquisition", 0, 0, split_date_time },
 	/* -> XML variable System.Location [Computer Room] doesn't map to any NUT variable */
 	/* -> XML variable System.Contact [Computer Room Manager] doesn't map to any NUT variable */
 	/* -> XML variable UPS.PowerSummary.iProduct [Evolution] doesn't map to any NUT variable */
 	/* -> XML variable UPS.PowerSummary.iModel [650] doesn't map to any NUT variable */
-	{ "ups.serial", 0, 0, "UPS.PowerSummary.iSerialNumber", 0, 0, NULL },
-	{ "ups.firmware", 0, 0, "UPS.PowerSummary.iVersion", 0, 0, NULL },
+	{ "ups.serial", ST_FLAG_STATIC, 0, "UPS.PowerSummary.iSerialNumber", 0, 0, NULL },
+	{ "ups.firmware", ST_FLAG_STATIC, 0, "UPS.PowerSummary.iVersion", 0, 0, NULL },
 	{ "ups.load", 0, 0, "UPS.PowerSummary.PercentLoad", 0, 0, NULL },
-	{ "ups.load.high", ST_FLAG_RW, 5, "UPS.Flow[4].ConfigPercentLoad", 0, 0, NULL },
+	{ "ups.load.high", 0, 0, "UPS.Flow[4].ConfigPercentLoad", 0, 0, NULL },
+	{ "ups.delay.shutdown", 0, 0, "System.ShutdownDuration", 0, 0, NULL },
 	{ "ups.timer.start", 0, 0, "UPS.PowerSummary.DelayBeforeStartup", 0, 0, NULL},
-	{ "ups.timer.shutdown", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", 0, 0, NULL },
-	{ "ups.timer.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", 0, 0, NULL},
+	{ "ups.timer.shutdown", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", 0, 0, mge_timer_shutdown },
+	{ "ups.timer.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", 0, 0, NULL },
 	{ "ups.test.result", 0, 0, "UPS.BatterySystem.Battery.Test", 0, 0, mge_test_result_info },
-	{ "ups.test.interval", ST_FLAG_RW, 8, "UPS.BatterySystem.Battery.TestPeriod", 0, 0, NULL },
+	{ "ups.test.interval", 0, 0, "UPS.BatterySystem.Battery.TestPeriod", 0, 0, NULL },
 	{ "ups.beeper.status", 0 ,0, "UPS.BatterySystem.Battery.AudibleAlarmControl", 0, 0, mge_beeper_info },
 	{ "ups.beeper.status", 0 ,0, "UPS.PowerSummary.AudibleAlarmControl", 0, 0, mge_beeper_info },
 	{ "ups.temperature", 0, 0, "UPS.PowerSummary.Temperature", 0, 0, NULL },
@@ -571,16 +609,16 @@ static xml_info_t mge_xml2nut[] = {
 	{ "ups.L1.power", 0, 0, "UPS.PowerConverter.Output.Phase[1].ApparentPower", 0, 0, ignore_if_zero },
 	{ "ups.L2.power", 0, 0, "UPS.PowerConverter.Output.Phase[2].ApparentPower", 0, 0, ignore_if_zero },
 	{ "ups.L3.power", 0, 0, "UPS.PowerConverter.Output.Phase[3].ApparentPower", 0, 0, ignore_if_zero },
-	{ "ups.power.nominal", 0, 0, "UPS.Flow[4].ConfigApparentPower", 0, 0, NULL },
+	{ "ups.power.nominal", ST_FLAG_STATIC, 0, "UPS.Flow[4].ConfigApparentPower", 0, 0, NULL },
 	{ "ups.realpower", 0, 0, "UPS.PowerConverter.Output.ActivePower", 0, 0, NULL },
 	{ "ups.L1.realpower", 0, 0, "UPS.PowerConverter.Output.Phase[1].ActivePower", 0, 0, ignore_if_zero },
 	{ "ups.L2.realpower", 0, 0, "UPS.PowerConverter.Output.Phase[2].ActivePower", 0, 0, ignore_if_zero },
 	{ "ups.L3.realpower", 0, 0, "UPS.PowerConverter.Output.Phase[3].ActivePower", 0, 0, ignore_if_zero },
-	{ "ups.realpower.nominal", 0, 0, "UPS.Flow[4].ConfigActivePower", 0, 0, NULL },
-	{ "ups.start.auto", ST_FLAG_RW, 5, "UPS.PowerConverter.Input[1].AutomaticRestart", 0, 0, yes_no_info },
-	{ "ups.start.battery", ST_FLAG_RW, 5, "UPS.PowerConverter.Input[3].StartOnBattery", 0, 0, yes_no_info },
-	{ "ups.start.reboot", ST_FLAG_RW, 5, "UPS.PowerConverter.Output.ForcedReboot", 0, 0, yes_no_info },
-	{ "ups.type", 0, 0, "UPS.PowerConverter.ConverterType", 0, 0, mge_upstype_conversion },
+	{ "ups.realpower.nominal", ST_FLAG_STATIC, 0, "UPS.Flow[4].ConfigActivePower", 0, 0, NULL },
+	{ "ups.start.auto", 0, 0, "UPS.PowerConverter.Input[1].AutomaticRestart", 0, 0, yes_no_info },
+	{ "ups.start.battery", 0, 0, "UPS.PowerConverter.Input[3].StartOnBattery", 0, 0, yes_no_info },
+	{ "ups.start.reboot", 0, 0, "UPS.PowerConverter.Output.ForcedReboot", 0, 0, yes_no_info },
+	{ "ups.type", ST_FLAG_STATIC, 0, "UPS.PowerConverter.ConverterType", 0, 0, mge_upstype_conversion },
 
 	/* Input page */
 	{ "input.voltage", 0, 0, "UPS.PowerConverter.Input[1].Voltage", 0, 0, NULL },
@@ -601,17 +639,17 @@ static xml_info_t mge_xml2nut[] = {
 	{ "input.current.nominal", 0, 0, "UPS.Flow[1].ConfigCurrent", 0, 0, NULL },
 	{ "input.frequency", 0, 0, "UPS.PowerConverter.Input[1].Frequency", 0, 0, NULL },
 	{ "input.frequency.nominal", 0, 0, "UPS.Flow[1].ConfigFrequency", 0, 0, NULL },
-	{ "input.voltage.extended", ST_FLAG_RW, 5, "UPS.PowerConverter.Output.ExtendedVoltageMode", 0, 0, yes_no_info },
-	{ "input.frequency.extended", ST_FLAG_RW, 5, "UPS.PowerConverter.Output.ExtendedFrequencyMode", 0, 0, yes_no_info },
+	{ "input.voltage.extended", 0, 0, "UPS.PowerConverter.Output.ExtendedVoltageMode", 0, 0, yes_no_info },
+	{ "input.frequency.extended", 0, 0, "UPS.PowerConverter.Output.ExtendedFrequencyMode", 0, 0, yes_no_info },
 	/* same as "input.transfer.boost.low" */
-	{ "input.transfer.low", ST_FLAG_RW, 5, "UPS.PowerConverter.Output.LowVoltageTransfer", 0, 0, NULL },
-	{ "input.transfer.boost.low", ST_FLAG_RW, 5, "UPS.PowerConverter.Output.LowVoltageBoostTransfer", 0, 0, NULL },
-	{ "input.transfer.boost.high", ST_FLAG_RW, 5, "UPS.PowerConverter.Output.HighVoltageBoostTransfer", 0, 0, NULL },
-	{ "input.transfer.trim.low", ST_FLAG_RW, 5, "UPS.PowerConverter.Output.LowVoltageBuckTransfer", 0, 0, NULL },
+	{ "input.transfer.low", 0, 0, "UPS.PowerConverter.Output.LowVoltageTransfer", 0, 0, NULL },
+	{ "input.transfer.boost.low", 0, 0, "UPS.PowerConverter.Output.LowVoltageBoostTransfer", 0, 0, NULL },
+	{ "input.transfer.boost.high", 0, 0, "UPS.PowerConverter.Output.HighVoltageBoostTransfer", 0, 0, NULL },
+	{ "input.transfer.trim.low", 0, 0, "UPS.PowerConverter.Output.LowVoltageBuckTransfer", 0, 0, NULL },
 	/* same as "input.transfer.trim.high" */
-	{ "input.transfer.high", ST_FLAG_RW, 5, "UPS.PowerConverter.Output.HighVoltageTransfer", 0, 0, NULL },
-	{ "input.transfer.trim.high", ST_FLAG_RW, 5, "UPS.PowerConverter.Output.HighVoltageBuckTransfer", 0, 0, NULL },
-	{ "input.sensitivity", ST_FLAG_RW, 10, "UPS.PowerConverter.Output.SensitivityMode", 0, 0, mge_sensitivity_info },
+	{ "input.transfer.high", 0, 0, "UPS.PowerConverter.Output.HighVoltageTransfer", 0, 0, NULL },
+	{ "input.transfer.trim.high", 0, 0, "UPS.PowerConverter.Output.HighVoltageBuckTransfer", 0, 0, NULL },
+	{ "input.sensitivity", 0, 0, "UPS.PowerConverter.Output.SensitivityMode", 0, 0, mge_sensitivity_info },
 
 	/* Bypass page */
 	{ "input.bypass.voltage", 0, 0, "UPS.PowerConverter.Input[2].Voltage", 0, 0, NULL },
@@ -644,7 +682,7 @@ static xml_info_t mge_xml2nut[] = {
 	{ "output.L1-L2.voltage", 0, 0, "UPS.PowerConverter.Output.Phase[11].Voltage", 0, 0, ignore_if_zero },
 	{ "output.L2-L3.voltage", 0, 0, "UPS.PowerConverter.Output.Phase[22].Voltage", 0, 0, ignore_if_zero },
 	{ "output.L3-L1.voltage", 0, 0, "UPS.PowerConverter.Output.Phase[33].Voltage", 0, 0, ignore_if_zero },
-	{ "output.voltage.nominal", ST_FLAG_RW, 5, "UPS.Flow[4].ConfigVoltage", 0, 0, NULL },
+	{ "output.voltage.nominal", 0, 0, "UPS.Flow[4].ConfigVoltage", 0, 0, NULL },
 	{ "output.current", 0, 0, "UPS.PowerConverter.Output.Current", 0, 0, NULL },
 	{ "output.L1.current", 0, 0, "UPS.PowerConverter.Output.Phase[1].Current", 0, 0, convert_deci },
 	{ "output.L2.current", 0, 0, "UPS.PowerConverter.Output.Phase[2].Current", 0, 0, convert_deci },
@@ -658,39 +696,64 @@ static xml_info_t mge_xml2nut[] = {
 	{ "ambient.humidity", 0, 0, "Environment.Humidity", 0, 0, NULL },
 	{ "ambient.humidity.high", 0, 0, "Environment.Humidity.HighThreshold", 0, 0, NULL },
 	{ "ambient.humidity.low", 0, 0, "Environment.Humidity.LowThreshold", 0, 0, NULL },
-	{ "ambient.humidity.maximum", 0, 0, "Environment.PresentStatus.HighHumidity", 0, 0, NULL },
-	{ "ambient.humidity.minimum", 0, 0, "Environment.PresentStatus.LowHumidity", 0, 0, NULL },
+	{ "ambient.humidity.maximum", 0, 0, "Environment.PresentStatus.HighHumidity", 0, 0, mge_ambient_info },
+	{ "ambient.humidity.minimum", 0, 0, "Environment.PresentStatus.LowHumidity", 0, 0, mge_ambient_info },
 	{ "ambient.temperature", 0, 0, "Environment.Temperature", 0, 0, NULL },
 	{ "ambient.temperature.high", 0, 0, "Environment.Temperature.HighThreshold", 0, 0, NULL },
 	{ "ambient.temperature.low", 0, 0, "Environment.Temperature.LowThreshold", 0, 0, NULL },
-	{ "ambient.temperature.maximum", 0, 0, "Environment.PresentStatus.HighTemperature", 0, 0, NULL },
-	{ "ambient.temperature.minimum", 0, 0, "Environment.PresentStatus.LowTemperature", 0, 0, NULL },
+	{ "ambient.temperature.maximum", 0, 0, "Environment.PresentStatus.HighTemperature", 0, 0, mge_ambient_info },
+	{ "ambient.temperature.minimum", 0, 0, "Environment.PresentStatus.LowTemperature", 0, 0, mge_ambient_info },
 
 	/* Outlet page (using MGE UPS SYSTEMS - PowerShare technology) */
 	{ "outlet.id", 0, 0, "UPS.OutletSystem.Outlet[1].OutletID", 0, 0, NULL },
-	{ "outlet.desc", ST_FLAG_RW, 20, "UPS.OutletSystem.Outlet[1].iName", 0, 0, NULL },
+	{ "outlet.desc", 0, 0, "UPS.OutletSystem.Outlet[1].iName", 0, 0, NULL },
 	{ "outlet.switchable", 0, 0, "UPS.OutletSystem.Outlet[1].PresentStatus.Switchable", 0, 0, yes_no_info },
-	/* -> XML variable System.ShutdownDuration [120] doesn't map to any NUT variable */
 
 	{ "outlet.1.id", 0, 0, "UPS.OutletSystem.Outlet[2].OutletID", 0, 0, NULL },
-	{ "outlet.1.desc", ST_FLAG_RW, 20, "UPS.OutletSystem.Outlet[2].iName", 0, 0, NULL },
+	{ "outlet.1.desc", 0, 0, "UPS.OutletSystem.Outlet[2].iName", 0, 0, NULL },
 	{ "outlet.1.switchable", 0, 0, "UPS.OutletSystem.Outlet[2].PresentStatus.Switchable", 0, 0, yes_no_info },
 	{ "outlet.1.status", 0, 0, "UPS.OutletSystem.Outlet[2].PresentStatus.SwitchOnOff", 0, 0, on_off_info },
 	/* For low end models, with 1 non backup'ed outlet */
 	{ "outlet.1.status", 0, 0, "UPS.PowerSummary.PresentStatus.ACPresent", 0, 0, NULL }, /* on_off_info */
-	{ "outlet.1.autoswitch.charge.low", ST_FLAG_RW, 3, "UPS.OutletSystem.Outlet[2].RemainingCapacityLimit", 0, 0, NULL },
-	{ "outlet.1.delay.shutdown", ST_FLAG_RW, 5, "UPS.OutletSystem.Outlet[2].ShutdownTimer", 0, 0, NULL },
-	{ "outlet.1.delay.start", ST_FLAG_RW, 5, "UPS.OutletSystem.Outlet[2].StartupTimer", 0, 0, NULL },
-	/* -> XML variable System.Outlet[2].ShutdownDuration [120] doesn't map to any NUT variable */
+	{ "outlet.1.battery.charge.low", 0, 0, "UPS.OutletSystem.Outlet[2].RemainingCapacityLimit", 0, 0, NULL },
+	{ "outlet.1.timer.start", 0, 0, "UPS.OutletSystem.Outlet[2].DelayBeforeStartup", 0, 0, NULL },
+	{ "outlet.1.timer.shutdown", 0, 0, "UPS.OutletSystem.Outlet[2].DelayBeforeShutdown", 0, 0, NULL },
+	{ "outlet.1.delay.start", 0, 0, "UPS.OutletSystem.Outlet[2].StartupTimer", 0, 0, NULL },
+	/* { "outlet.1.delay.shutdown", 0, 0, "UPS.OutletSystem.Outlet[2].ShutdownTimer", 0, 0, NULL }, */
+	{ "outlet.1.delay.shutdown", 0, 0, "System.Outlet[2].ShutdownDuration", 0, 0, NULL },
 
 	{ "outlet.2.id", 0, 0, "UPS.OutletSystem.Outlet[3].OutletID", 0, 0, NULL },
-	{ "outlet.2.desc", ST_FLAG_RW, 20, "UPS.OutletSystem.Outlet[3].iName", 0, 0, NULL },
+	{ "outlet.2.desc", 0, 0, "UPS.OutletSystem.Outlet[3].iName", 0, 0, NULL },
 	{ "outlet.2.switchable", 0, 0, "UPS.OutletSystem.Outlet[3].PresentStatus.Switchable", 0, 0, yes_no_info },
 	{ "outlet.2.status", 0, 0, "UPS.OutletSystem.Outlet[3].PresentStatus.SwitchOnOff", 0, 0, on_off_info },
-	{ "outlet.2.autoswitch.charge.low", ST_FLAG_RW, 3, "UPS.OutletSystem.Outlet[3].RemainingCapacityLimit", 0, 0, NULL },
-	{ "outlet.2.delay.shutdown", ST_FLAG_RW, 5, "UPS.OutletSystem.Outlet[3].ShutdownTimer", 0, 0, NULL },
-	{ "outlet.2.delay.start", ST_FLAG_RW, 5, "UPS.OutletSystem.Outlet[3].StartupTimer", 0, 0, NULL },
-	/* -> XML variable System.Outlet[3].ShutdownDuration [120] doesn't map to any NUT variable */
+	{ "outlet.2.battery.charge.low", 0, 0, "UPS.OutletSystem.Outlet[3].RemainingCapacityLimit", 0, 0, NULL },
+	{ "outlet.2.timer.start", 0, 0, "UPS.OutletSystem.Outlet[3].DelayBeforeStartup", 0, 0, NULL },
+	{ "outlet.2.timer.shutdown", 0, 0, "UPS.OutletSystem.Outlet[3].DelayBeforeShutdown", 0, 0, NULL },
+	{ "outlet.2.delay.start", 0, 0, "UPS.OutletSystem.Outlet[3].StartupTimer", 0, 0, NULL },
+	/* { "outlet.2.delay.shutdown", 0, 0, "UPS.OutletSystem.Outlet[3].ShutdownTimer", 0, 0, NULL }, */
+	{ "outlet.2.delay.shutdown", 0, 0, "System.Outlet[3].ShutdownDuration", 0, 0, NULL },
+
+	/* For newer ePDU Monitored */
+	{ "outlet.1.desc", 0, 0, "PDU.OutletSystem.Outlet[1].iName", 0, 0, NULL },
+	{ "outlet.1.current", 0, 0, "PDU.OutletSystem.Outlet[1].Current", 0, 0, convert_deci },
+	/* FIXME: also map these?
+	 * "PDU.OutletSystem.Outlet[1].CurrentLimit" => settable, triggers CurrentTooHigh
+	 * "PDU.OutletSystem.Outlet[1].PresentStatus.CurrentTooHigh" (0/1)
+	 */
+	{ "outlet.2.desc", 0, 0, "PDU.OutletSystem.Outlet[2].iName", 0, 0, NULL },
+	{ "outlet.2.current", 0, 0, "PDU.OutletSystem.Outlet[2].Current", 0, 0, convert_deci },
+	{ "outlet.3.desc", 0, 0, "PDU.OutletSystem.Outlet[3].iName", 0, 0, NULL },
+	{ "outlet.3.current", 0, 0, "PDU.OutletSystem.Outlet[3].Current", 0, 0, convert_deci },
+	{ "outlet.4.desc", 0, 0, "PDU.OutletSystem.Outlet[2].iName", 0, 0, NULL },
+	{ "outlet.4.current", 0, 0, "PDU.OutletSystem.Outlet[2].Current", 0, 0, convert_deci },
+	{ "outlet.5.desc", 0, 0, "PDU.OutletSystem.Outlet[3].iName", 0, 0, NULL },
+	{ "outlet.5.current", 0, 0, "PDU.OutletSystem.Outlet[3].Current", 0, 0, convert_deci },
+	{ "outlet.6.desc", 0, 0, "PDU.OutletSystem.Outlet[2].iName", 0, 0, NULL },
+	{ "outlet.6.current", 0, 0, "PDU.OutletSystem.Outlet[2].Current", 0, 0, convert_deci },
+	{ "outlet.7.desc", 0, 0, "PDU.OutletSystem.Outlet[3].iName", 0, 0, NULL },
+	{ "outlet.7.current", 0, 0, "PDU.OutletSystem.Outlet[3].Current", 0, 0, convert_deci },
+	{ "outlet.8.desc", 0, 0, "PDU.OutletSystem.Outlet[2].iName", 0, 0, NULL },
+	{ "outlet.8.current", 0, 0, "PDU.OutletSystem.Outlet[2].Current", 0, 0, convert_deci },
 
 	{ NULL, 0, 0, NULL, 0, 0, NULL }
 };
@@ -706,12 +769,29 @@ static int mge_xml_startelm_cb(void *userdata, int parent, const char *nspace, c
 		if (!strcasecmp(name, "PRODUCT_INFO")) {
 			/* name="Network Management Card" type="Mosaic M" version="BA" */
 			/* name="Network Management Card" type="Transverse" version="GB (SN 49EH29101)" */
+			/* name="Monitored ePDU" type="Monitored ePDU" version="Version Upgrade" */
 			int	i;
 			for (i = 0; atts[i] && atts[i+1]; i += 2) {
-				if (i == 0) {
+				if (!strcasecmp(atts[i], "name")) {
 					snprintf(val, sizeof(val), "%s", atts[i+1]);
-				} else {
+				}
+				if (!strcasecmp(atts[i], "type")) {
 					snprintfcat(val, sizeof(val), "/%s", atts[i+1]);
+					if (!strcasecmp(atts[i+1], "Transverse")) {
+						mge_ambient_value = 1;
+					} else if (strstr(atts[i+1], "ePDU")) {
+						dstate_setinfo("device.type", "pdu");
+					}
+				}
+				if (!strcasecmp(atts[i], "version")) {
+					char	*s;
+					snprintfcat(val, sizeof(val), "/%s", atts[i+1]);
+					s = strstr(val, " (SN ");
+					if (s) {
+						dstate_setinfo("ups.serial", "%s", rtrim(s + 5, ')'));
+						s[0] = '\0';
+					}
+					dstate_setinfo("ups.firmware.aux", "%s", val);
 				}
 			}
 			state = PRODUCT_INFO;
@@ -729,6 +809,27 @@ static int mge_xml_startelm_cb(void *userdata, int parent, const char *nspace, c
 			state = SET_OBJECT;
 			break;
 		}
+		if (!strcasecmp(name, "ALARM")) {
+			int	i;
+			var[0] = val[0] = '\0';
+			for (i = 0; atts[i] && atts[i+1]; i += 2) {
+				if (!strcasecmp(atts[i], "object")) {
+					snprintf(var, sizeof(var), "%s", atts[i+1]);
+				}
+				if (!strcasecmp(atts[i], "value")) {
+					snprintf(val, sizeof(var), "%s", atts[i+1]);
+				}
+				if (!strcasecmp(atts[i], "date")) {
+					dstate_setinfo("ups.time", "%s", split_date_time(atts[i+1]));
+				}
+			}
+			state = ALARM;
+			break;
+		}
+		if (!strcasecmp(name, "XML-CLIENT")) {
+			state = XML_CLIENT;
+			break;
+		}
 		break;
 
 	case PRODUCT_INFO:
@@ -747,7 +848,8 @@ static int mge_xml_startelm_cb(void *userdata, int parent, const char *nspace, c
 			break;
 		}
 
-		if (!strcasecmp(name, "UPS_DATA")) {
+		if ( (!strcasecmp(name, "UPS_DATA"))
+			|| (!strcasecmp(name, "DEV_DATA")) ) {
 			state = PI_UPS_DATA;
 			break;
 		}
@@ -760,12 +862,12 @@ static int mge_xml_startelm_cb(void *userdata, int parent, const char *nspace, c
 			break;
 		}
 		if (!strcasecmp(name, "XML_SUMMARY_PAGE")) {
-			/* url="upsprop.xml" */
+			/* url="upsprop.xml" or url="ws/summary.xml" */
 			int	i;
 			for (i = 0; atts[i] && atts[i+1]; i += 2) {
 				if (!strcasecmp(atts[i], "url")) {
-					free(mge_xml_subdriver.getobject);
-					mge_xml_subdriver.getobject = strdup(url_convert(atts[i+1]));
+					free(mge_xml_subdriver.summary);
+					mge_xml_subdriver.summary = strdup(url_convert(atts[i+1]));
 				}
 			}
 			state = PI_XML_SUMMARY_PAGE;
@@ -773,6 +875,13 @@ static int mge_xml_startelm_cb(void *userdata, int parent, const char *nspace, c
 		}
 		if (!strcasecmp(name, "CENTRAL_CFG")) {
 			/* url="config.xml" */
+			int	i;
+			for (i = 0; atts[i] && atts[i+1]; i += 2) {
+				if (!strcasecmp(atts[i], "url")) {
+					free(mge_xml_subdriver.configure);
+					mge_xml_subdriver.configure = strdup(url_convert(atts[i+1]));
+				}
+			}
 			state = PI_CENTRAL_CFG;
 			break;
 		}
@@ -786,6 +895,13 @@ static int mge_xml_startelm_cb(void *userdata, int parent, const char *nspace, c
 	case PI_ALARMS:
 		if (!strcasecmp(name, "SUBSCRIPTION")) {
 			/* url="subscribe.cgi" security="basic" */
+			int	i;
+			for (i = 0; atts[i] && atts[i+1]; i += 2) {
+				if (!strcasecmp(atts[i], "url")) {
+					free(mge_xml_subdriver.subscribe);
+					mge_xml_subdriver.subscribe = strdup(url_convert(atts[i+1]));
+				}
+			}
 			state = PI_SUBSCRIPTION;
 			break;
 		}
@@ -855,7 +971,7 @@ static int mge_xml_startelm_cb(void *userdata, int parent, const char *nspace, c
 			break;
 		}
 		break;
-			
+
 	case GET_OBJECT:
 		if (!strcasecmp(name, "OBJECT")) {
 			/* name="System.RunTimeToEmptyLimit" unit="s" access="RW" */
@@ -873,9 +989,41 @@ static int mge_xml_startelm_cb(void *userdata, int parent, const char *nspace, c
 			break;
 		}
 		break;
+
+	case XML_CLIENT:
+		if (!strcasecmp(name, "GENERAL")) {
+			state = XC_GENERAL;
+			break;
+		}
+
+	case XC_GENERAL:
+		if (!strcasecmp(name, "STARTUP")) {
+			/* config="CENTRALIZED" */
+			state = XC_STARTUP;
+			break;
+		}
+		if (!strcasecmp(name, "SHUTDOWN")) {
+			/* shutdownTimer="NONE" shutdownDuration="150" */
+			int	i;
+			for (i = 0; atts[i] && atts[i+1]; i += 2) {
+				if (!strcasecmp(atts[i], "shutdownTimer")) {
+					dstate_setinfo("driver.timer.shutdown", "%s", atts[i+1]);
+				}
+				if (!strcasecmp(atts[i], "shutdownDuration")) {
+					dstate_setinfo("driver.delay.shutdown", "%s", atts[i+1]);
+				}
+			}
+			state = XC_SHUTDOWN;
+			break;
+		}
+		if (!strcasecmp(name, "BROADCAST")) {
+			/* admins="ON" users="ON" */
+			state = XC_BROADCAST;
+			break;
+		}
 	}
 
-	upsdebugx(3, "%s: name <%s> (parent = %d, state = %d)\n", __func__, name, parent, state);
+	upsdebugx(3, "%s: name <%s> (parent = %d, state = %d)", __func__, name, parent, state);
 	return state;
 }
 
@@ -884,17 +1032,21 @@ static int mge_xml_cdata_cb(void *userdata, int state, const char *cdata, size_t
 {
 	/* skip empty lines */
 	if ((len == 1) && (cdata[0] == '\n')) {
-		upsdebugx(3, "%s: cdata ignored (state = %d)\n", __func__, state);
+		upsdebugx(3, "%s: cdata ignored (state = %d)", __func__, state);
 		return 0;
 	}
 
-	upsdebugx(3, "%s: cdata [%.*s] (state = %d)\n", __func__, len, cdata, state);
+	upsdebugx(3, "%s: cdata [%.*s] (state = %d)", __func__, (int)len, cdata, state);
 
 	switch(state)
 	{
+	case ALARM:
+		upsdebugx(2, "ALARM%.*s", (int)len, cdata);
+		break;
+
 	case SU_OBJECT:
 	case GO_OBJECT:
-		snprintf(val, sizeof(val), "%.*s", len, cdata);
+		snprintfcat(val, sizeof(val), "%.*s", (int)len, cdata);
 		break;
 	}
 
@@ -905,31 +1057,19 @@ static int mge_xml_cdata_cb(void *userdata, int state, const char *cdata, size_t
 static int mge_xml_endelm_cb(void *userdata, int state, const char *nspace, const char *name)
 {
 	xml_info_t	*info;
-	char		*value;
+	const char	*value;
 
 	/* ignore objects for which no value was set */
 	if (strlen(val) == 0) {
-		upsdebugx(3, "%s: name ignored, no value set (state = %d)\n", __func__, state);
+		upsdebugx(3, "%s: name </%s> ignored, no value set (state = %d)", __func__, name, state);
 		return 0;
 	}
 
-	upsdebugx(3, "%s: name </%s> (state = %d)\n", __func__, name, state);
+	upsdebugx(3, "%s: name </%s> (state = %d)", __func__, name, state);
 
 	switch(state)
 	{
-	case PRODUCT_INFO:
-		dstate_setinfo("ups.mfr", "MGE UPS SYSTEMS");
-
-		/* Some devices also return the serial number here */
-		value = strstr(val, " (SN ");
-		if (value) {
-			dstate_setinfo("ups.serial", "%s", rtrim(value + 5, ')'));
-			value[0] = '\0';
-		}
-
-		dstate_setinfo("ups.firmware.aux", "%s", val);
-		break;
-
+	case ALARM:
 	case SU_OBJECT:
 	case GO_OBJECT:
 		for (info = mge_xml2nut; info->xmlname != NULL; info++) {
@@ -939,6 +1079,10 @@ static int mge_xml_endelm_cb(void *userdata, int state, const char *nspace, cons
 
 			upsdebugx(3, "-> XML variable %s [%s] maps to NUT variable %s", var, val, info->nutname);
 
+			if ((info->nutflags & ST_FLAG_STATIC) && dstate_getinfo(info->nutname)) {
+				return 0;
+			}
+
 			if (info->convert) {
 				value = info->convert(val);
 			} else {
@@ -955,7 +1099,7 @@ static int mge_xml_endelm_cb(void *userdata, int state, const char *nspace, cons
 		upsdebugx(3, "-> XML variable %s [%s] doesn't map to any NUT variable", var, val);
 		break;
 	}
-	
+
 	return 0;
 }
 
@@ -965,6 +1109,9 @@ subdriver_t mge_xml_subdriver = {
 	MGE_XML_INITINFO,
 	NULL,
 	NULL,
+	NULL,
+	NULL,
+	NULL,
 	mge_xml_startelm_cb,
 	mge_xml_cdata_cb,
 	mge_xml_endelm_cb,
