@@ -1,7 +1,7 @@
 /* netxml-ups.c	Driver routines for network XML UPS units 
 
    Copyright (C)
-	2008		Arjen de Korte <adkorte-guest@alioth.debian.org>
+	2008-2009	Arjen de Korte <adkorte-guest@alioth.debian.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,17 +25,20 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <ne_request.h>
 #include <ne_basic.h>
 #include <ne_props.h>
 #include <ne_uri.h>
+#include <ne_xml.h>
 #include <ne_xmlreq.h>
 #include <ne_ssl.h>
 #include <ne_auth.h>
+#include <ne_socket.h>
 
-#define DRIVER_NAME	"network XML UPS driver"
-#define DRIVER_VERSION	"0.21"
+#define DRIVER_NAME	"network XML UPS"
+#define DRIVER_VERSION	"0.30"
 
 /* driver description structure */
 upsdrv_info_t	upsdrv_info = {
@@ -49,17 +52,13 @@ upsdrv_info_t	upsdrv_info = {
  * "built with neon library %s" LIBNEON_VERSION 
  * subdrivers (limited to MGE only ATM) */
 
-#define MAXRETRIES	5
-
-#ifdef DEBUG
-#define difftimeval(x,y)	((double)y.tv_sec-x.tv_sec+((double)(y.tv_usec-x.tv_usec)/1000000))
-#endif
-
 /* Global vars */
 uint32_t		ups_status = 0;
 static int		timeout = 5;
+static time_t		lastheard = 0;
 static subdriver_t	*subdriver = &mge_xml_subdriver;
 static ne_session	*session = NULL;
+static ne_socket	*sock = NULL;
 static ne_uri		uri;
 
 /* Support functions */
@@ -67,8 +66,13 @@ static void netxml_alarm_set(void);
 static void netxml_status_set(void);
 static int netxml_authenticate(void *userdata, const char *realm, int attempt, char *username, char *password);
 static int netxml_dispatch_request(ne_request *request, ne_xml_parser *parser);
+static int netxml_get_page(const char *page);
 
-#ifndef HAVE_LIBNEON_SET_CONNECT_TIMEOUT
+static int netxml_alarm_subscribe(const char *page);
+
+#if HAVE_NE_SET_CONNECT_TIMEOUT && HAVE_NE_SOCK_CONNECT_TIMEOUT
+	/* we don't need to use alarm() */
+#else
 static void netxml_alarm_handler(int sig)
 {
 	/* don't do anything here, just return */
@@ -77,96 +81,94 @@ static void netxml_alarm_handler(int sig)
 
 void upsdrv_initinfo(void)
 {
-	int		ret;
-	ne_request	*request;
-	ne_xml_parser	*parser;
+	char	*page, *last = NULL;
+	char	buf[SMALLBUF];
+	
+	snprintf(buf, sizeof(buf), "%s", subdriver->initinfo);
 
-	if (strlen(subdriver->initinfo) < 1) {
-		fatalx(EXIT_FAILURE, "%s: failure to read initinfo element", __func__);
-	}
+	for (page = strtok_r(buf, " ", &last); page != NULL; page = strtok_r(NULL, " ", &last)) {
 
-	upsdebugx(3, "ne_request_create(session, \"GET\", \"%s\");", subdriver->initinfo);
+		if (netxml_get_page(page) != NE_OK) {
+			continue;
+		}
 
-	request = ne_request_create(session, "GET", subdriver->initinfo);
+		dstate_setinfo("driver.version.internal", "%s", subdriver->version);
 
-	parser = ne_xml_create();
-
-	ne_xml_push_handler(parser, subdriver->startelm_cb, subdriver->cdata_cb, subdriver->endelm_cb, NULL);
-
-	ret = netxml_dispatch_request(request, parser);
-
-	ne_xml_destroy(parser);
-	ne_request_destroy(request);
-
-	if (ret != NE_OK) {
-		fatalx(EXIT_FAILURE, "%s: communication failure [%s]", __func__, ne_get_error(session));
-	}
-
-	if (!subdriver->getobject) {
-		fatalx(EXIT_FAILURE, "%s: found no way to get variables", __func__);
-	}
-
-	if (!subdriver->setobject) {
-		upsdebugx(2, "%s: found no way to set variables", __func__);
-	}
-
-	dstate_setinfo("driver.version.internal", "%s", subdriver->version);
-
-	/* upsh.instcmd = instcmd; */
-	/* upsh.setvar = setvar; */
-}
-
-void upsdrv_updateinfo(void)
-{
-	static int	retries = 0;
-	int		ret;
-	ne_request	*request;
-	ne_xml_parser	*parser;
-#ifdef DEBUG
-	struct timeval	start, stop;
-#endif
-
-	if (strlen(subdriver->getobject) < 1) {
-		fatalx(EXIT_FAILURE, "%s: failure to read getobject element", __func__);
-	}
-
-	upsdebugx(3, "ne_request_create(session, \"GET\", \"%s\");", subdriver->getobject);
-
-	request = ne_request_create(session, "GET", subdriver->getobject);
-
-	parser = ne_xml_create();
-
-	ne_xml_push_handler(parser, subdriver->startelm_cb, subdriver->cdata_cb, subdriver->endelm_cb, NULL);
-
-#ifdef DEBUG
-	gettimeofday(&start, NULL);
-#endif
-	ret = netxml_dispatch_request(request, parser);
-#ifdef DEBUG
-	gettimeofday(&stop, NULL);
-
-	upsdebugx(1, "dispatch request (%.3f seconds)", difftimeval(start, stop));
-#endif
-
-	ne_xml_destroy(parser);
-	ne_request_destroy(request);
-
-	if (ret != NE_OK) {
-#ifdef DEBUG
-		upslogx(LOG_ERR, "%s (%.3f seconds)", ne_get_error(session), difftimeval(start, stop));
-#else
-		upsdebugx(1, "%s (%d from %d)", ne_get_error(session), retries, MAXRETRIES);
-#endif
-		if (retries < MAXRETRIES) {
-			retries++;
-		} else {
-			dstate_datastale();
+		if (testvar("subscribe") && (netxml_alarm_subscribe(subdriver->subscribe) == NE_OK)) {
+			extrafd = ne_sock_fd(sock);
+			time(&lastheard);
 		}
 
 		return;
 	}
 
-	retries = 0;
+	fatalx(EXIT_FAILURE, "%s: communication failure [%s]", __func__, ne_get_error(session));
+}
+
+void upsdrv_updateinfo(void)
+{
+	int	ret, errors = 0;
+
+	/* We really should be dealing with alarms through a separate callback, so that we can keep the
+	 * processing of alarms and polling for data separated. Currently, this isn't supported by the
+	 * driver main body, so we'll have to revert to polling each time we're called, unless the
+	 * socket indicates we're no longer connected.
+	 */
+	if (testvar("subscribe")) {
+		char	buf[LARGEBUF];
+
+		ret = ne_sock_read(sock, buf, sizeof(buf));
+
+		if (ret > 0) {
+			/* alarm message received */
+
+			ne_xml_parser	*parser = ne_xml_create();
+			upsdebugx(2, "%s: ne_sock_read(%d bytes) => %s", __func__, ret, buf);
+			ne_xml_push_handler(parser, subdriver->startelm_cb, subdriver->cdata_cb, subdriver->endelm_cb, NULL);
+			ne_xml_parse(parser, buf, strlen(buf));
+			ne_xml_destroy(parser);
+			time(&lastheard);
+
+		} else if ((ret == NE_SOCK_TIMEOUT) && (difftime(time(NULL), lastheard) < 180)) {
+			/* timed out */
+
+			upsdebugx(2, "%s: ne_sock_read(timeout)", __func__);
+
+		} else {
+			/* connection closed or unknown error */
+
+			upslogx(LOG_ERR, "NSM connection with '%s' lost", uri.host);
+
+			upsdebugx(2, "%s: ne_sock_read(%d) => %s", __func__, ret, ne_sock_error(sock));
+			ne_sock_close(sock);
+
+			if (netxml_alarm_subscribe(subdriver->subscribe) == NE_OK) {
+				extrafd = ne_sock_fd(sock);
+				time(&lastheard);
+				return;
+			}
+
+			dstate_datastale();
+			extrafd = -1;
+			return;
+		}
+	}
+
+	/* get additional data */
+	ret = netxml_get_page(subdriver->getobject);
+	if (ret != NE_OK) {
+		errors++;
+	}
+
+	ret = netxml_get_page(subdriver->summary);
+	if (ret != NE_OK) {
+		errors++;
+	}
+
+	if (errors > 1) {
+		dstate_datastale();
+		return;
+	}
 
 	status_init();
 
@@ -206,7 +208,7 @@ static int instcmd(const char *cmdname, const char *extra)
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+	upslogx(LOG_NOTICE, "%s: unknown command [%s]", __func__, cmdname);
 	return STAT_INSTCMD_UNKNOWN;
 }
 */
@@ -219,7 +221,7 @@ static int setvar(const char *varname, const char *val)
 		return STAT_SET_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "setvar: unknown variable [%s]", varname);
+	upslogx(LOG_NOTICE, "%s: unknown variable [%s]", __func__, varname);
 	return STAT_SET_UNKNOWN;
 }
 */
@@ -233,9 +235,11 @@ void upsdrv_makevartable(void)
 {
 	char	buf[SMALLBUF];
 
-	snprintf(buf, sizeof(buf), "network timeout (default %d seconds)", timeout);
+	snprintf(buf, sizeof(buf), "network timeout (default: %d seconds)", timeout);
 	addvar(VAR_VALUE, "timeout", buf);
 
+	addvar(VAR_FLAG, "subscribe", "NSM subscribe to NMC");
+
 	addvar(VAR_VALUE | VAR_SENSITIVE, "login", "login value for authenticated mode");
 	addvar(VAR_VALUE | VAR_SENSITIVE, "password", "password value for authenticated mode");
 }
@@ -246,7 +250,9 @@ void upsdrv_initups(void)
 	char	*val;
 	FILE	*fp;
 
-#ifndef HAVE_LIBNEON_SET_CONNECT_TIMEOUT
+#if HAVE_NE_SET_CONNECT_TIMEOUT && HAVE_NE_SOCK_CONNECT_TIMEOUT
+	/* we don't need to use alarm() */
+#else
 	struct sigaction	sa;
 
 	sigemptyset(&sa.sa_mask);
@@ -257,7 +263,6 @@ void upsdrv_initups(void)
 #endif
 	/* allow override of default network timeout value */
 	val = getval("timeout");
-
 	if (val) {
 		timeout = atoi(val);
 
@@ -266,6 +271,10 @@ void upsdrv_initups(void)
 		}
 	}
 
+	if (nut_debug_level > 5) {
+		ne_debug_init(stderr, NE_DBG_HTTP | NE_DBG_HTTPBODY);
+	}
+
 	if (ne_sock_init()) {
 		fatalx(EXIT_FAILURE, "%s: failed to initialize socket libraries", progname);
 	}
@@ -289,9 +298,9 @@ void upsdrv_initups(void)
 	upsdebugx(1, "using %s://%s port %d", uri.scheme, uri.host, uri.port);
 
 	session = ne_session_create(uri.scheme, uri.host, uri.port);
-
+	
 	/* timeout if we can't (re)connect to the UPS */
-#ifdef HAVE_LIBNEON_SET_CONNECT_TIMEOUT
+#ifdef HAVE_NE_SET_CONNECT_TIMEOUT
 	ne_set_connect_timeout(session, timeout);
 #endif
 
@@ -335,9 +344,16 @@ void upsdrv_initups(void)
 
 void upsdrv_cleanup(void)
 {
+	free(subdriver->configure);
+	free(subdriver->subscribe);
+	free(subdriver->summary);
 	free(subdriver->getobject);
 	free(subdriver->setobject);
 
+	if (sock) {
+		ne_sock_close(sock);
+	}
+
 	if (session) {
 		ne_session_destroy(session);
 	}
@@ -349,11 +365,184 @@ void upsdrv_cleanup(void)
  * Support functions
  *********************************************************************/
 
+static int netxml_get_page(const char *page)
+{
+	int		ret;
+	ne_request	*request;
+	ne_xml_parser	*parser;
+
+	upsdebugx(2, "%s: %s", __func__, page);
+
+	request = ne_request_create(session, "GET", page);
+
+	parser = ne_xml_create();
+
+	ne_xml_push_handler(parser, subdriver->startelm_cb, subdriver->cdata_cb, subdriver->endelm_cb, NULL);
+
+	ret = netxml_dispatch_request(request, parser);
+
+	if (ret) {
+		upsdebugx(2, "%s: %s", __func__, ne_get_error(session));
+	}
+
+	ne_xml_destroy(parser);
+	ne_request_destroy(request);
+
+	return ret;
+}
+
+static int netxml_alarm_subscribe(const char *page)
+{
+	int	ret, port = -1, secret = -1;
+	char	buf[LARGEBUF], *s;
+	ne_request	*request;
+	ne_sock_addr	*addr;
+	const ne_inet_addr	*ai;
+
+	upsdebugx(2, "%s: %s", __func__, page);
+
+	sock = ne_sock_create();
+
+	if (gethostname(buf, sizeof(buf)) == 0) {
+		dstate_setinfo("driver.hostname", "%s", buf);
+	} else {
+		dstate_setinfo("driver.hostname", "<unknown>");
+	}
+
+#ifdef HAVE_NE_SOCK_CONNECT_TIMEOUT
+	ne_sock_connect_timeout(sock, timeout);
+#endif
+	ne_sock_read_timeout(sock, 1);
+
+	netxml_get_page(subdriver->configure);
+
+	snprintf(buf, sizeof(buf),	"<?xml version=\"1.0\">\n");
+	snprintfcat(buf, sizeof(buf),	"<Subscribe>\n");
+	snprintfcat(buf, sizeof(buf),		"<Class>%s v%s</Class>\n", progname, DRIVER_VERSION);
+	snprintfcat(buf, sizeof(buf),		"<Type>connected socket</Type>\n");
+	snprintfcat(buf, sizeof(buf),		"<HostName>%s</HostName>\n", dstate_getinfo("driver.hostname"));
+	snprintfcat(buf, sizeof(buf),		"<XMLClientParameters>\n");
+	snprintfcat(buf, sizeof(buf),			"<ShutdownDuration>%s</ShutdownDuration>\n", dstate_getinfo("driver.delay.shutdown"));
+	snprintfcat(buf, sizeof(buf),			"<ShutdownTimer>%s</ShutdownTimer>\n", dstate_getinfo("driver.timer.shutdown"));
+	snprintfcat(buf, sizeof(buf),			"<AutoConfig>CENTRALIZED</AutoConfig>\n");
+	snprintfcat(buf, sizeof(buf),			"<OutletGroup>1</OutletGroup>\n");
+	snprintfcat(buf, sizeof(buf),		"</XMLClientParameters>\n");
+/*	snprintfcat(buf, sizeof(buf),		"<Warning>NUT driver</Warning>\n"); */
+	snprintfcat(buf, sizeof(buf),	"</Subscribe>\n");
+
+	/* now send subscription message setting all the proper flags */
+	request = ne_request_create(session, "POST", page);
+	ne_set_request_body_buffer(request, buf, strlen(buf));
+
+	/* as the NMC reply is not xml standard compliant let's parse it this way */
+	do {
+#ifndef HAVE_NE_SOCK_CONNECT_TIMEOUT
+		alarm(timeout+1);
+#endif
+		ret = ne_begin_request(request);
+
+#ifndef HAVE_NE_SOCK_CONNECT_TIMEOUT
+		alarm(0);
+#endif
+		if (ret != NE_OK) {
+			break;
+		}
+
+		ret = ne_read_response_block(request, buf, sizeof buf);
+
+		if (ret == NE_OK) {
+			ret = ne_end_request(request);
+		}
+
+	} while (ret == NE_RETRY);
+
+	ne_request_destroy(request);
+
+	/* due to different formats used by the various NMCs, we need to\
+	   break up the reply in lines and parse each one separately */
+	for (s = strtok(buf, "\r\n"); s != NULL; s = strtok(NULL, "\r\n")) {
+		upsdebugx(2, "%s: parsing %s", __func__, s);
+
+		if (!strncasecmp(s, "<Port>", 6) && (sscanf(s+6, "%u", &port) != 1)) {
+			return NE_RETRY;
+		}
+
+		if (!strncasecmp(s, "<Secret>", 8) && (sscanf(s+8, "%u", &secret) != 1)) {
+			return NE_RETRY;
+		}
+	}
+
+	if ((port == -1) || (secret == -1)) {
+		upsdebugx(2, "%s: parsing initial subcription failed", __func__);
+		return NE_RETRY;
+	}
+
+	/* Resolve the given hostname.  'flags' must be zero.  Hex
+	* string IPv6 addresses (e.g. `::1') may be enclosed in brackets
+	* (e.g. `[::1]'). */
+	addr = ne_addr_resolve(uri.host, 0);
+
+	/* Returns zero if name resolution was successful, non-zero on
+	* error. */
+	if (ne_addr_result(addr) != 0) {
+		upsdebugx(2, "%s: name resolution failure on %s: %s", __func__, uri.host, ne_addr_error(addr, buf, sizeof(buf)));
+		ne_addr_destroy(addr);
+		return NE_RETRY;
+	}
+
+	for (ai = ne_addr_first(addr); ai != NULL; ai = ne_addr_next(addr)) {
+
+		upsdebugx(2, "%s: connecting to host %s port %d", __func__, ne_iaddr_print(ai, buf, sizeof(buf)), port);
+
+#ifndef HAVE_NE_SOCK_CONNECT_TIMEOUT
+		alarm(timeout+1);
+#endif
+		ret = ne_sock_connect(sock, ai, port);
+
+#ifndef HAVE_NE_SOCK_CONNECT_TIMEOUT
+		alarm(0);
+#endif
+		if (ret == NE_OK) {
+			upsdebugx(2, "%s: connection to %s open on fd %d", __func__, uri.host, ne_sock_fd(sock));
+			break;
+		}
+	}
+
+	ne_addr_destroy(addr);
+
+	if (ai == NULL) {
+		upsdebugx(2, "%s: failed to create listening socket", __func__);
+		return NE_RETRY;
+	}
+
+	snprintf(buf, sizeof(buf), "<Subscription Identification=\"%u\"></Subscription>\n", secret);
+	ret = ne_sock_fullwrite(sock, buf, strlen(buf));
+
+	if (ret != NE_OK) {
+		upsdebugx(2, "%s: send failed: %s", __func__, ne_sock_error(sock));
+		return NE_RETRY;
+	}
+
+	ret = ne_sock_read(sock, buf, sizeof(buf));
+
+	if (ret < 1) {
+		upsdebugx(2, "%s: read failed: %s", __func__, ne_sock_error(sock));
+		return NE_RETRY;
+	}
+
+	if (strcasecmp(buf, "<Subscription Answer=\"ok\"></Subscription>")) {
+		upsdebugx(2, "%s: subscription rejected", __func__);
+		return NE_RETRY;
+	}
+
+	upslogx(LOG_INFO, "NSM connection to '%s' established", uri.host);
+	return NE_OK;
+}
+
 static int netxml_dispatch_request(ne_request *request, ne_xml_parser *parser)
 {
 	int ret;
 
-#ifdef HAVE_LIBNEON_SET_CONNECT_TIMEOUT
 	/*
 	 * Starting with neon-0.27.0 the ne_xml_dispatch_request() function will check
 	 * for a valid XML content-type (following RFC 3023 rules) in the header.
@@ -361,8 +550,14 @@ static int netxml_dispatch_request(ne_request *request, ne_xml_parser *parser)
 	 * we can't use this anymore and we'll have to roll our own here.
 	 */
 	do {
+#ifndef HAVE_NE_SET_CONNECT_TIMEOUT
+		alarm(timeout+1);
+#endif
 		ret = ne_begin_request(request);
 
+#ifndef HAVE_NE_SET_CONNECT_TIMEOUT
+		alarm(0);
+#endif
 		if (ret != NE_OK) {
 			break;
 		}
@@ -375,20 +570,6 @@ static int netxml_dispatch_request(ne_request *request, ne_xml_parser *parser)
 
 	} while (ret == NE_RETRY);
 
-#else
-	/*
-	 * Up to neon-0.27.0 this library would connect() in blocking mode. We
-	 * don't want that, so we interrupt it with an alarm if it takes longer
-	 * than we think is reasonable so that we don't have to wait until it
-	 * finally times out (which may take several minutes).
-	 */
-	alarm(timeout+1);
-
-	ret = ne_xml_dispatch_request(request, parser);
-
- 	alarm(0);
-#endif
-
 	return ret;
 }
 
@@ -476,7 +657,7 @@ static void netxml_status_set(void)
 	if (STATUS_BIT(CHRG)) {
 		status_set("CHRG");		/* charging */
 	}
-	if (STATUS_BIT(LOWBATT) || STATUS_BIT(SHUTDOWNIMM)) {
+	if (STATUS_BIT(LOWBATT)) {
 		status_set("LB");		/* low battery */
 	}
 	if (STATUS_BIT(OVERLOAD)) {
@@ -497,4 +678,9 @@ static void netxml_status_set(void)
 	if (STATUS_BIT(OFF)) {
 		status_set("OFF");		/* ups is off */
 	}
+
+	if (STATUS_BIT(SHUTDOWNIMM)) {
+		status_set("FSD");		/* shutdown imminent */
+	}
 }
+
