@@ -23,6 +23,23 @@
    ==================================================================================
 */
 
+#include "main.h"
+#include "serial.h"
+
+#define DRIVER_NAME	"Opti-UPS driver"
+#define DRIVER_VERSION "1.01"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Russell Kroll <rkroll@exploits.org>\n" \
+	"Scott Heavner <debian@scottheavner.com>\n" \
+	"Matthias Goebl <matthias.goebl@goebl.net>",
+	DRV_STABLE,
+	{ NULL }
+};
+
 #define HELP                                                                    "\n" \
    "**********************************************************"                 "\n" \
    "This driver has been tested only with the OPTI SAFE 420E using"             "\n" \
@@ -49,10 +66,6 @@
  * at least the older OPTI UPS models (420E, 820ES).
  */
 
-#include "main.h"
-#include "serial.h"
-#include "optiups.h"
-
 #define ENDCHAR		'\n'
 #define IGNCHARS	"\r"
 
@@ -155,7 +168,7 @@ static inline int optireadline()
 static inline int optiquery( const char *cmd )
 {
 	upsdebugx(2, "SEND: \"%s\"", cmd );
-	ser_send( upsfd, cmd );
+	ser_send( upsfd, "%s", cmd );
 	if ( optimodel == OPTIMODEL_ZINTO )
 		ser_send( upsfd, "\r\n" );
 	return optireadline();
@@ -195,7 +208,7 @@ static void optifill( ezfill_t *a, int len )
 		}
 		else
 		{
-			dstate_setinfo( a[i].var, _buf);
+			dstate_setinfo( a[i].var, "%s", _buf);
 		}
 	}
 }
@@ -301,8 +314,6 @@ void upsdrv_initinfo(void)
 {
 	int r;
 
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
-
 	/* If an Zinto Online-USV is off, switch it on first. */
 	/* It sends only "2" when off, without "\r\n", and doesn't */
 	/* answer other commands. Therefore without power we'll be */
@@ -382,11 +393,11 @@ void upsdrv_initinfo(void)
 
 	if ( optimodel == OPTIMODEL_ZINTO )
 	{
-		dstate_setinfo("outlet.0.desc", "%s", "Main Outlet 1+2");
+		dstate_setinfo("outlet.desc", "%s", "Main Outlet 1+2");
 		dstate_setinfo("outlet.1.desc", "%s", "Switchable Outlet 3+4");
-		dstate_setinfo("outlet.0.id", "%d", 1);
+		dstate_setinfo("outlet.id", "%d", 1);
 		dstate_setinfo("outlet.1.id", "%d", 2);
-		dstate_setinfo("outlet.0.switchable", "%d", 0);
+		dstate_setinfo("outlet.switchable", "%d", 0);
 		dstate_setinfo("outlet.1.switchable", "%d", 1);
 		dstate_setinfo("outlet.1.switch", "%d", 1);
 		dstate_setflags("outlet.1.switch", ST_FLAG_RW | ST_FLAG_STRING);
@@ -529,12 +540,6 @@ void upsdrv_makevartable(void)
 	addvar(VAR_FLAG, OPTI_POWERUP, "(Zinto D) Power-up UPS at start (cannot identify a powered-down Zinto D)");
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Opti-UPS driver %s (%s)\n\n", 
-		DRV_VERSION, UPS_VERSION);
-}
-
 void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path);
