@@ -27,8 +27,6 @@
 
 */
 
-#define DRV_VERSION "0.50"
-
 #include <stdio.h>
 #include <math.h>
 
@@ -36,6 +34,18 @@
 #include "serial.h"
 #include "timehead.h"
 
+#define DRIVER_NAME		"Microsol Rhino UPS driver"
+#define DRIVER_VERSION	"0.51"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Silvino B. Magalhaes <sbm2yk@gmail.com>",
+	DRV_STABLE,
+	{ NULL }
+};
+
 #define UPSDELAY 500 /* 0.5 ms delay */
 
 typedef int bool_t;
@@ -714,7 +724,6 @@ void upsdrv_initinfo(void)
 	getbaseinfo();
 
 	upsh.instcmd = instcmd;
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 }
 
 void upsdrv_updateinfo(void)
@@ -728,14 +737,14 @@ void upsdrv_updateinfo(void)
 
 	/* output and bypass tests */
 	if( OutputOn )
-	  dstate_setinfo("outlet.0.switchable", "%s", "Yes");
+	  dstate_setinfo("outlet.switchable", "%s", "yes");
 	else
-	  dstate_setinfo("outlet.0.switchable", "%s", "No");
+	  dstate_setinfo("outlet.switchable", "%s", "no");
 
 	if( BypassOn )
-	  dstate_setinfo("outlet.1.switchable", "%s", "Yes");
+	  dstate_setinfo("outlet.1.switchable", "%s", "yes");
 	else
-	  dstate_setinfo("outlet.1.switchable", "%s", "No");
+	  dstate_setinfo("outlet.1.switchable", "%s", "no");
 
 	status_init();
 
@@ -786,13 +795,6 @@ void upsdrv_makevartable(void)
 
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Microsol Rhino UPS driver %s (%s)\n", 
-		DRV_VERSION, UPS_VERSION);
-        printf("by Silvino B. Magalhaes for Microsol - sbm2yk@gmail.com\n\n");
-}
-
 void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path);
