@@ -151,7 +151,7 @@ static void usb_comm_fail(const char *fmt, ...)
 	va_end(ap);
 
 	if ((ret < 1) || (ret >= (int) sizeof(why))) {
-		upslogx(LOG_WARNING, "usb_comm_fail: vsnprintf needed more than %d bytes", sizeof(why));
+		upslogx(LOG_WARNING, "usb_comm_fail: vsnprintf needed more than %d bytes", (int)sizeof(why));
 	}
 
 	upslogx(LOG_WARNING, "Communications with UPS lost: %s", why);
@@ -322,7 +322,7 @@ static int usb_device_open(usb_dev_handle **handlep, USBDevice_t *device, USBDev
 					upsdebugx(4, "USB device [%04x:%04x] opened", device->VendorID, device->ProductID);
 					return ret;
 				}
-#if LIBUSB_HAS_DETACH_KRNL_DRV
+#ifdef HAVE_USB_DETACH_KERNEL_DRIVER_NP
 				/* this method requires at least libusb 0.1.8:
 				 * it force device claiming by unbinding
 				 * attached driver... From libhid */
