@@ -1,6 +1,16 @@
 /*
 	anything commented is optional
 	anything else is mandatory
+	 
+	for more information, refer to:
+	* docs/developers.txt
+	* docs/new-drivers.txt
+	* docs/new-names.txt
+	
+	and possibly also to:
+	* docs/hid-subdrivers.txt for USB/HID devices
+	* or docs/snmp-subdrivers.txt for SNMP devices
+	* or docs/megatec.txt for Megatec / Q1 devices
 */
 
 #include "main.h"
@@ -10,15 +20,28 @@
 /* #define ENDCHAR	'\r'	*/
 /* #define IGNCHARS	""	*/
 
-#define DRV_VERSION	"0.01"
+#define DRIVER_NAME	"Skeleton UPS driver"
+#define DRIVER_VERSION	"0.02"
+
+/* driver description structure */
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"John Doe <john@doe.org>",
+	DRV_STABLE,
+	{ NULL }
+};
 
 void upsdrv_initinfo(void)
 {
 	/* try to detect the UPS here - call fatal_with_errno(EXIT_FAILURE, ) if it fails */
 
-	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
-	/* dstate_setinfo("ups.mfr", "skel driver"); */
+	/* dstate_setinfo("ups.mfr", "skel manufacturer"); */
 	/* dstate_setinfo("ups.model", "longrun 15000"); */
+	/* note: for a transition period, these data are redundant! */
+	/* dstate_setinfo("device.mfr", "skel manufacturer"); */
+	/* dstate_setinfo("device.model", "longrun 15000"); */
+
 
 	/* upsh.instcmd = instcmd; */
 }
@@ -127,12 +150,6 @@ void upsdrv_makevartable(void)
 	/* addvar(VAR_VALUE, "foo", "Override foo setting"); */
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools - Skeleton UPS driver %s (%s)\n\n", 
-		DRV_VERSION, UPS_VERSION);
-}
-
 void upsdrv_initups(void)
 {
 	/* upsfd = ser_open(device_path); */
