@@ -2,13 +2,15 @@
  *
  *  Based on NetSNMP API (Simple Network Management Protocol V1-2)
  *
- *  Copyright (C) 
- *   2002-2008  Arnaud Quette <arnaud.quette@free.fr>
- *   2002-2006	Dmitry Frolov <frolov@riss-telecom.ru>
- *  			J.W. Hoogervorst <jeroen@hoogervorst.net>
- *  			Niels Baggesen <niels@baggesen.net>
+ *  Copyright (C)
+ *	2002 - 2008	Arnaud Quette <arnaud.quette@free.fr>
+ *	2002 - 2006	Dmitry Frolov <frolov@riss-telecom.ru>
+ *			J.W. Hoogervorst <jeroen@hoogervorst.net>
+ *			Niels Baggesen <niels@baggesen.net>
+ *	2009 - 2010	Arjen de Korte <adkorte-guest@alioth.debian.org>
  *
- *  Sponsored by MGE UPS SYSTEMS <http://opensource.mgeups.com/>
+ *  Sponsored by Eaton <http://www.eaton.com>
+ *   and originally by MGE UPS SYSTEMS <http://www.mgeups.com/>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,46 +30,52 @@
  */
 
 /* NUT SNMP common functions */
-#include "snmp-ups.h"
 #include "main.h"
+#include "snmp-ups.h"
 #include "parseconf.h"
 
 /* include all known mib2nut lookup tables */
-#include "apccmib.h"
-#include "eaton-aphel-mib.h"
-#include "raritan-mib.h"
-#include "ietfmib.h"
-#include "mgemib.h"
-#include "netvisionmib.h"
-#include "pwmib.h"
+#include "apc-mib.h"
+#include "mge-mib.h"
+#include "netvision-mib.h"
+#include "powerware-mib.h"
+#include "eaton-mib.h"
+#include "raritan-pdu-mib.h"
+#include "baytech-mib.h"
+#include "compaq-mib.h"
+#include "ietf-mib.h"
 
-mib2nut_info_t mib2nut[] = {
-	{ "apcc", APCC_MIB_VERSION, APCC_OID_POWER_STATUS,
-		".1.3.6.1.4.1.318.1.1.1.1.1.1.0", apcc_mib },
-	{ "mge", MGE_MIB_VERSION, "",
-		MGE_OID_MODEL_NAME, mge_mib },
-	{ "netvision", NETVISION_MIB_VERSION, "",
-		NETVISION_OID_UPSIDENTMODEL, netvision_mib },
-	{ "pw", PW_MIB_VERSION, "",
-		PW_OID_MODEL_NAME, pw_mib },
-	{ "ietf", IETF_MIB_VERSION, IETF_OID_POWER_STATUS,
-		IETF_OID_MFR_NAME, ietf_mib },
-	{ "aphel_genesisII", EATON_APHEL_MIB_VERSION, "",
-		APHEL1_OID_MODEL_NAME, eaton_aphel_genesisII_mib },
-	{ "aphel_revelation", EATON_APHEL_MIB_VERSION, "",
-		APHEL2_OID_MODEL_NAME, eaton_aphel_revelation_mib },
-	{ "raritan", RARITAN_MIB_VERSION, "",
-		RARITAN_OID_MODEL_NAME, raritan_mib },
-	{ NULL }
+static mib2nut_info_t *mib2nut[] = {
+	&apc,
+	&mge,
+	&netvision,
+	&powerware,
+	&aphel_genesisII,
+	&aphel_revelation,
+	&raritan,
+	&baytech,
+	&compaq,
+	/*
+	 * Prepend vendor specific MIB mappings before IETF, so that
+	 * if a device supports both IETF and vendor specific MIB,
+	 * the vendor specific one takes precedence (when mib=auto)
+	 */
+	&ietf,
+	/* end of structure. */
+	NULL
 };
 
 /* pointer to the Snmp2Nut lookup table */
+mib2nut_info_t *mib2nut_info;
+/* FIXME: to be trashed */
 snmp_info_t *snmp_info;
 const char *mibname;
 const char *mibvers;
 
+static void disable_transfer_oids(void);
+
 #define DRIVER_NAME	"Generic SNMP UPS driver"
-#define DRIVER_VERSION		"0.44"
+#define DRIVER_VERSION		"0.47"
 
 /* driver description structure */
 upsdrv_info_t	upsdrv_info = {
@@ -76,7 +84,8 @@ upsdrv_info_t	upsdrv_info = {
 	"Arnaud Quette <arnaud.quette@free.fr>\n" \
 	"Dmitry Frolov <frolov@riss-telecom.ru>\n" \
 	"J.W. Hoogervorst <jeroen@hoogervorst.net>\n" \
-	"Niels Baggesen <niels@baggesen.net>",
+	"Niels Baggesen <niels@baggesen.net>\n" \
+	"Arjen de Korte <adkorte-guest@alioth.debian.org>",
 	DRV_STABLE,
 	{ NULL }
 };
@@ -84,6 +93,10 @@ upsdrv_info_t	upsdrv_info = {
 
 time_t lastpoll = 0;
 
+/* outlet OID index start with 0 or 1,
+ * automatically guessed at the first pass */
+int outlet_index_base = -1;
+
 /* ---------------------------------------------
  * driver functions implementations
  * --------------------------------------------- */
@@ -98,32 +111,34 @@ void upsdrv_initinfo(void)
 		DRIVER_VERSION, mibname, mibvers);
 	dstate_setinfo("driver.version.internal", "%s", version);
 
-	/* add instant commands to the info database. */
-	for (su_info_p = &snmp_info[0]; su_info_p->info_type != NULL ; su_info_p++)			
+	/* add instant commands to the info database.
+	 * outlet commands are processed during initial walk */
+	for (su_info_p = &snmp_info[0]; su_info_p->info_type != NULL ; su_info_p++)
 	{
 		su_info_p->flags |= SU_FLAG_OK;
-		if (SU_TYPE(su_info_p) == SU_TYPE_CMD)
+		if ((SU_TYPE(su_info_p) == SU_TYPE_CMD)
+			&& !(su_info_p->flags & SU_OUTLET))
 			dstate_addcmd(su_info_p->info_type);
 	}
 
-	/* setup handlers for instcmd and setvar functions */
-	upsh.setvar = su_setvar;
-	upsh.instcmd = su_instcmd;
-
 	if (testvar("notransferoids"))
 		disable_transfer_oids();
 
 	/* initialize all other INFO_ fields from list */
 	if (snmp_ups_walk(SU_WALKMODE_INIT))
 		dstate_dataok();
-	else		
+	else
 		dstate_datastale();
+
+	/* setup handlers for instcmd and setvar functions */
+	upsh.setvar = su_setvar;
+	upsh.instcmd = su_instcmd;
 }
 
 void upsdrv_updateinfo(void)
 {
 	upsdebugx(1,"SNMP UPS driver : entering upsdrv_updateinfo()");
-	
+
 	/* only update every pollfreq */
 	if (time(NULL) > (lastpoll + pollfreq)) {
 
@@ -132,8 +147,8 @@ void upsdrv_updateinfo(void)
 		/* update all dynamic info fields */
 		if (snmp_ups_walk(SU_WALKMODE_UPDATE))
 			dstate_dataok();
-		else		
-			dstate_datastale();	
+		else
+			dstate_datastale();
 
 		status_commit();
 
@@ -166,7 +181,7 @@ void upsdrv_makevartable(void)
 	upsdebugx(1, "entering upsdrv_makevartable()");
 
 	addvar(VAR_VALUE, SU_VAR_MIBS,
-	    "Set MIB compliance (default=ietf, allowed mge,apcc,netvision,pw)");
+	    "Set MIB compliance (default=ietf, allowed mge,apcc,netvision,pw,cpqpower)");
 	addvar(VAR_VALUE | VAR_SENSITIVE, SU_VAR_COMMUNITY,
 	    "Set community name (default=public)");
 	addvar(VAR_VALUE, SU_VAR_VERSION,
@@ -185,7 +200,7 @@ void upsdrv_initups(void)
 	const char *community, *version, *mibs;
 
 	upsdebugx(1, "SNMP UPS driver : entering upsdrv_initups()");
-	
+
 	community = testvar(SU_VAR_COMMUNITY) ? getval(SU_VAR_COMMUNITY) : "public";
 	version = testvar(SU_VAR_VERSION) ? getval(SU_VAR_VERSION) : "v1";
 	mibs = testvar(SU_VAR_MIBS) ? getval(SU_VAR_MIBS) : "auto";
@@ -193,8 +208,9 @@ void upsdrv_initups(void)
 	/* init SNMP library, etc... */
 	nut_snmp_init(progname, device_path, version, community);
 
+	/* FIXME: first test if the device is reachable to avoid timeouts! */
+
 	/* Load the SNMP to NUT translation data */
-	/* read_mibconf(SU_VAR_MIBS) ? getval(SU_VAR_MIBS) : "ietf"); */
 	load_mib2nut(mibs);
 
 	/* init polling frequency */
@@ -202,7 +218,7 @@ void upsdrv_initups(void)
 		pollfreq = atoi(getval(SU_VAR_POLLFREQ));
 	else
 		pollfreq = DEFAULT_POLLFREQ;
-	
+
   	/* Get UPS Model node to see if there's a MIB */
 	su_info_p = su_find_info("ups.model");
 	status = nut_snmp_get_str(su_info_p->OID, model, sizeof(model), NULL);
@@ -226,7 +242,7 @@ void upsdrv_cleanup(void)
 
 void nut_snmp_init(const char *type, const char *hostname, const char *version,
 		const char *community)
-{  
+{
 	upsdebugx(2, "SNMP UPS driver : entering nut_snmp_init(%s, %s, %s, %s)",
 		type, hostname, version, community);
 
@@ -247,7 +263,7 @@ void nut_snmp_init(const char *type, const char *hostname, const char *version,
 		fatalx(EXIT_FAILURE, "Bad SNMP version: %s", version);
 
 	/* Open the session */
-	SOCK_STARTUP;
+	SOCK_STARTUP; /* wrapper not needed on Unix! */
 	g_snmp_sess_p = snmp_open(&g_snmp_sess);	/* establish the session */
 	if (g_snmp_sess_p == NULL) {
 		nut_snmp_perror(&g_snmp_sess, 0, NULL, "nut_snmp_init: snmp_open");
@@ -262,7 +278,7 @@ void nut_snmp_cleanup(void)
 		snmp_close(g_snmp_sess_p);
 		g_snmp_sess_p = NULL;
 	}
-	SOCK_CLEANUP;
+	SOCK_CLEANUP; /* wrapper not needed on Unix! */
 }
 
 struct snmp_pdu *nut_snmp_get(const char *OID)
@@ -283,10 +299,10 @@ struct snmp_pdu *nut_snmp_get(const char *OID)
 	}
 
 	pdu = snmp_pdu_create(SNMP_MSG_GET);
-	
+
 	if (pdu == NULL)
 		fatalx(EXIT_FAILURE, "Not enough memory");
-	
+
 	snmp_add_null_var(pdu, name, name_len);
 
 	status = snmp_synch_response(g_snmp_sess_p, pdu, &response);
@@ -310,7 +326,7 @@ struct snmp_pdu *nut_snmp_get(const char *OID)
 				upsname?upsname:device_name);
 
 		if ((numerr < SU_ERR_LIMIT) || ((numerr % SU_ERR_RATE) == 0))
-			nut_snmp_perror(g_snmp_sess_p, status, response, 
+			nut_snmp_perror(g_snmp_sess_p, status, response,
 				"nut_snmp_get: %s", OID);
 
 		snmp_free_pdu(response);
@@ -326,7 +342,7 @@ bool_t nut_snmp_get_str(const char *OID, char *buf, size_t buf_len, info_lkp_t *
 {
 	size_t len = 0;
 	struct snmp_pdu *pdu;
-	
+
 	/* zero out buffer. */
 	memset(buf, 0, buf_len);
 
@@ -425,7 +441,9 @@ bool_t nut_snmp_set(const char *OID, char type, const char *value)
 	struct snmp_pdu *pdu, *response = NULL;
 	oid name[MAX_OID_LEN];
 	size_t name_len = MAX_OID_LEN;
-	
+
+	upsdebugx(1, "entering nut_snmp_set (%s, %c, %s)", OID, type, value);
+
 	if (!snmp_parse_oid(OID, name, &name_len)) {
 		upslogx(LOG_ERR, "[%s] nut_snmp_set: %s: %s",
 			upsname?upsname:device_name, OID, snmp_api_errstring(snmp_errno));
@@ -439,7 +457,7 @@ bool_t nut_snmp_set(const char *OID, char type, const char *value)
 	if (snmp_add_var(pdu, name, name_len, type, value)) {
 		upslogx(LOG_ERR, "[%s] nut_snmp_set: %s: %s",
 			upsname?upsname:device_name, OID, snmp_api_errstring(snmp_errno));
-		
+
 		return FALSE;
 	}
 
@@ -495,9 +513,19 @@ void nut_snmp_perror(struct snmp_session *sess, int status,
 			upsname?upsname:device_name, buf, snmperrstr);
 		free(snmperrstr);
 	} else if (status == STAT_SUCCESS) {
-		if (response->errstat != SNMP_ERR_NOERROR)
+		switch (response->errstat)
+		{
+		case SNMP_ERR_NOERROR:
+			break;
+		case SNMP_ERR_NOSUCHNAME:	/* harmless */
+			upsdebugx(2, "[%s] %s: %s",
+					 upsname?upsname:device_name, buf, snmp_errstring(response->errstat));
+			break;
+		default:
 			upslogx(LOG_ERR, "[%s] %s: Error in packet: %s",
 				upsname?upsname:device_name, buf, snmp_errstring(response->errstat));
+			break;
+		}
 	} else if (status == STAT_TIMEOUT) {
 		upslogx(LOG_ERR, "[%s] %s: Timeout: no response from %s",
 			upsname?upsname:device_name, buf, sess->peername);
@@ -534,26 +562,25 @@ static void disable_transfer_oids(void)
 }
 
 /* universal function to add or update info element. */
-void su_setinfo(const char *type, const char *value, int flags, int auxdata)
+void su_setinfo(snmp_info_t *su_info_p, const char *value)
 {
-	snmp_info_t *su_info_p;
-	
-	upsdebugx(1, "SNMP UPS driver : entering su_setinfo(%s)", type);
-			
-	su_info_p = su_find_info(type);
-	
+	upsdebugx(1, "entering su_setinfo(%s)", su_info_p->info_type);
+
 	if (SU_TYPE(su_info_p) == SU_TYPE_CMD)
 		return;
 
-	if (strcasecmp(type, "ups.status")) {
-		dstate_setinfo(type, "%s", value);
-		dstate_setflags(type, flags);
-		dstate_setaux(type, auxdata);
+	if (strcasecmp(su_info_p->info_type, "ups.status")) {
+		if (value != NULL)
+			dstate_setinfo(su_info_p->info_type, "%s", value);
+		else
+			dstate_setinfo(su_info_p->info_type, "%s", su_info_p->dfl);
+		dstate_setflags(su_info_p->info_type, su_info_p->info_flags);
+		dstate_setaux(su_info_p->info_type, su_info_p->info_len);
 	}
 }
 
 void su_status_set(snmp_info_t *su_info_p, long value)
-{	
+{
 	const char *info_value = NULL;
 
 	upsdebugx(2, "SNMP UPS driver : entering su_status_set()");
@@ -573,54 +600,51 @@ snmp_info_t *su_find_info(const char *type)
 	snmp_info_t *su_info_p;
 
 	for (su_info_p = &snmp_info[0]; su_info_p->info_type != NULL ; su_info_p++)
-		if (!strcasecmp(su_info_p->info_type, type))
+		if (!strcasecmp(su_info_p->info_type, type)) {
+			upsdebugx(3, "su_find_info: \"%s\" found", type);
 			return su_info_p;
-		
-	fatalx(EXIT_FAILURE, "nut_snmp_find_info: unknown info type: %s", type);
+		}
+
+	upsdebugx(3, "su_find_info: unknown info type (%s)", type);
 	return NULL;
 }
 
 /* Load the right snmp_info_t structure matching mib parameter */
 bool_t load_mib2nut(const char *mib)
 {
-	bool_t ret = FALSE;
-	mib2nut_info_t *mp = mib2nut;
+	int	i;
+	char	buf[LARGEBUF];
+
 	upsdebugx(2, "SNMP UPS driver : entering load_mib2nut(%s)", mib);
-	
-/*	read_mibconf(mib); */
-	
-	while (mp->mib_name) {
-		if (strcmp(mib, mp->mib_name) == 0)
-			break;
-		else if (strcmp(mib, "auto") == 0) {
-			int status;
-			char buf[1024];
-			upsdebugx(1, "load_mib2nut: trying %s mib", mp->mib_name);
-			status = nut_snmp_get_str(mp->oid_auto_check,
-						buf, sizeof buf, NULL);
-			if (status) {
-				ret = TRUE;
-				break;
-			}
+
+	/* FIXME: first try SysOID (.1.3.6.1.2.1.1.2)
+	 * to speed up detection if mib==auto
+	 * This is an indirection on the MIB's entry point
+	 * examples:
+	 * APHEL-GENESIS-II-MIB => .iso.org.dod.internet.private.enterprises.17373
+	 * APHEL Revelation MIB => .iso.org.dod.internet.private.enterprises.534.6.6.6
+	 */
+	for (i = 0; mib2nut[i] != NULL; i++) {
+		if (strcmp(mib, "auto") && strcmp(mib, mib2nut[i]->mib_name)) {
+			continue;
 		}
-		mp++;
-	}
-	if (mp->mib_name) {
-		snmp_info = mp->snmp_info;
-		OID_pwr_status = mp->oid_pwr_status;
-		mibname = mp->mib_name;
-		mibvers = mp->mib_version;
+		upsdebugx(1, "load_mib2nut: trying %s mib", mib2nut[i]->mib_name);
+		if (!nut_snmp_get_str(mib2nut[i]->oid_auto_check, buf, sizeof(buf), NULL)) {
+			continue;
+		}
+		snmp_info = mib2nut[i]->snmp_info;
+		OID_pwr_status = mib2nut[i]->oid_pwr_status;
+		mibname = mib2nut[i]->mib_name;
+		mibvers = mib2nut[i]->mib_version;
 		upsdebugx(1, "load_mib2nut: using %s mib", mibname);
+		return TRUE;
 	}
-	else {
-		/* Did we find something or is it really an unknown mib */
-		/* we assume (ret == FALSE) */
-		if (strcmp(mib, "auto") != 0)
-			fatalx(EXIT_FAILURE, "Unknown mibs value: %s", mib);
-		else
-			fatalx(EXIT_FAILURE, "No supported device detected");
+	/* Did we find something or is it really an unknown mib */
+	if (strcmp(mib, "auto") != 0) {
+		fatalx(EXIT_FAILURE, "Unknown mibs value: %s", mib);
+	} else {
+		fatalx(EXIT_FAILURE, "No supported device detected");
 	}
-	return ret;
 }
 
 /* find the OID value matching that INFO_* value */
@@ -630,11 +654,11 @@ long su_find_valinfo(info_lkp_t *oid2info, char* value)
 
 	for (info_lkp = oid2info; (info_lkp != NULL) &&
 		(strcmp(info_lkp->info_value, "NULL")); info_lkp++) {
-			
+
 		if (!(strcmp(info_lkp->info_value, value))) {
 			upsdebugx(1, "su_find_valinfo: found %s (value: %s)",
 					info_lkp->info_value, value);
-			
+
 			return info_lkp->oid_value;
 		}
 	}
@@ -646,14 +670,14 @@ long su_find_valinfo(info_lkp_t *oid2info, char* value)
 const char *su_find_infoval(info_lkp_t *oid2info, long value)
 {
 	info_lkp_t *info_lkp;
-		
+
 	for (info_lkp = oid2info; (info_lkp != NULL) &&
 		(strcmp(info_lkp->info_value, "NULL")); info_lkp++) {
-			
+
 		if (info_lkp->oid_value == value) {
 			upsdebugx(1, "su_find_infoval: found %s (value: %ld)",
 					info_lkp->info_value, value);
-			
+
 			return info_lkp->info_value;
 		}
 	}
@@ -674,41 +698,156 @@ static void disable_competition(snmp_info_t *entry)
 	}
 }
 
+/* instantiate an snmp_info_t from a template.
+ * mostly (only?) useful for outlet templates.
+ * Note: remember to adapt info_type, OID and optionaly dfl */
+snmp_info_t *instantiate_info(snmp_info_t *info_template, snmp_info_t *new_instance)
+{
+	/* sanity check */
+	if (info_template == NULL)
+		return NULL;
+
+	if (new_instance == NULL)
+		new_instance = (snmp_info_t *)xmalloc(sizeof(snmp_info_t));
+
+	new_instance->info_type = (char *)xmalloc(SU_INFOSIZE);
+	if (info_template->OID != NULL)
+		new_instance->OID = (char *)xmalloc(SU_INFOSIZE);
+	else
+		new_instance->OID = NULL;
+	new_instance->info_flags = info_template->info_flags;
+	new_instance->info_len = info_template->info_len;
+	/* FIXME: check if we need to adapt this one... */
+	new_instance->dfl = info_template->dfl;
+	new_instance->flags = info_template->flags;
+	new_instance->oid2info = info_template->oid2info;
+	new_instance->setvar = info_template->setvar;
+
+	return new_instance;
+}
+
+/* free a dynamically allocated snmp_info_t.
+ * mostly (only?) useful for outlet templates */
+void free_info(snmp_info_t *su_info_p)
+{
+	/* sanity check */
+	if (su_info_p == NULL)
+		return;
+
+	if (su_info_p->info_type != NULL)
+		free ((char *)su_info_p->info_type);
+
+	if (su_info_p->OID != NULL)
+		free ((char *)su_info_p->OID);
+
+	free (su_info_p);
+}
+
+/* return the base SNMP index (0 or 1) to start outlet iteration on the MIB,
+ * based on a test using a template OID */
+int base_snmp_outlet_index(const char *OID_template)
+{
+	int base_index = outlet_index_base;
+
+	if (outlet_index_base == -1)
+	{
+		/* not initialised yet */
+		char test_OID[SU_INFOSIZE];
+		for (base_index = 0 ; base_index < 2 ; base_index++) {
+			sprintf(test_OID, OID_template, base_index);
+			if (nut_snmp_get(test_OID) != NULL)
+				break;
+		}
+		outlet_index_base = base_index;
+	}
+	upsdebugx(3, "base_snmp_outlet_index: %i", outlet_index_base);
+	return base_index;
+}
+
+/* return the NUT offset (increment) based on outlet_index_base
+ * ie (outlet_index_base == 0) => increment +1
+ *    (outlet_index_base == 1) => increment +0 */
+int base_nut_outlet_offset()
+{
+	return (outlet_index_base==0)?1:0;
+}
+
+/* process a single data from a walk */
+bool_t get_and_process_data(int mode, snmp_info_t *su_info_p)
+{
+	bool_t status = FALSE;
+
+	upsdebugx(1, "getting data: %s (%s)", su_info_p->info_type, su_info_p->OID);
+
+	/* ok, update this element. */
+	status = su_ups_get(su_info_p);
+
+	/* set stale flag if data is stale, clear if not. */
+	if (status == TRUE) {
+		if (su_info_p->flags & SU_FLAG_STALE) {
+			upslogx(LOG_INFO, "[%s] snmp_ups_walk: data resumed for %s",
+				upsname?upsname:device_name, su_info_p->info_type);
+			su_info_p->flags &= ~SU_FLAG_STALE;
+		}
+		if(su_info_p->flags & SU_FLAG_UNIQUE) {
+			/* We should be the only provider of this */
+			disable_competition(su_info_p);
+			su_info_p->flags &= ~SU_FLAG_UNIQUE;
+		}
+		dstate_dataok();
+	} else {
+		if (mode == SU_WALKMODE_INIT) {
+			/* handle unsupported vars */
+			su_info_p->flags &= ~SU_FLAG_OK;
+		} else	{
+			if (!(su_info_p->flags & SU_FLAG_STALE)) {
+				upslogx(LOG_INFO, "[%s] snmp_ups_walk: data stale for %s",
+					upsname?upsname:device_name, su_info_p->info_type);
+				su_info_p->flags |= SU_FLAG_STALE;
+			}
+			dstate_datastale();
+		}
+	}
+	return status;
+}
+
 /* walk ups variables and set elements of the info array. */
 bool_t snmp_ups_walk(int mode)
 {
 	static unsigned long iterations = 0;
 	snmp_info_t *su_info_p;
 	bool_t status = FALSE;
-	
+
 	for (su_info_p = &snmp_info[0]; su_info_p->info_type != NULL ; su_info_p++) {
 
-		/* Check if we are asked to stop (reactivity++). */
+		/* Check if we are asked to stop (reactivity++) */
 		if (exit_flag != 0)
 			return TRUE;
 
-		/* skip instcmd. */
-		if (SU_TYPE(su_info_p) == SU_TYPE_CMD) {
+		/* skip instcmd, not linked to outlets */
+		if ((SU_TYPE(su_info_p) == SU_TYPE_CMD)
+			&& !(su_info_p->flags & SU_OUTLET)) {
 			upsdebugx(1, "SU_CMD_MASK => %s", su_info_p->OID);
 			continue;
 		}
-		/* skip elements we shouldn't show. */
+		/* skip elements we shouldn't show */
 		if (!(su_info_p->flags & SU_FLAG_OK))
 			continue;
-		
-		/* skip static elements in update mode. */
-		if (mode == SU_WALKMODE_UPDATE && 
+
+		/* skip static elements in update mode */
+		if (mode == SU_WALKMODE_UPDATE &&
 				su_info_p->flags & SU_FLAG_STATIC)
 			continue;
 
 		/* set default value if we cannot fetch it */
-		/* and set static flag on this element. */
-		if (su_info_p->flags & SU_FLAG_ABSENT) {
+		/* and set static flag on this element.
+		 * not applicable to outlets (need SU_FLAG_STATIC tagging) */
+		if ((su_info_p->flags & SU_FLAG_ABSENT)
+			&& !(su_info_p->flags & SU_OUTLET)) {
 			if (mode == SU_WALKMODE_INIT) {
 				if (su_info_p->dfl) {
 					/* Set default value if we cannot fetch it from ups. */
-					su_setinfo(su_info_p->info_type, su_info_p->dfl,
-						su_info_p->info_flags, su_info_p->info_len);
+					su_setinfo(su_info_p, NULL);
 				}
 				su_info_p->flags |= SU_FLAG_STATIC;
 			}
@@ -768,40 +907,71 @@ bool_t snmp_ups_walk(int mode)
 			}
 		}
 
-		/* ok, update this element. */
-		status = su_ups_get(su_info_p);
-		
-		/* set stale flag if data is stale, clear if not. */
-		if (status == TRUE) {
-			if (su_info_p->flags & SU_FLAG_STALE) {
-				upslogx(LOG_INFO, "[%s] snmp_ups_walk: data resumed for %s",
-					upsname?upsname:device_name, su_info_p->info_type);
-				su_info_p->flags &= ~SU_FLAG_STALE;
+		/* process outlet template definition */
+		if (su_info_p->flags & SU_OUTLET) {
+			upsdebugx(1, "outlet template definition found (%s)...", su_info_p->info_type);
+			int cur_outlet_number = 1;
+			int cur_nut_index = 0;
+			int outlet_count = 0;
+			snmp_info_t cur_info_p;
+
+			if(dstate_getinfo("outlet.count") == NULL) {
+				/* FIXME: should we disable it?
+				 * su_info_p->flags &= ~SU_FLAG_OK; */
+				continue;
 			}
-			if(su_info_p->flags & SU_FLAG_UNIQUE) {
-				/* We should be the only provider of this */
-				disable_competition(su_info_p);
-				su_info_p->flags &= ~SU_FLAG_UNIQUE;
-			}
-			dstate_dataok();
-		} else {
-			if (mode == SU_WALKMODE_INIT) {
-				/* handle unsupported vars */
-				su_info_p->flags &= ~SU_FLAG_OK;
-			} else	{
-				if (!(su_info_p->flags & SU_FLAG_STALE)) {
-					upslogx(LOG_INFO, "[%s] snmp_ups_walk: data stale for %s",
-						upsname?upsname:device_name, su_info_p->info_type);
-					su_info_p->flags |= SU_FLAG_STALE;
+			outlet_count = atoi(dstate_getinfo("outlet.count"));
+
+			/* general init of data using the template */
+			instantiate_info(su_info_p, &cur_info_p);
+
+			for (cur_outlet_number = base_snmp_outlet_index(su_info_p->OID) ;
+					cur_outlet_number < outlet_count ; cur_outlet_number++) {
+
+				cur_nut_index = cur_outlet_number + base_nut_outlet_offset();
+				sprintf((char*)cur_info_p.info_type, su_info_p->info_type,
+						cur_nut_index);
+
+				/* check if default value is also a template */
+				if ((cur_info_p.dfl != NULL) &&
+					(strstr(su_info_p->dfl, "%i") != NULL)) {
+					cur_info_p.dfl = (char *)xmalloc(SU_INFOSIZE);
+					sprintf((char *)cur_info_p.dfl, su_info_p->dfl, cur_nut_index);
 				}
-				dstate_datastale();
+
+				if (cur_info_p.OID != NULL) {
+					sprintf((char *)cur_info_p.OID, su_info_p->OID, cur_outlet_number);
+
+					/* add outlet instant commands to the info database. */
+					if (SU_TYPE(su_info_p) == SU_TYPE_CMD) {
+						if (mode == SU_WALKMODE_INIT)
+							dstate_addcmd(cur_info_p.info_type);
+					}
+					else /* get and process this data */
+						status = get_and_process_data(mode, &cur_info_p);
+				} else {
+					/* server side (ABSENT) data */
+					su_setinfo(&cur_info_p, NULL);
+				}
+				/* set back the flag */
+				su_info_p->flags = cur_info_p.flags;
 			}
+			free((char*)cur_info_p.info_type);
+			if (cur_info_p.OID != NULL)
+				free((char*)cur_info_p.OID);
+			if ((cur_info_p.dfl != NULL) &&
+				(strstr(su_info_p->dfl, "%i") != NULL))
+				free((char*)cur_info_p.dfl);
+
+		} else {
+			/* get and process this data */
+			status = get_and_process_data(mode, su_info_p);
 		}
 	}	/* for (su_info_p... */
 
 	iterations++;
-	
-	return status;	
+
+	return status;
 }
 
 bool_t su_ups_get(snmp_info_t *su_info_p)
@@ -811,7 +981,7 @@ bool_t su_ups_get(snmp_info_t *su_info_p)
 	long value;
 
 	upsdebugx(2, "su_ups_get: %s %s", su_info_p->info_type, su_info_p->OID);
-	
+
 	if (!strcasecmp(su_info_p->info_type, "ups.status")) {
 
 		status = nut_snmp_get_int(su_info_p->OID, &value);
@@ -853,11 +1023,10 @@ bool_t su_ups_get(snmp_info_t *su_info_p)
 		}
 
 		snprintf(buf, sizeof(buf), "%.1f", temp);
-		su_setinfo(su_info_p->info_type, buf,
-			su_info_p->info_flags, su_info_p->info_len);
+		su_setinfo(su_info_p, buf);
 
 		return TRUE;
-	}			
+	}
 
 	if (su_info_p->info_flags == 0) {
 		status = nut_snmp_get_int(su_info_p->OID, &value);
@@ -879,91 +1048,184 @@ bool_t su_ups_get(snmp_info_t *su_info_p)
 	} else {
 		status = nut_snmp_get_str(su_info_p->OID, buf, sizeof(buf), su_info_p->oid2info);
 	}
-		
+
 	if (status == TRUE) {
-		su_setinfo(su_info_p->info_type, buf,
-			su_info_p->info_flags, su_info_p->info_len);
-		
+		su_setinfo(su_info_p, buf);
 		upsdebugx(2, "=> value: %s", buf);
 	}
-	else upsdebugx(2, "=> Failed");
+	else
+		upsdebugx(2, "=> Failed");
+
 	return status;
 }
 
 /* set r/w INFO_ element to a value. */
 int su_setvar(const char *varname, const char *val)
 {
-	snmp_info_t *su_info_p;
-	bool_t ret;
+	snmp_info_t *su_info_p = NULL;
+	bool_t status;
+	int retval = STAT_SET_FAILED;
 
-	upsdebugx(2, "entering su_setvar()");
+	upsdebugx(2, "entering su_setvar(%s, %s)", varname, val);
 
-	su_info_p = su_find_info(varname);
+	if (strncmp(varname, "outlet", 6))
+		su_info_p = su_find_info(varname);
+	else {
+		snmp_info_t *tmp_info_p;
+		char *outlet_number_ptr = strchr(varname, '.');
+		int outlet_number = atoi(++outlet_number_ptr);
+		if (dstate_getinfo("outlet.count") == NULL) {
+			upsdebugx(2, "su_setvar: can't get outlet.count...");
+			return STAT_SET_INVALID;
+		}
+
+		upsdebugx(3, "su_setvar: outlet %i / %i", outlet_number,
+			atoi(dstate_getinfo("outlet.count")));
+
+		/* ensure the outlet number is supported (filtered upstream though)! */
+		if (outlet_number > atoi(dstate_getinfo("outlet.count"))) {
+			/* out of bound outlet number */
+			upsdebugx(2, "su_setvar: outlet is out of bound (%i / %s)",
+				outlet_number, dstate_getinfo("outlet.count"));
+			return STAT_SET_INVALID;
+		}
+		/* find back the outlet template */
+		char *outlet_varname = (char *)xmalloc(SU_INFOSIZE);
+		sprintf(outlet_varname, "outlet.%s%s", "%i", strchr(outlet_number_ptr++, '.'));
+		upsdebugx(3, "su_setvar: searching for template\"%s\"", outlet_varname);
+		tmp_info_p = su_find_info(outlet_varname);
+		free(outlet_varname);
+
+		/* for an snmp_info_t instance */
+		su_info_p = instantiate_info(tmp_info_p, su_info_p);
+
+		/* check if default value is also a template */
+		if ((su_info_p->dfl != NULL) &&
+			(strstr(tmp_info_p->dfl, "%i") != NULL)) {
+			su_info_p->dfl = (char *)xmalloc(SU_INFOSIZE);
+			sprintf((char *)su_info_p->dfl, tmp_info_p->dfl,
+				outlet_number - base_nut_outlet_offset());
+		}
+		/* adapt the OID */
+		if (su_info_p->OID != NULL) {
+			sprintf((char *)su_info_p->OID, tmp_info_p->OID,
+				outlet_number - base_nut_outlet_offset());
+		}
+		/* else, don't return STAT_SET_INVALID since we can be setting
+		 * a server side variable! */
+
+		/* adapt info_type */
+		if (su_info_p->info_type != NULL)
+			sprintf((char *)su_info_p->info_type, "%s", varname);
+	}
 
 	if (!su_info_p || !su_info_p->info_type || !(su_info_p->flags & SU_FLAG_OK)) {
 		upsdebugx(2, "su_setvar: info element unavailable %s", varname);
+
+		if (!strncmp(varname, "outlet", 6))
+			free_info(su_info_p);
+
 		return STAT_SET_UNKNOWN;
 	}
 
 	if (!(su_info_p->info_flags & ST_FLAG_RW) || su_info_p->OID == NULL) {
 		upsdebugx(2, "su_setvar: not writable %s", varname);
+
+		if (!strncmp(varname, "outlet", 6))
+			free_info(su_info_p);
+
 		return STAT_SET_INVALID;
 	}
 
-	/* set value. */
-	if (SU_TYPE(su_info_p) == SU_TYPE_STRING) {
-		ret = nut_snmp_set_str(su_info_p->OID, val);
+	/* set value into the device */
+	if (su_info_p->info_flags & ST_FLAG_STRING) {
+		status = nut_snmp_set_str(su_info_p->OID, val);
 	} else {
-		ret = nut_snmp_set_int(su_info_p->OID, strtol(val, NULL, 0));
+		status = nut_snmp_set_int(su_info_p->OID, strtol(val, NULL, 0));
 	}
 
-	if (ret == FALSE) {
+	if (status == FALSE)
 		upsdebugx(1, "su_setvar: cannot set value %s for %s", val, su_info_p->OID);
-		return STAT_SET_FAILED;
+	else {
+		retval = STAT_SET_HANDLED;
+		upsdebugx(1, "su_setvar: successfully set %s to \"%s\"", varname, val);
+
+		/* update info array */
+		su_setinfo(su_info_p, val);
 	}
+	if (!strncmp(varname, "outlet", 6))
+		free_info(su_info_p);
 
-	upsdebugx(1, "su_setvar: sucessfully set %s to \"%s\"", su_info_p->info_type, val);
-
-	/* update info array. */
-	su_setinfo(varname, val, su_info_p->info_flags, su_info_p->info_len);
-
-	/* TODO: check su_setinfo() retcode */
-	return STAT_SET_HANDLED;
+	return retval;
 }
 
 /* process instant command and take action. */
 int su_instcmd(const char *cmdname, const char *extradata)
 {
-	snmp_info_t *su_info_p;
+	snmp_info_t *su_info_p = NULL;
 	int status;
+	int retval = STAT_INSTCMD_FAILED;
 
-	upsdebugx(2, "entering su_instcmd()");
+	upsdebugx(2, "entering su_instcmd(%s, %s)", cmdname, extradata);
 
-	su_info_p = su_find_info(cmdname);
+	if (strncmp(cmdname, "outlet", 6))
+		su_info_p = su_find_info(cmdname);
+	else {
+		snmp_info_t *tmp_info_p;
+		char *outlet_number_ptr = strchr(cmdname, '.');
+		int outlet_number = atoi(++outlet_number_ptr);
+		if (dstate_getinfo("outlet.count") == NULL) {
+			upsdebugx(2, "su_instcmd: can't get outlet.count...");
+			return STAT_INSTCMD_UNKNOWN;
+		}
+
+		upsdebugx(3, "su_instcmd: outlet %i / %i", outlet_number,
+			atoi(dstate_getinfo("outlet.count")));
+
+		/* ensure the outlet number is supported! */
+		if (outlet_number > atoi(dstate_getinfo("outlet.count"))) {
+			/* out of bound outlet number */
+			upsdebugx(2, "su_instcmd: outlet is out of bound (%i / %s)",
+				outlet_number, dstate_getinfo("outlet.count"));
+			return STAT_INSTCMD_INVALID;
+		}
+
+		/* find back the outlet template */
+		char *outlet_cmdname = (char *)xmalloc(SU_INFOSIZE);
+		sprintf(outlet_cmdname, "outlet.%s%s", "%i", strchr(outlet_number_ptr++, '.'));
+		upsdebugx(3, "su_instcmd: searching for template\"%s\"", outlet_cmdname);
+		tmp_info_p = su_find_info(outlet_cmdname);
+		free(outlet_cmdname);
+
+		/* for an snmp_info_t instance */
+		su_info_p = instantiate_info(tmp_info_p, su_info_p);
+
+		/* check if default value is also a template */
+		if ((su_info_p->dfl != NULL) &&
+			(strstr(tmp_info_p->dfl, "%i") != NULL)) {
+			su_info_p->dfl = (char *)xmalloc(SU_INFOSIZE);
+			sprintf((char *)su_info_p->dfl, tmp_info_p->dfl,
+				outlet_number - base_nut_outlet_offset());
+		}
+		/* adapt the OID */
+		if (su_info_p->OID != NULL) {
+			sprintf((char *)su_info_p->OID, tmp_info_p->OID,
+				outlet_number - base_nut_outlet_offset());
+		} else {
+			free_info(su_info_p);
+			return STAT_INSTCMD_UNKNOWN;
+		}
+	}
 
 	if (!su_info_p || !su_info_p->info_type || !(su_info_p->flags & SU_FLAG_OK)) {
 		upsdebugx(2, "su_instcmd: %s unavailable", cmdname);
+
+		if (!strncmp(cmdname, "outlet", 6))
+			free_info(su_info_p);
+
 		return STAT_INSTCMD_UNKNOWN;
 	}
 
-	if (extradata) {
-		int	ret = STAT_SET_INVALID;
-
-		if (strcasecmp(cmdname, "shutdown.return")) {
-			ret = su_setvar("ups.delay.start", extradata);
-		}
-		if (strcasecmp(cmdname, "shutdown.stayoff")) {
-			ret = su_setvar("ups.delay.shutdown", extradata);
-		}
-		if (strcasecmp(cmdname, "shutdown.reboot")) {
-			ret = su_setvar("ups.delay.reboot", extradata);
-		}
-
-		if (ret == STAT_SET_HANDLED) {
-			extradata = NULL;
-		}
-	}
-
 	/* set value. */
 	if (su_info_p->info_flags & ST_FLAG_STRING) {
 		status = nut_snmp_set_str(su_info_p->OID, extradata ? extradata : su_info_p->dfl);
@@ -971,13 +1233,17 @@ int su_instcmd(const char *cmdname, const char *extradata)
 		status = nut_snmp_set_int(su_info_p->OID, extradata ? atoi(extradata) : su_info_p->info_len);
 	}
 
-	if (status == FALSE) {
+	if (status == FALSE)
 		upsdebugx(1, "su_instcmd: cannot set value for %s", cmdname);
-		return STAT_INSTCMD_FAILED;
+	else {
+		retval = STAT_INSTCMD_HANDLED;
+		upsdebugx(1, "su_instcmd: successfully sent command %s", cmdname);
 	}
 
-	upsdebugx(1, "su_instcmd: successfully sent command %s", cmdname);
-	return STAT_INSTCMD_HANDLED;
+	if (!strncmp(cmdname, "outlet", 6))
+		free_info(su_info_p);
+
+	return retval;
 }
 
 /* TODO: complete rewrite */
@@ -1012,7 +1278,7 @@ void su_shutdown_ups(void)
 			su_ups_instcmd(CMD_SOFTDOWN, 0, 0);
 		su_ups_instcmd(CMD_SDRET, 0, 0);
 		break;
-*/	
+*/
 	default:
 		/* if on battery... */
 /*		if (pwr_status == su_find_valinfo(info_lkp_t *oid2info, "OB")) {
@@ -1028,11 +1294,13 @@ void su_shutdown_ups(void)
 	}
 }
 
+/* FIXME: the below functions can be removed since these were for loading
+ * the mib2nut information from a file instead of the .h definitions... */
 /* return 1 if usable, 0 if not */
 static int parse_mibconf_args(int numargs, char **arg)
 {
 	bool_t ret;
-	
+
 	/* everything below here uses up through arg[1] */
 	if (numargs < 6)
 		return 0;
@@ -1047,7 +1315,7 @@ static int parse_mibconf_args(int numargs, char **arg)
 		} else {
 			ret = nut_snmp_set_int(arg[3], strtol(arg[4], NULL, 0));
 		}
-	
+
 		if (ret == FALSE)
 			upslogx(LOG_ERR, "su_setvar: cannot set value %s for %s", arg[4], arg[3]);
 		else
@@ -1073,7 +1341,7 @@ void read_mibconf(char *mib)
 	PCONF_CTX_t	ctx;
 
 	upsdebugx(2, "SNMP UPS driver : entering read_mibconf(%s)", mib);
-	
+
 	snprintf(fn, sizeof(fn), "%s/snmp/%s.conf", CONFPATH, mib);
 
 	pconf_init(&ctx, mibconf_err);
@@ -1095,15 +1363,15 @@ void read_mibconf(char *mib)
 			unsigned int	i;
 			char	errmsg[SMALLBUF];
 
-			snprintf(errmsg, sizeof(errmsg), 
+			snprintf(errmsg, sizeof(errmsg),
 				"mib.conf: invalid directive");
 
 			for (i = 0; i < ctx.numargs; i++)
-				snprintfcat(errmsg, sizeof(errmsg), " %s", 
+				snprintfcat(errmsg, sizeof(errmsg), " %s",
 					ctx.arglist[i]);
 
 			upslogx(LOG_WARNING, "%s", errmsg);
 		}
 	}
-	pconf_finish(&ctx);		
+	pconf_finish(&ctx);
 }
