@@ -1,9 +1,9 @@
 /*  tripplite-hid.c - data to monitor Tripp Lite USB/HID devices with NUT
  *
- *  Copyright (C)  
+ *  Copyright (C)
  *	2003 - 2005 Arnaud Quette <arnaud.quette@free.fr>
  *	2005 - 2006 Peter Selinger <selinger@users.sourceforge.net>
- *	2008        Arjen de Korte <adkorte-guest@alioth.debian.org>
+ *	2008 - 2009 Arjen de Korte <adkorte-guest@alioth.debian.org>
  *
  *  Sponsored by MGE UPS SYSTEMS <http://www.mgeups.com>
  *
@@ -24,11 +24,9 @@
  *
  */
 
+#include "main.h"
 #include "usbhid-ups.h"
 #include "tripplite-hid.h"
-#include "main.h"
-#include "extstate.h" /* for ST_FLAG_STRING */
-#include "common.h"
 #include "usb-common.h"
 
 #define TRIPPLITE_HID_VERSION "TrippLite HID 0.4"
@@ -56,7 +54,7 @@ static void *battery_scale_0dot1()
 }
 
 /* TrippLite */
-#define TRIPPLITE_VENDORID 0x09ae 
+#define TRIPPLITE_VENDORID 0x09ae
 
 /* Hewlett Packard */
 #define HP_VENDORID 0x03f0
@@ -67,6 +65,8 @@ static usb_device_id_t tripplite_usb_device_table[] = {
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x1003), battery_scale_0dot1 },
 	/* e.g. TrippLite AVR750U */
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x1007), battery_scale_0dot1 },
+	/* e.g. TrippLite ECO550UPS */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x1008), battery_scale_0dot1 },
 	/* e.g. TrippLite OMNI1000LCD */
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x2005), battery_scale_0dot1 },
 	/* e.g. TrippLite OMNI900LCD */
@@ -84,7 +84,9 @@ static usb_device_id_t tripplite_usb_device_table[] = {
 
 	/* HP R/T 2200 INTL (like SMART2200RMXL2U) */
 	{ USB_DEVICE(HP_VENDORID, 0x1f0a), battery_scale_1dot0 },
-	
+	/* HP T750 INTL */
+	{ USB_DEVICE(HP_VENDORID, 0x1f06), battery_scale_1dot0 },
+
 	/* Terminating entry */
 	{ -1, -1, NULL }
 };
@@ -136,7 +138,7 @@ static info_lkp_t tripplite_battvolt[] = {
 
 /* TRIPPLITE usage table */
 static usage_lkp_t tripplite_usage_lkp[] = {
-	/* currently unknown: 
+	/* currently unknown:
 	   ffff0010, 00ff0001, ffff007d, ffff00c0, ffff00c1, ffff00c2,
 	   ffff00c3, ffff00c4, ffff00c5, ffff00d2, ffff0091, ffff00c7 */
 
@@ -240,7 +242,7 @@ static hid_info_t tripplite_hid2nut[] = {
 	{ "ups.watchdog.status", 0, 0, "UPS.OutletSystem.Outlet.TLWatchdog", NULL, "%.0f", 0, NULL },
 
 	/* Special case: ups.status */
-	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.InternalFailure", NULL, NULL, 0, commfault_info }, 
+	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.InternalFailure", NULL, NULL, 0, commfault_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ShutdownImminent", NULL, NULL, 0, shutdownimm_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ACPresent", NULL, NULL, HU_FLAG_QUICK_POLL, online_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.BelowRemainingCapacityLimit", NULL, NULL, HU_FLAG_QUICK_POLL, lowbatt_info },
@@ -280,7 +282,7 @@ static hid_info_t tripplite_hid2nut[] = {
 	{ "input.frequency", 0, 0, "UPS.PowerConverter.Input.Frequency", NULL, "%.1f", 0, NULL },
 	{ "input.transfer.low", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.LowVoltageTransfer", NULL, "%.1f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "input.transfer.low.max", 0, 0, "UPS.PowerConverter.Output.TLLowVoltageTransferMax", NULL, "%.0f", HU_FLAG_STATIC, NULL },
-	{ "input.transfer.low.min", 0, 0, "UPS.PowerConverter.Output.TLLowVoltageTransferMin", NULL, "%.0f", HU_FLAG_STATIC, NULL }, 
+	{ "input.transfer.low.min", 0, 0, "UPS.PowerConverter.Output.TLLowVoltageTransferMin", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "input.transfer.high", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.HighVoltageTransfer", NULL, "%.1f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "input.transfer.high.max", 0, 0, "UPS.PowerConverter.Output.TLHighVoltageTransferMax", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "input.transfer.high.min", 0, 0, "UPS.PowerConverter.Output.TLHighVoltageTransferMin", NULL, "%.0f", HU_FLAG_STATIC, NULL },
@@ -297,13 +299,16 @@ static hid_info_t tripplite_hid2nut[] = {
 	{ "test.battery.start.quick", 0, 0, "UPS.BatterySystem.Test", NULL, "1", HU_TYPE_CMD, NULL }, /* reported to work on OMNI1000 */
 	{ "test.battery.start.deep", 0, 0, "UPS.BatterySystem.Test", NULL, "2", HU_TYPE_CMD, NULL }, /* reported not to work */
 	{ "test.battery.stop", 0, 0, "UPS.BatterySystem.Test", NULL, "3", HU_TYPE_CMD, NULL }, /* reported not to work */
-	
+
 	{ "load.off.delay", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, DEFAULT_OFFDELAY, HU_TYPE_CMD, NULL },
 	{ "load.on.delay", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, DEFAULT_ONDELAY, HU_TYPE_CMD, NULL },
 
 	{ "shutdown.stop", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, "-1", HU_TYPE_CMD, NULL },
 	{ "shutdown.reboot", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeReboot", NULL, "10", HU_TYPE_CMD, NULL },
 
+	/*  NOTE: the ECO550UPS doesn't support DelayBeforeStartup, so we use the watchdog to trigger a reboot */
+	{ "shutdown.reboot", 0, 0, "UPS.OutletSystem.Outlet.TLWatchdog", NULL, "10", HU_TYPE_CMD, NULL },
+
 	/* WARNING: if this timer expires, the UPS will reboot! Defaults to 60 seconds */
 	{ "reset.watchdog", 0, 0, "UPS.OutletSystem.Outlet.TLWatchdog", NULL, "60", HU_TYPE_CMD, NULL },
 
@@ -312,7 +317,7 @@ static hid_info_t tripplite_hid2nut[] = {
 	{ "beeper.disable", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "1", HU_TYPE_CMD, NULL },
 	{ "beeper.enable", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "2", HU_TYPE_CMD, NULL },
 	{ "beeper.mute", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "3", HU_TYPE_CMD, NULL },
-	
+
 	/* end of structure. */
 	{ NULL, 0, 0, NULL, NULL, NULL, 0, NULL }
 };
@@ -348,28 +353,28 @@ static int tripplite_claim(HIDDevice_t *hd) {
 		 */
 		case HP_VENDORID:
 			return 0;
-	
-		/* reject known non-HID devices */
-		/* not all Tripp Lite products are HID, some are "serial over USB". */
+
 		case TRIPPLITE_VENDORID:
+			/* reject known non-HID devices */
+			/* not all Tripp Lite products are HID, some are "serial over USB". */
 			if (hd->ProductID == 0x0001) {
 				/* e.g. SMART550USB, SMART3000RM2U */
 				upsdebugx(0, "This Tripp Lite device (%04x/%04x) is not supported by usbhid-ups.\n"
 						 "Please use the tripplite_usb driver instead.\n",
 						 hd->VendorID, hd->ProductID);
-			} else {
-				possibly_supported("Tripp Lite", hd);
+				return 0;
 			}
 
-			return 0;
-
-		/* by default, reject, unless the productid option is given */
-		default:
+			/* by default, reject, unless the productid option is given */
 			if (getval("productid")) {
 				return 1;
 			}
 
-			possibly_supported("Belkin", hd);
+			possibly_supported("TrippLite", hd);
+			return 0;
+
+		/* catch all (not really needed) */
+		default:
 			return 0;
 		}
 
