@@ -130,7 +130,7 @@
 POD ("Plain Old Documentation") - run through pod2html or perldoc. See
 perlpod(1) for more information.
 
-pod2man --name='TRIPPLITE_USB' --section=8 --release='$Rev: 1695 $' --center='Network UPS Tools (NUT)' tripplite_usb.c
+pod2man --name='TRIPPLITE_USB' --section=8 --release='$Rev: 2336 $' --center='Network UPS Tools (NUT)' tripplite_usb.c
 
 =head1 NAME
 
@@ -158,10 +158,14 @@ This driver has been tested with the following models:
 
 =over
 
+=item * INTERNETOFFICE700
+
 =item * OMNISV1000
 
 =item * OMNISV1500XL (some warnings)
 
+=item * SMART700USB
+
 =item * SMART1500RM2U
 
 =item * SMART2200RMXL2U
@@ -271,14 +275,16 @@ SMART1500RM2U units, you would have to find which USB bus number each unit is
 on (via C<lsusb>), which may result in ambiguities if the available USB ports
 are on the same bus.
 
-Some of the SMART*2U models seem to have an ID number that could be used in a
-future version of this driver to monitor multiple similar UPSes.
+Some of the SMART*2U models have an ID number, but because this ID is not
+exposed as a USB string descriptor, there is no easy way to use this ID to
+distinguish between multiple UPS units on a single machine.
 
 =head1 AUTHORS
 
-Charles Lepple E<lt>clepple+nut@ghz.ccE<gt>, based on the tripplite driver by
+Charles Lepple E<lt>clepple+nut@gmail.com<gt>, based on the tripplite driver by
 Rickard E. (Rik) Faith E<lt>faith@alephnull.comE<gt> and Nicholas Kain
-E<lt>nicholas@kain.usE<gt>.
+E<lt>nicholas@kain.usE<gt>. Please do not email the authors directly - use the
+nut-upsdev mailing list.
 
 A Tripp Lite OMNIVS1000 was graciously donated to the NUT project by:
 
@@ -854,7 +860,7 @@ static int instcmd(const char *cmdname, const char *extra)
 		return control_outlet(0, 0) ? STAT_INSTCMD_HANDLED : STAT_INSTCMD_UNKNOWN;
 	}
 	if (!strcasecmp(cmdname, "load.on")) {
-		return control_outlet(0, 0) ? STAT_INSTCMD_HANDLED : STAT_INSTCMD_UNKNOWN;
+		return control_outlet(0, 1) ? STAT_INSTCMD_HANDLED : STAT_INSTCMD_UNKNOWN;
 	}
 	/* code for individual outlets is in setvar() */
 #if 0
@@ -968,20 +974,7 @@ void upsdrv_initinfo(void)
 	int  va, ret;
 	unsigned int proto_number = 0;
 
-	/* Reset watchdog: */
-	ret = send_cmd(w_msg, sizeof(w_msg), w_value, sizeof(w_value)-1);
-	if(ret <= 0) {
-		if(ret == -EPIPE) {
-			fatalx(EXIT_FAILURE, "Could not reset watchdog. Please check and"
-				"see if usbhid-ups(8) works with this UPS.");
-		} else {
-			upslogx(3, "Could not reset watchdog. Please send model "
-				"information to nut-upsdev mailing list");
-		}
-	}
-
-	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
-
+	/* Read protocol: */
 	ret = send_cmd(proto_msg, sizeof(proto_msg), proto_value, sizeof(proto_value)-1);
 	if(ret <= 0) {
 		fatalx(EXIT_FAILURE, "Error reading protocol");
@@ -998,6 +991,23 @@ void upsdrv_initinfo(void)
 
 	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
 
+	/* Reset watchdog: */
+	/* Watchdog not supported on TRIPP_LITE_SMARTPRO models */
+	if(tl_model != TRIPP_LITE_SMARTPRO ) {
+		ret = send_cmd(w_msg, sizeof(w_msg), w_value, sizeof(w_value)-1);
+		if(ret <= 0) {
+			if(ret == -EPIPE) {
+				fatalx(EXIT_FAILURE, "Could not reset watchdog. Please check and"
+						"see if usbhid-ups(8) works with this UPS.");
+			} else {
+				upslogx(3, "Could not reset watchdog. Please send model "
+						"information to nut-upsdev mailing list");
+			}
+		}
+	}
+
+	/* - * - * - * - * - * - * - * - * - * - * - * - * - * - * - */
+
 	ret = send_cmd(s_msg, sizeof(s_msg), s_value, sizeof(s_value)-1);
 	if(ret <= 0) {
 		fatalx(EXIT_FAILURE, "Could not retrieve status ... is this an OMNIVS model?");
