@@ -1,28 +1,34 @@
 /* usbhid-ups.c - Driver for USB and serial (MGE SHUT) HID UPS units
- * 
+ *
  * Copyright (C)
- *   2003-2005 Arnaud Quette <http://arnaud.quette.free.fr/contact.html>
+ *   2003-2009 Arnaud Quette <arnaud.quette@gmail.com>
  *   2005      John Stamp <kinsayder@hotmail.com>
  *   2005-2006 Peter Selinger <selinger@users.sourceforge.net>
- *   2007      Arjen de Korte <adkorte-guest@alioth.debian.org>
+ *   2007-2009 Arjen de Korte <adkorte-guest@alioth.debian.org>
  *
- * This program is sponsored by MGE UPS SYSTEMS - opensource.mgeups.com
+ * This program was sponsored by MGE UPS SYSTEMS, and now Eaton
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or 
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ *
+ * TODO list:
+ * - set ST_FLAG_RW according to HIDData_t->Attribute (ATTR_DATA_CST-ATTR_NVOL_VOL)
  */
 
+#define DRIVER_NAME	"Generic HID driver"
+#define DRIVER_VERSION		"0.34"
+
 #include "main.h"
 #include "libhid.h"
 #include "usbhid-ups.h"
@@ -37,13 +43,10 @@
 	#include "belkin-hid.h"
 	#include "cps-hid.h"
 	#include "liebert-hid.h"
+	#include "powercom-hid.h"
 	#include "tripplite-hid.h"
 #endif
 
-#ifdef HAVE_SSL
-#include <openssl/sha.h>
-#endif
-
 /* master list of avaiable subdrivers */
 static subdriver_t *subdriver_list[] = {
 #ifndef SHUT_MODE
@@ -55,17 +58,33 @@ static subdriver_t *subdriver_list[] = {
 	&belkin_subdriver,
 	&cps_subdriver,
 	&liebert_subdriver,
+	&powercom_subdriver,
 	&tripplite_subdriver,
 #endif
 	NULL
 };
 
+upsdrv_info_t upsdrv_info = {
+	DRIVER_NAME,
+	DRIVER_VERSION,
+	"Arnaud Quette <arnaud.quette@gmail.com>\n" \
+	"Peter Selinger <selinger@users.sourceforge.net>\n" \
+	"Arjen de Korte <adkorte-guest@alioth.debian.org>\n" \
+	"John Stamp <kinsayder@hotmail.com>",
+	/*FIXME: link the subdrivers? do the same as for the mibs! */
+#ifndef SHUT_MODE
+	DRV_STABLE,
+#else
+	DRV_EXPERIMENTAL,
+#endif
+	{ &comm_upsdrv_info, NULL }
+};
+
 /* Data walk modes */
 typedef enum {
 	HU_WALKMODE_INIT = 0,
 	HU_WALKMODE_QUICK_UPDATE,
-	HU_WALKMODE_FULL_UPDATE,
-	HU_WALKMODE_RELOAD
+	HU_WALKMODE_FULL_UPDATE
 } walkmode_t;
 
 /* pointer to the active subdriver object (changed in callback() function) */
@@ -78,11 +97,15 @@ static HIDDeviceMatcher_t *subdriver_matcher = NULL;
 #ifndef SHUT_MODE
 static HIDDeviceMatcher_t *exact_matcher = NULL;
 static HIDDeviceMatcher_t *regex_matcher = NULL;
-static unsigned char *checksum = NULL;
 #endif
 static int pollfreq = DEFAULT_POLLFREQ;
 static int ups_status = 0;
 static bool_t data_has_changed = FALSE; /* for SEMI_STATIC data polling */
+#ifndef SUN_LIBUSB
+bool_t use_interrupt_pipe = TRUE;
+#else
+bool_t use_interrupt_pipe = FALSE;
+#endif
 static time_t lastpoll; /* Timestamp the last polling */
 hid_dev_handle_t udev;
 
@@ -105,9 +128,6 @@ static double interval(void);
 /* global variables */
 HIDDesc_t	*pDesc = NULL;		/* parsed Report Descriptor */
 reportbuf_t	*reportbuf = NULL;	/* buffer for most recent reports */
-#ifndef SHUT_MODE
-matching_t	matching = NORMAL;
-#endif
 
 /* ---------------------------------------------------------------------- */
 /* data for processing boolean values from UPS */
@@ -187,20 +207,6 @@ static status_lkp_t status_info[] = {
 	{ NULL, 0 },
 };
 
-#ifndef SHUT_MODE
-typedef struct {
-	char		*val;
-	matching_t	mode;
-} matching_lkp_t;
-
-static matching_lkp_t matching_info[] = {
-	{ "normal", NORMAL },
-	{ "checksum", CHECKSUM },
-	{ "delayed", DELAYED },
-	{ NULL, NORMAL },
-};
-#endif
-
 /* ---------------------------------------------------------------------- */
 /* value lookup tables and generic lookup functions */
 
@@ -212,7 +218,7 @@ static matching_lkp_t matching_info[] = {
    variable ups_status. Interpretation happens in ups_status_set,
    where they are converted to standard NUT status strings. Notice
    that the below conversions do not yield standard NUT status
-   strings; this in indicated being in lower-case characters. 
+   strings; this in indicated being in lower-case characters.
 
    The reason to separate the collection of information from its
    interpretation is that not each report received from the UPS may
@@ -409,7 +415,7 @@ static char *date_conversion_fun(double value)
 		return "not set";
 	}
 
-	year = 1980 + ((long)value >> 9); /* negative value represents pre-1980 date */ 
+	year = 1980 + ((long)value >> 9); /* negative value represents pre-1980 date */
 	month = ((long)value >> 5) & 0x0f;
 	day = (long)value & 0x1f;
 
@@ -427,7 +433,7 @@ info_lkp_t date_conversion[] = {
 static char *hex_conversion_fun(double value)
 {
 	static char buf[20];
-	
+
 	snprintf(buf, sizeof(buf), "%08lx", (long)value);
 
 	return buf;
@@ -455,7 +461,7 @@ info_lkp_t stringid_conversion[] = {
 static char *divide_by_10_conversion_fun(double value)
 {
 	static char buf[20];
-	
+
 	snprintf(buf, sizeof(buf), "%0.1f", value * 0.1);
 
 	return buf;
@@ -470,7 +476,7 @@ info_lkp_t divide_by_10_conversion[] = {
 static char *kelvin_celsius_conversion_fun(double value)
 {
 	static char buf[20];
-	
+
 	snprintf(buf, sizeof(buf), "%.1f", value - 273.15);
 
 	return buf;
@@ -530,7 +536,7 @@ int instcmd(const char *cmdname, const char *extradata)
 
 	upsdebugx(1, "instcmd(%s, %s)", cmdname, extradata ? extradata : "[NULL]");
 
-	/* Retrieve and check netvar & item_path */	
+	/* Retrieve and check netvar & item_path */
 	hidups_item = find_nut_info(cmdname);
 
 	/* Check for fallback if not found */
@@ -567,15 +573,15 @@ int instcmd(const char *cmdname, const char *extradata)
 		}
 
 		upsdebugx(2, "instcmd: info element unavailable %s\n", cmdname);
-		return STAT_INSTCMD_UNKNOWN;
+		return STAT_INSTCMD_INVALID;
 	}
 
 	/* Check if the item is an instant command */
-	if (!hidups_item->hidflags & HU_TYPE_CMD) {
+	if (!(hidups_item->hidflags & HU_TYPE_CMD)) {
 		upsdebugx(2, "instcmd: %s is not an instant command\n", cmdname);
-		return STAT_INSTCMD_UNKNOWN;
+		return STAT_INSTCMD_INVALID;
 	}
-	
+
 	/* If extradata is empty, use the default value from the HID-to-NUT table */
 	val = extradata ? extradata : hidups_item->dfl;
 
@@ -593,9 +599,9 @@ int instcmd(const char *cmdname, const char *extradata)
 		data_has_changed = TRUE;
 		return STAT_INSTCMD_HANDLED;
 	}
-	
+
 	upsdebugx(3, "instcmd: FAILED\n"); /* TODO: HANDLED but FAILED, not UNKNOWN! */
-	return STAT_INSTCMD_UNKNOWN;
+	return STAT_INSTCMD_FAILED;
 }
 
 /* set r/w variable to a value. */
@@ -606,16 +612,16 @@ int setvar(const char *varname, const char *val)
 
 	upsdebugx(1, "setvar(%s, %s)", varname, val);
 
-	/* retrieve and check netvar & item_path */	
+	/* retrieve and check netvar & item_path */
 	hidups_item = find_nut_info(varname);
-	
+
 	if (hidups_item == NULL) {
 		upsdebugx(2, "setvar: info element unavailable %s\n", varname);
 		return STAT_SET_UNKNOWN;
 	}
 
 	/* Checking item writability and HID Path */
-	if (!hidups_item->info_flags & ST_FLAG_RW) {
+	if (!(hidups_item->info_flags & ST_FLAG_RW)) {
 		upsdebugx(2, "setvar: not writable %s\n", varname);
 		return STAT_SET_UNKNOWN;
 	}
@@ -627,7 +633,7 @@ int setvar(const char *varname, const char *val)
 		return STAT_SET_HANDLED;
 	}
 
-	/* SHUT_FLAG_ABSENT is the only case of HID Path == NULL */
+	/* HU_FLAG_ABSENT is the only case of HID Path == NULL */
 	if (hidups_item->hidpath == NULL) {
 		upsdebugx(2, "setvar: ID Path is NULL for %s\n", varname);
 		return STAT_SET_UNKNOWN;
@@ -655,7 +661,7 @@ int setvar(const char *varname, const char *val)
 void upsdrv_shutdown(void)
 {
 	upsdebugx(1, "upsdrv_shutdown...");
-	
+
 	/* Try to shutdown with delay */
 	if (instcmd("shutdown.return", NULL) == STAT_INSTCMD_HANDLED) {
 		/* Shutdown successful */
@@ -668,6 +674,12 @@ void upsdrv_shutdown(void)
 		return;
 	}
 
+	/* If the above doesn't work, try load.off.delay */
+	if (instcmd("load.off.delay", NULL) == STAT_INSTCMD_HANDLED) {
+		/* Shutdown successful */
+		return;
+	}
+
 	fatalx(EXIT_FAILURE, "Shutdown failed!");
 }
 
@@ -676,21 +688,24 @@ void upsdrv_help(void)
 	/* FIXME: to be completed */
 }
 
-void upsdrv_makevartable(void) 
+void upsdrv_makevartable(void)
 {
 	char temp [MAX_STRING_SIZE];
-	
+
 	upsdebugx(1, "upsdrv_makevartable...");
 
 	snprintf(temp, sizeof(temp), "Set shutdown delay, in seconds (default=%s)", DEFAULT_OFFDELAY);
 	addvar(VAR_VALUE, HU_VAR_OFFDELAY, temp);
-	
+
 	snprintf(temp, sizeof(temp), "Set startup delay, in seconds (default=%s)", DEFAULT_ONDELAY);
 	addvar(VAR_VALUE, HU_VAR_ONDELAY, temp);
-	
+
 	snprintf(temp, sizeof(temp), "Set polling frequency, in seconds, to reduce data flow (default=%d)",
 		DEFAULT_POLLFREQ);
 	addvar(VAR_VALUE, HU_VAR_POLLFREQ, temp);
+
+	addvar(VAR_FLAG, "pollonly", "Don't use interrupt pipe, only use polling");
+
 #ifndef SHUT_MODE
 	/* allow -x vendor=X, vendorid=X, product=X, productid=X, serial=X */
 	addvar(VAR_VALUE, "vendor", "Regular expression to match UPS Manufacturer string");
@@ -699,24 +714,13 @@ void upsdrv_makevartable(void)
 	addvar(VAR_VALUE, "vendorid", "Regular expression to match UPS Manufacturer numerical ID (4 digits hexadecimal)");
 	addvar(VAR_VALUE, "productid", "Regular expression to match UPS Product numerical ID (4 digits hexadecimal)");
 	addvar(VAR_VALUE, "bus", "Regular expression to match USB bus name");
-	addvar(VAR_VALUE, "matching", "Use 'normal' or 'delayed' matching");
 	addvar(VAR_FLAG, "explore", "Diagnostic matching of unsupported UPS");
 #endif
 }
 
-void upsdrv_banner(void)
-{
-	printf("Network UPS Tools: %s %s - core %s (%s)\n\n",
-		comm_driver->name, comm_driver->version,
-		DRIVER_VERSION, UPS_VERSION);
-#ifdef SHUT_MODE /* newmge-shut is experimental */
-	experimental_driver = 1;
-#endif
-}
-
 #define	MAX_EVENT_NUM	32
 
-void upsdrv_updateinfo(void) 
+void upsdrv_updateinfo(void)
 {
 	hid_info_t	*item;
 	HIDData_t	*event[MAX_EVENT_NUM];
@@ -754,19 +758,18 @@ void upsdrv_updateinfo(void)
 	interval();
 #endif
 	/* Get HID notifications on Interrupt pipe first */
-	evtCount = HIDGetEvents(udev, event, MAX_EVENT_NUM);
-
-	upsdebugx(1, "Got %i HID objects...", evtCount);
+	if (use_interrupt_pipe == TRUE) {
+		evtCount = HIDGetEvents(udev, event, MAX_EVENT_NUM);
+		upsdebugx(1, "Got %i HID objects...", (evtCount >= 0) ? evtCount : 0);
+	} else {
+		evtCount = 0;
+		upsdebugx(1, "Not using interrupt pipe...");
+	}
 
 	/* Process pending events (HID notifications on Interrupt pipe) */
 	for (i = 0; i < evtCount; i++) {
 
-		/* Many UPSes have broken input reports, so we only use the
-		   corresponding feature reports. By setting the age to zero
-		   for the first value we retrieve, we make sure that the
-		   contents of the report buffer are considered stale and we
-		   poll for fresh data. */
-		if (HIDGetDataValue(udev, event[i], &value, (i > 0) ? poll_interval : 0) != 1)
+		if (HIDGetDataValue(udev, event[i], &value, poll_interval) != 1)
 			continue;
 
 		if (nut_debug_level >= 2) {
@@ -828,20 +831,20 @@ void upsdrv_initinfo(void)
 
 	upsdebugx(1, "upsdrv_initinfo...");
 
+	dstate_setinfo("driver.version.data", "%s", subdriver->name);
+	dstate_setinfo("driver.version.internal", DRIVER_VERSION);
+
 	/* init polling frequency */
 	val = getval(HU_VAR_POLLFREQ);
-	if (val)
+	if (val) {
 		pollfreq = atoi(val);
+	}
 
 	dstate_setinfo("driver.parameter.pollfreq", "%d", pollfreq);
 
-	if (dstate_getinfo("ups.delay.shutdown") && dstate_getinfo("ups.delay.start")) {
-		/* Add defaults with a delay value of '0' (= immediate) */
-		dstate_addcmd("load.off");
-		dstate_addcmd("load.on");
-		/* Add composite instcmds (require setting multiple HID values) */
-		dstate_addcmd("shutdown.return");
-		dstate_addcmd("shutdown.stayoff");
+	/* ignore (broken) interrupt pipe */
+	if (testvar("pollonly")) {
+		use_interrupt_pipe = FALSE;
 	}
 
 	time(&lastpoll);
@@ -865,7 +868,6 @@ void upsdrv_initups(void)
 	subdriver_matcher = device_path;
 #else
 	char *regex_array[6];
-	matching_lkp_t	*key;
 
 	upsdebugx(1, "upsdrv_initups...");
 
@@ -895,24 +897,8 @@ void upsdrv_initups(void)
 		fatalx(EXIT_FAILURE, "invalid regular expression: %s", regex_array[ret]);
 	}
 
-	val = getval("matching");
-
-	for (key = matching_info; val && key->val; key++) {
-		if (!strcasecmp(val, key->val)) {
-			matching = key->mode;
-			dstate_setinfo("driver.parameter.matching", "%s", key->val);
-			break;
-		}
-	}
-
-	if (val && !key->val) {
-		fatalx(EXIT_FAILURE, "Error: '%s' not a valid value for 'matching'", val);
-	}
-
-	if (matching != DELAYED) {
-		/* link the matchers */
-		subdriver_matcher->next = regex_matcher;
-	}
+	/* link the matchers */
+	subdriver_matcher->next = regex_matcher;
 #endif /* SHUT_MODE */
 
 	/* Search for the first supported UPS matching the
@@ -924,19 +910,42 @@ void upsdrv_initups(void)
 	hd = &curDevice;
 
 	upsdebugx(1, "Detected a UPS: %s/%s", hd->Vendor ? hd->Vendor : "unknown",
-		 hd->Product ? hd->Product : "unknown");
+		hd->Product ? hd->Product : "unknown");
 
 	if (hid_ups_walk(HU_WALKMODE_INIT) == FALSE) {
 		fatalx(EXIT_FAILURE, "Can't initialize data from HID UPS");
 	}
 
-	/* Retrieve user defined delay settings */
-	if (dstate_getinfo("ups.delay.start") && (val = getval(HU_VAR_ONDELAY))) {
-		dstate_setinfo("ups.delay.start", val);
+	if (dstate_getinfo("ups.delay.start")) {
+		/* Retrieve user defined delay settings */
+		val = getval(HU_VAR_ONDELAY);
+		if (val) {
+			dstate_setinfo("ups.delay.start", "%ld", strtol(val, NULL, 10));
+		}
 	}
 
-	if (dstate_getinfo("ups.delay.shutdown") && (val = getval(HU_VAR_OFFDELAY))) {
-		dstate_setinfo("ups.delay.start", val);
+	if (dstate_getinfo("ups.delay.shutdown")) {
+		/* Retrieve user defined delay settings */
+		val = getval(HU_VAR_OFFDELAY);
+		if (val) {
+			dstate_setinfo("ups.delay.shutdown", "%ld", strtol(val, NULL, 10));
+		}
+	}
+
+	if (find_nut_info("load.off.delay")) {
+		/* Adds default with a delay value of '0' (= immediate) */
+		dstate_addcmd("load.off");
+	}
+
+	if (find_nut_info("load.on.delay")) {
+		/* Adds default with a delay value of '0' (= immediate) */
+		dstate_addcmd("load.on");
+	}
+
+	if (find_nut_info("load.off.delay") && find_nut_info("load.on.delay")) {
+		/* Add composite instcmds (require setting multiple HID values) */
+		dstate_addcmd("shutdown.return");
+		dstate_addcmd("shutdown.stayoff");
 	}
 }
 
@@ -1012,38 +1021,6 @@ static int callback(hid_dev_handle_t udev, HIDDevice_t *hd, unsigned char *rdbuf
 	int ret;
 #endif
 	upsdebugx(2, "Report Descriptor size = %d", rdlen);
-	
-#ifndef SHUT_MODE
-	if (matching == CHECKSUM) {
-#ifdef HAVE_SSL
-		unsigned char buf[SHA_DIGEST_LENGTH];
-
-		SHA1(rdbuf, rdlen, buf);
-
-		upsdebug_hex(3, "SHA-1 checksum of report descriptor\n", buf, sizeof(buf));
-
-		if (checksum) {
-			if (!memcmp(checksum, buf, sizeof(buf))) {
-				upsdebugx(2, "SHA-1 checksum matches!");
-				return 1;
-			}
-
-			upsdebugx(2, "SHA-1 checksum doesn't match!");
-			return 0;
-		}
-
-		checksum = calloc(1, sizeof(buf));
-		if (!checksum) {
-			fatal_with_errno(EXIT_FAILURE, "Can't store SHA-1 checksum of report descriptor");
-		}
-
-		memcpy(checksum, buf, sizeof(buf));
-#else
-		fatalx(EXIT_FAILURE, "Checksum matching requires SSL support!");
-#endif /* HAVE_SSL */
-	}
-#endif /* SHUT_MODE */
-
 	upsdebug_hex(3, "Report Descriptor", rdbuf, rdlen);
 
 	/* Parse Report Descriptor */
@@ -1081,36 +1058,6 @@ static int callback(hid_dev_handle_t udev, HIDDevice_t *hd, unsigned char *rdbuf
 	HIDDumpTree(udev, subdriver->utab);
 
 #ifndef SHUT_MODE
-	if (matching == DELAYED) {
-		USBDeviceMatcher_t *m;
-
-		/* apply subdriver specific formatting
-		 */
-		mfr = subdriver->format_mfr(hd);
-		model = subdriver->format_model(hd);
-		serial = subdriver->format_serial(hd);
-
-		upsdebugx(2, "- VendorID: %04x", hd->VendorID);
-		upsdebugx(2, "- ProductID: %04x", hd->ProductID);
-		upsdebugx(2, "- Manufacturer: %s", hd->Vendor ? hd->Vendor : "unknown");
-		upsdebugx(2, "- Product: %s", hd->Product ? hd->Product : "unknown");
-		upsdebugx(2, "- Serial Number: %s", hd->Serial ? hd->Serial : "unknown");
-		upsdebugx(2, "- Bus: %s", hd->Bus ? hd->Bus : "unknown");
-
-		for (m = regex_matcher; m; m = m->next) {
-			ret = matches(m, hd);
-			if (ret != 1) {
-				return 0;
-			}
-		}
-
-		if (exact_matcher) {
-			/* reload all NUT variables & commands */
-			hid_ups_walk(HU_WALKMODE_RELOAD);
-			return 1;
-		}
-	}
-
 	/* create a new matcher for later matching */
 	USBFreeExactMatcher(exact_matcher);
 	ret = USBNewExactMatcher(&exact_matcher, hd);
@@ -1120,17 +1067,13 @@ static int callback(hid_dev_handle_t udev, HIDDevice_t *hd, unsigned char *rdbuf
 	}
 
 	regex_matcher->next = exact_matcher;
-
-	if (matching != DELAYED) {
 #endif /* SHUT_MODE */
-		/* apply subdriver specific formatting
-		 */
-		mfr = subdriver->format_mfr(hd);
-		model = subdriver->format_model(hd);
-		serial = subdriver->format_serial(hd);
-#ifndef SHUT_MODE
-	}
-#endif
+
+	/* apply subdriver specific formatting */
+	mfr = subdriver->format_mfr(hd);
+	model = subdriver->format_model(hd);
+	serial = subdriver->format_serial(hd);
+
 	if (mfr != NULL) {
 		dstate_setinfo("ups.mfr", "%s", mfr);
 	} else {
@@ -1152,13 +1095,6 @@ static int callback(hid_dev_handle_t udev, HIDDevice_t *hd, unsigned char *rdbuf
 	dstate_setinfo("ups.vendorid", "%04x", hd->VendorID);
 	dstate_setinfo("ups.productid", "%04x", hd->ProductID);
 
-#ifndef SHUT_MODE
-	if (exact_matcher) {
-		/* reload all NUT variables & commands */
-		hid_ups_walk(HU_WALKMODE_RELOAD);
-		return 1;
-	}
-#endif
 	return 1;
 }
 
@@ -1186,7 +1122,7 @@ static bool_t hid_ups_walk(walkmode_t mode)
 	int		retcode;
 
 	/* 3 modes: HU_WALKMODE_INIT, HU_WALKMODE_QUICK_UPDATE and HU_WALKMODE_FULL_UPDATE */
-	
+
 	/* Device data walk ----------------------------- */
 	for (item = subdriver->hid2nut; item->info_type != NULL; item++) {
 
@@ -1201,44 +1137,17 @@ static bool_t hid_ups_walk(walkmode_t mode)
 		switch (mode)
 		{
 		/* Device capabilities enumeration */
-		case HU_WALKMODE_RELOAD:
-			/* Special case for handling server side variables */
-			if (item->hidflags & HU_FLAG_ABSENT) {
-				continue;
-			}
-
-			/* We never mapped this HID item, so skip it */
-			if (item->hiddata == NULL)
-				continue;
-
-			/* Refresh the NUT-to-HID mapping and fetch new data */
-			item->hiddata = HIDGetItemData(item->hidpath, subdriver->utab);
-			if (item->hiddata != NULL)
-				continue;
-
-			upsdebugx(1, "hid_ups_walk: RELOAD of %s failed!", item->info_type);
-
-			/* Uh oh, it was lost after reconnect, remove it */
-			if (item->hidflags & HU_TYPE_CMD) {
-				dstate_delcmd(item->info_type);
-				continue;
-			}
-
-			/* If NUT tupe is not "BOOL", remove this variable */
-			if (strncmp(item->info_type, "BOOL", 4)) {
-				dstate_delinfo(item->info_type);
-				continue;
-			}
-
-			/* We only flag here, no communication should take place */
-			continue;
-			
 		case HU_WALKMODE_INIT:
 			/* Apparently, we are reconnecting, so
 			 * NUT-to-HID translation is already good */
 			if (item->hiddata != NULL)
 				break;
 
+			/* Create the NUT-to-HID mapping */
+			item->hiddata = HIDGetItemData(item->hidpath, subdriver->utab);
+			if (item->hiddata == NULL)
+				continue;
+
 			/* Special case for handling server side variables */
 			if (item->hidflags & HU_FLAG_ABSENT) {
 
@@ -1246,7 +1155,7 @@ static bool_t hid_ups_walk(walkmode_t mode)
 				if (dstate_getinfo(item->info_type))
 					continue;
 
-				dstate_setinfo(item->info_type, item->dfl);
+				dstate_setinfo(item->info_type, "%s", item->dfl);
 				dstate_setflags(item->info_type, item->info_flags);
 
 				/* Set max length for strings, if needed */
@@ -1256,11 +1165,6 @@ static bool_t hid_ups_walk(walkmode_t mode)
 				continue;
 			}
 
-			/* Create the NUT-to-HID mapping */
-			item->hiddata = HIDGetItemData(item->hidpath, subdriver->utab);
-			if (item->hiddata == NULL)
-				continue;
-
 			/* Allow duplicates for these NUT variables... */
 			if (!strncmp(item->info_type, "ups.alarm", 9)) {
 				break;
@@ -1279,8 +1183,8 @@ static bool_t hid_ups_walk(walkmode_t mode)
 			/* Quick update only deals with status and alarms! */
 			if (!(item->hidflags & HU_FLAG_QUICK_POLL))
 				continue;
-			
-			break; 
+
+			break;
 
 		case HU_WALKMODE_FULL_UPDATE:
 			/* These don't need polling after initinfo() */
@@ -1291,8 +1195,8 @@ static bool_t hid_ups_walk(walkmode_t mode)
 			if ( (item->hidflags & HU_FLAG_SEMI_STATIC) && (data_has_changed == FALSE) )
 				continue;
 
-			break; 
-		
+			break;
+
 		default:
 			fatalx(EXIT_FAILURE, "hid_ups_walk: unknown update mode!");
 		}
@@ -1301,14 +1205,14 @@ static bool_t hid_ups_walk(walkmode_t mode)
 
 		switch (retcode)
 		{
-		case -EBUSY:
-			upslog_with_errno(LOG_DEBUG, "Got disconnected by another driver");
-		case -EPERM:
-		case -EPIPE:
-		case -ENODEV:
-		case -EACCES:
-		case -EIO:
-		case -ENOENT:
+		case -EBUSY:		/* Device or resource busy */
+			upslog_with_errno(LOG_CRIT, "Got disconnected by another driver");
+		case -EPERM:		/* Operation not permitted */
+		case -ENODEV:		/* No such device */
+		case -EACCES:		/* Permission denied */
+		case -EIO:		/* I/O error */
+		case -ENXIO:		/* No such device or address */
+		case -ENOENT:		/* No such file or directory */
 			/* Uh oh, got to reconnect! */
 			hd = NULL;
 			return FALSE;
@@ -1319,9 +1223,12 @@ static bool_t hid_ups_walk(walkmode_t mode)
 		case 0:
 			continue;
 
+		case -ETIMEDOUT:	/* Connection timed out */
+		case -EOVERFLOW:	/* Value too large for defined data type */
+		case -EPROTO:		/* Protocol error */
+		case -EPIPE:		/* Broken pipe */
 		default:
 			/* Don't know what happened, try again later... */
-			upsdebug_with_errno(2, "HIDGetDataValue");
 			continue;
 		}
 
@@ -1340,6 +1247,7 @@ static bool_t hid_ups_walk(walkmode_t mode)
 			continue;
 
 		if (mode == HU_WALKMODE_INIT) {
+			info_lkp_t	*info_lkp;
 
 			dstate_setflags(item->info_type, item->info_flags);
 
@@ -1347,6 +1255,20 @@ static bool_t hid_ups_walk(walkmode_t mode)
 			if (item->info_flags & ST_FLAG_STRING) {
 				dstate_setaux(item->info_type, item->info_len);
 			}
+
+			/* Set enumerated values, only if the data has ST_FLAG_RW */
+			if (!(item->hidflags & HU_FLAG_ENUM) || !(item->info_flags & ST_FLAG_RW)) {
+				continue;
+			}
+
+			/* Loop on all existing values */
+			for (info_lkp = item->hid2info; info_lkp != NULL
+				&& info_lkp->nut_value != NULL; info_lkp++) {
+				/* Check if this value is supported */
+				if (hu_find_infoval(item->hid2info, info_lkp->hid_value) != NULL) {
+					dstate_addenum(item->info_type, "%s", info_lkp->nut_value);
+				}
+			}
 		}
 	}
 
@@ -1361,12 +1283,8 @@ static int reconnect_ups(void)
 	upsdebugx(4, "= device has been disconnected, try to reconnect =");
 	upsdebugx(4, "==================================================");
 
-	ret = comm_driver->open(&udev, &curDevice, subdriver_matcher,
-#ifdef SHUT_MODE
-		NULL);
-#else
-		(matching == NORMAL) ? NULL : &callback);
-#endif
+	ret = comm_driver->open(&udev, &curDevice, subdriver_matcher, NULL);
+
 	if (ret > 0) {
 		return 1;
 	}
@@ -1481,12 +1399,9 @@ static hid_info_t *find_nut_info(const char *varname)
 
 		if (hidups_item->hiddata != NULL)
 			return hidups_item;
-
-		if (hidups_item->hidflags & HU_FLAG_ABSENT)
-			return hidups_item;
 	}
 
-	upsdebugx(2, "find_nut_info: unknown info type: %s\n", varname);
+	upsdebugx(2, "find_nut_info: unknown info type: %s", varname);
 	return NULL;
 }
 
@@ -1496,13 +1411,13 @@ static hid_info_t *find_nut_info(const char *varname)
 static hid_info_t *find_hid_info(const HIDData_t *hiddata)
 {
 	hid_info_t *hidups_item;
-	
+
 	for (hidups_item = subdriver->hid2nut; hidups_item->info_type != NULL ; hidups_item++) {
 
-		/* Skip NULL HID path (server side vars) */
-		if (hidups_item->hidpath == NULL)
+		/* Skip server side vars */
+		if (hidups_item->hidflags & HU_FLAG_ABSENT)
 			continue;
-	
+
 		if (hidups_item->hiddata == hiddata)
 			return hidups_item;
 	}
@@ -1514,17 +1429,23 @@ static hid_info_t *find_hid_info(const HIDData_t *hiddata)
 /* useful for set with value lookup... */
 static long hu_find_valinfo(info_lkp_t *hid2info, const char* value)
 {
-	info_lkp_t *info_lkp;
-	
-	for (info_lkp = hid2info; info_lkp->nut_value != NULL; info_lkp++) {
+	info_lkp_t	*info_lkp;
 
+	/* if a conversion function is defined, use 'value' as argument for it */
+	if (hid2info->nuf != NULL) {
+		double	hid_value;
+		hid_value = hid2info->nuf(value);
+		upsdebugx(5, "hu_find_valinfo: found %g (value: %s)", hid_value, value);
+		return hid_value;
+	}
+
+	for (info_lkp = hid2info; info_lkp->nut_value != NULL; info_lkp++) {
 		if (!(strcmp(info_lkp->nut_value, value))) {
-			upsdebugx(5, "hu_find_valinfo: found %s (value: %ld)\n",
-				info_lkp->nut_value, info_lkp->hid_value);
-	
+			upsdebugx(5, "hu_find_valinfo: found %s (value: %ld)", info_lkp->nut_value, info_lkp->hid_value);
 			return info_lkp->hid_value;
 		}
 	}
+
 	upsdebugx(3, "hu_find_valinfo: no matching HID value for this INFO_* value (%s)", value);
 	return -1;
 }
@@ -1532,28 +1453,21 @@ static long hu_find_valinfo(info_lkp_t *hid2info, const char* value)
 /* find the NUT value matching that HID Item value */
 static char *hu_find_infoval(info_lkp_t *hid2info, const double value)
 {
-	info_lkp_t *info_lkp;
-	char *nut_value;
+	info_lkp_t	*info_lkp;
 
-	upsdebugx(5, "hu_find_infoval: searching for value = %g\n", value);
-
-	/* if a conversion function is defined,
-	 * use 'value' as argument for it */
+	/* if a conversion function is defined, use 'value' as argument for it */
 	if (hid2info->fun != NULL) {
-		nut_value = hid2info->fun(value);
-		upsdebugx(5, "hu_find_infoval: found %s (value: %g)\n", nut_value, value);
-		return nut_value;
+		return hid2info->fun(value);
 	}
 
 	/* use 'value' as an index for a lookup in an array */
 	for (info_lkp = hid2info; info_lkp->nut_value != NULL; info_lkp++) {
 		if (info_lkp->hid_value == (long)value) {
-			upsdebugx(5, "hu_find_infoval: found %s (value: %ld)\n",
-					info_lkp->nut_value, (long)value);
-
+			upsdebugx(5, "hu_find_infoval: found %s (value: %ld)", info_lkp->nut_value, (long)value);
 			return info_lkp->nut_value;
 		}
 	}
+
 	upsdebugx(3, "hu_find_infoval: no matching INFO_* value for this HID value (%g)", value);
 	return NULL;
 }
@@ -1561,7 +1475,7 @@ static char *hu_find_infoval(info_lkp_t *hid2info, const double value)
 /* return -1 on failure, 0 for a status update and 1 in all other cases */
 static int ups_infoval_set(hid_info_t *item, double value)
 {
-	char *nutvalue;
+	char	*nutvalue;
 
 	/* need lookup'ed translation? */
 	if (item->hid2info != NULL){
@@ -1583,7 +1497,7 @@ static int ups_infoval_set(hid_info_t *item, double value)
 			return 0;
 		}
 
-		dstate_setinfo(item->info_type, item->dfl, nutvalue);
+		dstate_setinfo(item->info_type, "%s", nutvalue);
 	} else {
 		dstate_setinfo(item->info_type, item->dfl, value);
 	}
