@@ -6,7 +6,7 @@
 # drivers/usbhid-ups -DD -u root -x generic -x vendorid=XXXX auto
 #
 # Usage: cat debuginfo | path-to-subdriver.sh
-# 
+#
 # See also: docs/hid-subdrivers.txt
 
 usage() {
@@ -128,8 +128,9 @@ cat > "$HFILE" <<EOF
 /* ${HFILE} - subdriver to monitor ${DRIVER} USB/HID devices with NUT
  *
  *  Copyright (C)
- *  2003 - 2005 Arnaud Quette <arnaud.quette@free.fr>
- *  2005 - 2006 Peter Selinger <selinger@users.sourceforge.net>         
+ *  2003 - 2009	Arnaud Quette <ArnaudQuette@Eaton.com>
+ *  2005 - 2006	Peter Selinger <selinger@users.sourceforge.net>
+ *  2008 - 2009	Arjen de Korte <adkorte-guest@alioth.debian.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -144,7 +145,6 @@ cat > "$HFILE" <<EOF
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
  */
 
 #ifndef ${UDRIVER}_HID_H
@@ -152,8 +152,6 @@ cat > "$HFILE" <<EOF
 
 #include "usbhid-ups.h"
 
-#define ${UDRIVER}_HID_VERSION	"${DRIVER} HID 0.1"
-
 extern subdriver_t ${LDRIVER}_subdriver;
 
 #endif /* ${UDRIVER}_HID_H */
@@ -165,8 +163,9 @@ cat > "$CFILE" <<EOF
 /* ${CFILE} - subdriver to monitor ${DRIVER} USB/HID devices with NUT
  *
  *  Copyright (C)
- *  2003 - 2005 Arnaud Quette <arnaud.quette@free.fr>
- *  2005 - 2006 Peter Selinger <selinger@users.sourceforge.net>         
+ *  2003 - 2009	Arnaud Quette <ArnaudQuette@Eaton.com>
+ *  2005 - 2006	Peter Selinger <selinger@users.sourceforge.net>
+ *  2008 - 2009	Arjen de Korte <adkorte-guest@alioth.debian.org>
  *
  *  Note: this subdriver was initially generated as a "stub" by the
  *  path-to-subdriver script. It must be customized.
@@ -184,19 +183,28 @@ cat > "$CFILE" <<EOF
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
  */
 
 #include "usbhid-ups.h"
 #include "${HFILE}"
-#include "extstate.h" /* for ST_FLAG_STRING */
-#include "dstate.h"   /* for STAT_INSTCMD_HANDLED */
-#include "main.h"     /* for getval() */
-#include "common.h"
+#include "main.h"	/* for getval() */
+#include "usb-common.h"
 
-#define ${UDRIVER}_HID_VERSION      "${DRIVER} HID 0.1"
+#define ${UDRIVER}_HID_VERSION	"${DRIVER} HID 0.1"
+/* FIXME: experimental flag to be put in upsdrv_info */
+
+/* ${DRIVER} */
+#define ${UDRIVER}_VENDORID	0x${VENDORID}
+
+/* USB IDs device table */
+static usb_device_id_t ${LDRIVER}_usb_device_table[] = {
+	/* ${DRIVER} */
+	{ USB_DEVICE(${UDRIVER}_VENDORID, 0x${PRODUCTID}), NULL },
+
+	/* Terminating entry */
+	{ -1, -1, NULL }
+};
 
-#define ${UDRIVER}_VENDORID 0x${VENDORID}
 
 /* --------------------------------------------------------------- */
 /*      Vendor-specific usage table */
@@ -209,7 +217,7 @@ EOF
 cat "$SUBST" | sed 's/\(.*\) \(.*\)/\t{ "\2",\t0x\1 },/' >> "$CFILE"
 
 cat >> "$CFILE" <<EOF
-	{  "\0", 0x0 }
+	{  NULL, 0 }
 };
 
 static usage_tables_t ${LDRIVER}_utab[] = {
@@ -226,10 +234,10 @@ static hid_info_t ${LDRIVER}_hid2nut[] = {
 
 EOF
 
-cat "$NEWUTABLE" | while read U; do
+cat "$NEWUTABLE" | sort -u | while read U; do
     UL=`echo $U | tr A-Z a-z`
     cat >> "$CFILE" <<EOF
-  { "unmapped.${UL}", 0, 0, "${U}", NULL, "%.0f", HU_FLAG_OK, NULL },
+  { "unmapped.${UL}", 0, 0, "${U}", NULL, "%.0f", 0, NULL },
 EOF
 done
 
@@ -239,57 +247,40 @@ cat >> "$CFILE" <<EOF
   { NULL, 0, 0, NULL, NULL, NULL, 0, NULL }
 };
 
-/* shutdown method for ${DRIVER} */
-static int ${LDRIVER}_shutdown(int ondelay, int offdelay) {
-	/* FIXME: ondelay, offdelay currently not used */
-	
-	/* Default method */
-	upsdebugx(2, "Trying load.off.");
-	if (instcmd("load.off", NULL) == STAT_INSTCMD_HANDLED) {
-		return 1;
-	}
-	upsdebugx(2, "Shutdown failed.");
-	return 0;
-}
-
-static char *${LDRIVER}_format_model(HIDDevice *hd) {
+static char *${LDRIVER}_format_model(HIDDevice_t *hd) {
 	return hd->Product;
 }
 
-static char *${LDRIVER}_format_mfr(HIDDevice *hd) {
+static char *${LDRIVER}_format_mfr(HIDDevice_t *hd) {
 	return hd->Vendor ? hd->Vendor : "${DRIVER}";
 }
 
-static char *${LDRIVER}_format_serial(HIDDevice *hd) {
+static char *${LDRIVER}_format_serial(HIDDevice_t *hd) {
 	return hd->Serial;
 }
 
 /* this function allows the subdriver to "claim" a device: return 1 if
  * the device is supported by this subdriver, else 0. */
-static int ${LDRIVER}_claim(HIDDevice *hd) {
-	if (hd->VendorID != ${UDRIVER}_VENDORID) {
-		return 0;
-	}
-	switch (hd->ProductID) {
+static int ${LDRIVER}_claim(HIDDevice_t *hd)
+{
+	int status = is_usb_device_supported(${LDRIVER}_usb_device_table, hd->VendorID, hd->ProductID);
 
-	/* accept any known UPS - add devices here as needed */
-	case ${PRODUCTID}:
-		return 1;
-
-	/* by default, reject, unless the productid option is given */
-	default:
+	switch (status)
+	{
+	case POSSIBLY_SUPPORTED:
+		/* by default, reject, unless the productid option is given */
 		if (getval("productid")) {
 			return 1;
-		} else {
-			upsdebugx(1,
-"This ${DRIVER} device (%04x/%04x) is not (or perhaps not yet) supported\n"
-"by usbhid-ups. Please make sure you have an up-to-date version of NUT. If\n"
-"this does not fix the problem, try running the driver with the\n"
-"'-x productid=%04x' option. Please report your results to the NUT user's\n"
-"mailing list <nut-upsuser@lists.alioth.debian.org>.\n",
-						 hd->VendorID, hd->ProductID, hd->ProductID);
-			return 0;
 		}
+		possibly_supported("${DRIVER}", hd);
+		return 0;
+
+	case SUPPORTED:
+		return 1;
+
+	case NOT_SUPPORTED:
+	default:
+		return 0;
 	}
 }
 
@@ -298,7 +289,6 @@ subdriver_t ${LDRIVER}_subdriver = {
 	${LDRIVER}_claim,
 	${LDRIVER}_utab,
 	${LDRIVER}_hid2nut,
-	${LDRIVER}_shutdown,
 	${LDRIVER}_format_model,
 	${LDRIVER}_format_mfr,
 	${LDRIVER}_format_serial,
@@ -306,14 +296,12 @@ subdriver_t ${LDRIVER}_subdriver = {
 EOF
 
 cat <<EOF
-Done. 
+Done.
 
 Do not forget to:
-* add #include "${HFILE}" to usbhid-ups.c, 
+* add #include "${HFILE}" to usbhid-ups.c,
 * add &${LDRIVER}_subdriver to usbhid-ups.c:subdriver_list,
-* add ${LDRIVER}-hid.o to USBHID_UPS_SUBDRIVERS in drivers/Makefile.am
+* add ${LDRIVER}-hid.c to USBHID_UPS_SUBDRIVERS in drivers/Makefile.am
+* add ${LDRIVER}-hid.h to dist_noinst_HEADERS in drivers/Makefile.am
 * "autoreconf" from the top level directory
-* "make depend" in drivers/
 EOF
-
-
