@@ -66,9 +66,7 @@ void net_fsd(ctype_t *client, int numarg, const char **arg)
 	}
 
 	/* make sure this user is allowed to do FSD */	
-	if (!user_checkaction(&client->sock, client->username,
-		client->password, "FSD")) {
-
+	if (!user_checkaction(client->username, client->password, "FSD")) {
 		send_err(client, NUT_ERR_ACCESS_DENIED);
 		return;
 	}
